/*
** program to flash with mac address and factory default settings
** 
*/
/*----- Types and Constants Definition ----------------------*/
#define _REG(x) (*(volatile unsigned long *)(x))
#define _REG16(x) (*(volatile unsigned short *)(x))

#define MDU_CSR      0x80006000
#define MDU_CFGR0    0x80006004
#define MDU_TIMR00   0x80006008    
#define MDU_TIMR01   0x8000600C
#define MDU_CFGR1    0x80006010
#define MDU_TIMR10   0x80006014
#define MDU_TIMR11   0x80006018
#define MDU_CFGR2    0x8000601C
#define MDU_TIMR20   0x80006020
#define MDU_TIMR21   0x80006024
#define MDU_CFGR3    0x80006028
#define MDU_TIMR30   0x8000602C
#define MDU_TIMR31   0x80006030
#define MDU_CFGR4    0x80006034
#define MDU_TIMR40   0x80006038
#define MDU_TIMR41   0x8000603C
#define MDU_CFGR5    0x80006040
#define MDU_TIMR50   0x80006044
#define MDU_TIMR51   0x80006048

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <rt_misc.h>
#include "board.h"

void ConfigMDU(int);
void program_image(int what);
int macAddrProcess(char *src, char *dst);
int stringtoNum(char *src);

int total_size;
int errors = 0;

struct _board_map {
    char *prog;
    int what;
    int board;
} board_map[]={
    {"flash21.axf", ALL, AP21_BOARD}, 
    {"flash21_boot.axf", BOOT, AP21_BOARD}, 
    {"flash21_no_boot.axf", NO_BOOT, AP21_BOARD}, 
    {"flash21_no_boot2.axf", NO_BOOT2, AP21_BOARD}, 
    {"flash21_boot_factory.axf", BOOT_FACTORY, AP21_BOARD}, 
    {"flash21_no_boot_factory.axf", NO_BOOT_FACTORY, AP21_BOARD}, 
    {"flash21m.axf", ALL, AP21M_BOARD}, 
    {"flash21m_boot.axf", BOOT, AP21M_BOARD}, 
    {"flash21m_no_boot.axf", NO_BOOT, AP21M_BOARD}, 
    {"flash21m_no_boot2.axf", NO_BOOT2, AP21M_BOARD}, 
    {"flash21m_boot_factory.axf", BOOT_FACTORY, AP21M_BOARD}, 
    {"flash21m_no_boot_factory.axf", NO_BOOT_FACTORY, AP21M_BOARD}, 
    {"flash22.axf", ALL, AP22_BOARD}, 
    {"flash22_boot.axf", BOOT, AP22_BOARD}, 
    {"flash22_no_boot.axf", NO_BOOT, AP22_BOARD}, 
    {"flash22_no_boot2.axf", NO_BOOT2, AP22_BOARD}, 
    {"flash22_boot_factory.axf", BOOT_FACTORY, AP22_BOARD}, 
    {"flash22_no_boot_factory.axf", NO_BOOT_FACTORY, AP22_BOARD}, 
    {"flash22N.axf", ALL, AP22N_BOARD}, 
    {"flash22N_boot.axf", BOOT, AP22N_BOARD}, 
    {"flash22N_no_boot.axf", NO_BOOT, AP22N_BOARD}, 
    {"flash22N_no_boot2.axf", NO_BOOT2, AP22N_BOARD}, 
    {"flash22N_boot_factory.axf", BOOT_FACTORY, AP22N_BOARD}, 
    {"flash22N_no_boot_factory.axf", NO_BOOT_FACTORY, AP22N_BOARD}, 
    {"flash22C.axf", ALL, AP22C_BOARD}, 
    {"flash22C_boot.axf", BOOT, AP22C_BOARD}, 
    {"flash22C_no_boot.axf", NO_BOOT, AP22C_BOARD}, 
    {"flash22C_no_boot2.axf", NO_BOOT2, AP22C_BOARD}, 
    {"flash22C_boot_factory.axf", BOOT_FACTORY, AP22C_BOARD}, 
    {"flash22C_no_boot_factory.axf", NO_BOOT_FACTORY, AP22C_BOARD}, 
    {"flash_eval.axf", ALL, EVAL_BOARD}, 
    {"flash_eval_boot.axf", BOOT, EVAL_BOARD}, 
    {"flash_eval_no_boot.axf", NO_BOOT, EVAL_BOARD}, 
    {"flash_eval_no_boot2.axf", NO_BOOT2, EVAL_BOARD}, 
    {"flash_eval_boot_factory.axf", BOOT_FACTORY, EVAL_BOARD}, 
    {"flash_eval_no_boot_factory.axf", NO_BOOT_FACTORY, EVAL_BOARD}, 
    {"flash22N2.axf", ALL, AP22N2_BOARD}, 
    {"flash22N2_boot.axf", BOOT, AP22N2_BOARD}, 
    {"flash22N2_no_boot.axf", NO_BOOT, AP22N2_BOARD}, 
    {"flash22N2_no_boot2.axf", NO_BOOT2, AP22N2_BOARD}, 
    {"flash22N2_boot_factory.axf", BOOT_FACTORY, AP22N2_BOARD}, 
    {"flash22N2_no_boot_factory.axf", NO_BOOT_FACTORY, AP22N2_BOARD}, 
    {"flash31_no_boot.axf", NO_BOOT, CONDOR_TEST_BOARD}, 
    {"flash31_no_boot2.axf", NO_BOOT2, CONDOR_TEST_BOARD}, 
    {"flash31_boot_factory.axf", BOOT_FACTORY, CONDOR_TEST_BOARD}, 
    {"flash31_no_boot_factory.axf", NO_BOOT_FACTORY, CONDOR_TEST_BOARD}, 
    {"flash3x_no_boot.axf", NO_BOOT, CONDOR_BOARD}, 
    {"flash3x_no_boot2.axf", NO_BOOT2, CONDOR_BOARD}, 
    {"flash3x_boot_factory.axf", BOOT_FACTORY, CONDOR_BOARD}, 
    {"flash3x_no_boot_factory.axf", NO_BOOT_FACTORY, CONDOR_BOARD}, 
    {0, 0, 0}
};
char *progname;
int 
main(int argc, char *argv[])
{
   FILE *fp;
   int ch;
   int addr;
   unsigned short val;
   unsigned short *sptr = (unsigned short *)0xd0000000;
   char *p;
   char buffer[200];
   int i;
   progname = argv[0];

   memset((char *)0xd0000000, 0xff, 1*1024*1024);

   p = strrchr(progname, '\\');
   if(p) {
       strcpy(buffer, p+1);
   } else {
       strcpy(buffer, progname);
   }
   for(i=0;i < sizeof(board_map) / sizeof(struct _board_map);i++) {
       if(!strcmp(buffer, board_map[i].prog)) {
       break;
       }
   }
   if(i == (sizeof(board_map) / sizeof(struct _board_map))) {
       fprintf(stderr, "Cannot find this program/flash %s\n", buffer);
       exit(1);
   }

   //
   // Config MDU
   //

   fprintf(stderr, "File to flash: ");
   gets(buffer);
   fp = fopen(buffer, "rb");

   if(!fp) {
     printf("Cannot open %s for reading\n", buffer);
     exit(1);
   }

   printf("Config MDU for %s\n", board_map[i].prog);
   ConfigMDU(board_map[i].board);

   // AMD FLASH test
   // Erase sector
   if( board_map[i].what == BOOT_FACTORY)
   {
        fprintf(stderr, "Default factory setting read\n\n");
        SetFactoryConfig((void *)0xd0000000);
        //duplicated for user setting
        SetFactoryConfig((void *)0xd0000400);
        sptr += (64*1024/2);    //unsigned short pointer increase
   }

   printf("read image\n");
   
   addr =0;
   total_size = 0;
       while((ch=fgetc(fp)) != EOF) {
            if(!(addr & 0x7fe)) {
                printf("@addr %x\n", addr);
            }
            val = ch;
            if((ch=fgetc(fp)) == EOF)
                break;
            val |= ch<<8;  
            *sptr = val;
            if(*sptr != val) {
                printf("Mismatch in DRAM @%x got %x want %x\n", addr, *sptr, val);
                errors++;
                if(errors > ERROR_LIMIT) {
                    printf("Too many errors, bailing\n");
                    exit(1);
                }
            }
            addr += 2;
            total_size += 2;
            sptr++;
       }
   fclose(fp);
   if( board_map[i].what == BOOT_FACTORY)
   {
      char ptr[6];
      int ii = 0;
      char newCalData[12*8];
#if 1      
      printf("Board manufacturing data:");
      for( ii = 0; ii < 12*8; ii++) {
          if(!(ii%16)) {
              printf("\n");
          }
          printf("%02x ",((char *)0xffffff70)[ii]);
      }
#endif
      printf("\n\n");
      printf("Source of new manufacturing data?\n");
      printf("[1: Preserve current data / 2: Read new data from file / 3: Erase all] ");
      gets(buffer);

      if(buffer[0] == '1') {
          MfgSetCalibData((void *)0xd001ff70, (char *)0xffffff70);
      } else if(buffer[0] == '2') {

          printf("New manufacturing data file: ");
          gets(buffer);

          fp = fopen(buffer, "rb");
          if(!fp) {
              printf("Cannot open %s for reading\n", buffer);
              exit(1);
          }

          fread((void *)newCalData,1,12*8,fp);
          MfgSetCalibData((void *)0xd001ff70, newCalData);

      } else if(buffer[0] == '3') {
          /* Proceed will erase all */

      } else {
          printf("Invalid input\n");
          exit(1);
      }

      printf("Mac Address to Flash: \nformat xx-xx-xx-xx-xx-xx ");
      gets(buffer);
      if (macAddrProcess(buffer, ptr))
      {
        MfgSetMacAddr((void *)0xd001ff70, ptr);
        total_size = 2*64*1024;
      }
      printf("Domain Code to Flash: ");
      gets(buffer);
      MfgSetDomain((void *)0xd001ff70, stringtoNum(buffer));
   }


   program_image(board_map[i].what);
   printf("Done\n");
}
__value_in_regs struct __initial_stackheap __user_initial_stackheap(
        unsigned R0, unsigned SP, unsigned R2, unsigned SL)
{
    struct __initial_stackheap config;
    int aa, bb;

    aa=0x50078;
    bb=(0x80000|(4 << 1));
    __asm{
        ldr  r3, [&bb]
        MCR  p15, 0, r3, c9, c1, 0
        MRC  p15, 0, r3, c1, c0, 0
        ldr  r0, [&aa]
        MCR  p15, 0, r0, c1, c0, 0
        MRC  p15, 0, r3, c1, c0, 0
        ORR  r3, r3, #((1<<16) | (1<<18))
        MCR  p15, 0, r3, c1, c0, 0
    }
   
    config.heap_base = 0xc0008000;
    config.stack_base = 0xc000c000;

    return config;
}

int hextoNum(char a)
{
    if ( (a >= '0') && (a <='9') )
        return (a - '0');
    else if ( (a >= 'a') && (a <= 'f'))
        return (a - 87);
    else if ( (a >= 'A') && (a <= 'F'))
        return (a - 55);
    else
        return -1;
}
int digtoNum(char a)
{
    if ( (a >= '0') && (a <='9') )
        return (a - '0');
    else
        return -1;
}
int stringtoNum(char *src)
{
    int ret = 0;
    int hex = 0;
    while(*src)
    {
        if( hex || (*src++ == '0' && ((*src++ == 'x') || (*src == 'X'))))
        {
            hex = 1;
            ret = ret*16 + hextoNum(*src++);
        }
        else
        {
            ret = ret*10 + digtoNum(*src++);
        }
    }
    return ret;
}
int macAddrProcess(char *src, char *dst)
{
    int ch;
    int ret = 0;
    int i,j;
    while (*src)
    {
        ret++;
        i = hextoNum(*src);
        j = hextoNum(*(++src));
        ch = i*16 + j;
        *dst++ = ch; 
        src++;
        if(*src == '-')
            src++;
    }
    return ret;
}

