/******************** (c) Marvell Semiconductor, Inc., 2001 *******************
* 
* $Header: E:\\APCVSDEPOT/wlan/prod/ap/ap31/bridge/incl/ds.h,v 1.9 2003/12/04 01:16:07 jshen Exp $ 
* 
* Purpose: 
*    This file contains the function prototypes and definitions for the 
*    Distribution Service Module. 
* 
* Public Procedures: 
*    ds_Init       Initialzies Distribution Service Task and 
*                           related components 
*    ds_Start      Starts running the Distribution Service Task 
*    ds_SendEvent  Mechanism used to trigger an event indicating that 
*                           messages are present in the queue or receive
*                           buffers 
* 
* Notes: 
*    None. 
* 
*****************************************************************************/

#ifndef _DS_H_
#define _DS_H_

/*============================================================================= */
/*                               INCLUDE FILES */
/*============================================================================= */
#include "wltypes.h"
#include "osif.h"

/*============================================================================= */
/*                            PUBLIC DEFINITIONS */
/*============================================================================= */
/* None. */

/*============================================================================= */
/*                          PUBLIC TYPE DEFINITIONS */
/*============================================================================= */
#define ds_802_11_DATA_RCVD         (1<<0)
#define ds_802_3_DATA_RCVD          (1<<1)
#define ds_MGMT_NOTIFICATION_RCVD   (1<<2)
#define ds_LLC_DATA_RCVD            (1<<3)
#define ds_802_3_TX_DONE            (1<<4)
#define ds_802_11_TX_DONE           (1<<5)
#define ds_POLLING_TIMEOUT_RCVD     (1<<6)

#define NO_OF_PKTS 10

#define ETHER_HDR_SIZE 14
#define MAX_ETHER_PKT_SIZE 1600

#define IS_BROADCAST(macaddr) ((*(UINT32*)macaddr == 0xffff ) &&  \
                          *(UINT16 *)((UINT8*)macaddr+4) == 0xff)

#define IS_MULTICAST(macaddr)  ((*(UINT8*)macaddr & 0x01) == 0x01)

#define IS_GROUP(macaddr)  ((*(UINT8*)macaddr & 0x01) == 0x01)

#ifdef NTOHS
#undef NTOHS
#define NTOHS(x)  ((x >> 8) | (x << 8))
#endif

#define MACADDR_CMP(macaddr1, macaddr2) ((*(UINT32*)macaddr1 == *(UINT32*)macaddr2) && \
		(*(UINT16 *)((UINT8*)macaddr1+4) == (*(UINT16 *)((UINT8*)macaddr2+4))) ? 0 : 1)

#define MACADDR_CPY(macaddr1,macaddr2) { *(UINT16*)macaddr1 = *(UINT16*)macaddr2; \
                    *(UINT16 *)((UINT16*)macaddr1+1) = *(UINT16 *)((UINT16*)macaddr2+1); \
                    *(UINT16 *)((UINT16*)macaddr1+2) = *(UINT16 *)((UINT16*)macaddr2+2);}
typedef enum {
    ASSOCIATE_NOTIFICATION,
    DISASSOCIATE_NOTIFICATION
} ds_mgmt_notify_msg_e;

typedef struct
{
    ds_mgmt_notify_msg_e MsgType;
    IEEEtypes_MacAddr_t StnAddr;
}
ds_mgmt_notify_msg_t;

typedef struct
{
    UINT32 Key;
    UINT32 totalLen;
    UINT8 data[1600];
}
dsQ_LlcData_t;
typedef struct
{
    IEEEtypes_8023_Frame_t DataFrame;
}
PACK_END rx8023_DataFrame_t;

typedef struct llc_snap_hdr
{
    UINT8 Dsap;
    UINT8 Ssap;
    UINT8 Control;
    UINT8 Org[3];
    UINT16 Type;
}
llc_snap_hdr_t;

extern IEEEtypes_MacAddr_t bcast;

/*============================================================================= */
/*                                PUBLIC DATA */
/*============================================================================= */
/* None. */

/*============================================================================= */
/*                    PUBLIC PROCEDURES (ANSI Prototypes) */
/*============================================================================= */

/******************************************************************************
 * 
 * Name: ds_Init 
 * 
 * Description: 
 *   This routine is called to initialize the the Distribution Service Task 
 *   and related components. 
 * 
 * Conditions For Use: 
 *   None. 
 * 
 * Arguments: 
 *   None. 
 * 
 * Return Value: 
 *   Status indicating success or failure 
 * 
 * Notes: 
 *   None. 
 * 
 *****************************************************************************/
extern WL_STATUS ds_Init( void );


/******************************************************************************
 * 
 * Name: ds_SendEvent 
 * 
 * Description: 
 *   This routine provides the mechanism to trigger an event indicating to 
 *   the Distribution Service Task that a message has been written to one 
 *   of its queues or received data is available for processing. 
 * 
 * Conditions For Use: 
 *   The task and its associated queues has been initialized by calling 
 *   ds_Init(). 
 * 
 * Arguments: 
 *   Arg1 (i  ): Event - value indicating where data or message is available 
 * 
 * Return Value: 
 *   Status indicating success or failure 
 * 
 * Notes: 
 *   None. 
 * 
 *****************************************************************************/
extern WL_STATUS ds_SendEvent( UINT16 Event );

#endif /* _DS_H_ */
