/*
*                Copyright 2004, Marvell Semiconductor, Inc.
* This code contains confidential information of Marvell Semiconductor, Inc.
* No rights are granted herein under any patent, mask work right or copyright
* of Marvell or any third party.
* Marvell reserves the right at its sole discretion to request that this code
* be immediately returned to Marvell. This code is provided "as is".
* Marvell makes no warranties, express, implied or otherwise, regarding its
* accuracy, completeness or performance.
*/

/*!
 * \file    dsport.h
 * \brief   Public prototypes for distribution service port used for Live AP.
 */

/*=============================================================================
 *                                PUBLIC DEFINITIONS                                  
 *=============================================================================
 */
 /******************************************************************************
 *
 * Name: dsPort_Init
 *
 * Description:
 *   This routine is to set the dsport to use the Ethernet port.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   status
 *
 * Notes:
 *   None.
 *
 * PDL:
 *    
 * END PDL
 *
 ****************************************************************************/
#ifdef AP_PCI
extern void dsPort_Init(void);
#else
#define dsPort_Init() (void *)0
#endif
/******************************************************************************
 *
 * Name: dsPort_SetEth
 *
 * Description:
 *   This routine is to set the dsport to use the Ethernet port.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   status
 *
 * Notes:
 *   None.
 *
 * PDL:
 *    
 * END PDL
 *
 ****************************************************************************/
extern UINT32 dsPort_SetEth(int);

/******************************************************************************
 *
 * Name: dsPort_SetPci
 *
 * Description:
 *   This routine is to set the dsport to use the PCI port.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   status
 *
 * Notes:
 *   None.
 *
 * PDL:
 *    
 * END PDL
 *
 ****************************************************************************/
extern UINT32 dsPort_SetPci(void);

/******************************************************************************
 *
 * Name: dsPort_Write
 *
 * Description:
 *   This routine writes a buffer to the dsport.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   status
 *
 * Notes:
 *   None.
 *
 * PDL:
 *    
 * END PDL
 *
 ****************************************************************************/
#ifdef AP_PCI
extern UINT32 dsPort_Write( apio_bufdescr_t *pBufDescr);
#else
#define dsPort_Write apio_etherWrite
#endif
/******************************************************************************
 *
 * Name: dsPort_Read
 *
 * Description:
 *   This routine reads a buffer from the dsport.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   apio_bufdescr_t *pBufDescr
 *
 * Return Value:
 *   status
 *
 * Notes:
 *   None.
 *
 * PDL:
 *    
 * END PDL
 *
 ****************************************************************************/
#ifdef AP_PCI
extern UINT32 dsPort_Read(apio_bufdescr_t *pBufDescr);
#else
#define dsPort_Read apio_etherRead
#endif
/******************************************************************************
 *
 * Name: dsPort_PowerSense
 *
 * Description:
 *   This routine reads the GPIO power sense bit..
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   nonzero - power active
 *   0       - power inactive
 * Notes:
 *   None.
 *
 * PDL:
 *    
 * END PDL
 *
 ****************************************************************************/
extern UINT32 dsPort_PowerSense(void);

/******************************************************************************
 *
 * Name: dsPort_ReadEthernetControl
 *
 * Description:
 *   This routine reads the GPIO Ethernet control bit..
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   nonzero - ethernet active
 *   0       - ethernet inactive
 * Notes:
 *   None.
 *
 * PDL:
 *    
 * END PDL
 *
 ****************************************************************************/
extern UINT32 dsPort_ReadEthernetControl(void);