/******************** (c) Marvell Semiconductor, Inc., 2001 *******************
*
* $Header: E:\\APCVSDEPOT/wlan/prod/ap/ap31/bridge/incl/host.h,v 1.2 2003/09/22 21:48:53 jployer Exp $
*
* Purpose:
*    This file contains definitions relating to commands sent from the
*    host and the responses that are returned to the host.
*
* Public Procedures:
*    None.
*
* Notes:
*    None.
*
*****************************************************************************/

#ifndef _HOST_H_
#define _HOST_H_

#include "wltypes.h"

#define AP_API_RESP_RCVD            (1 << 0)

/*
ARMulator protocol packet type
*/
#define host_TYPE_NONE                0x0000
#define host_TYPE_PPA_DNLD            0x0001
#define host_TYPE_PPA_UPLD            0x0002
#define host_TYPE_WCB_DNLD            0x0003
#define host_TYPE_WCB_UPLD            0x0004
#define host_TYPE_DATA_DNLD           0x0005
#define host_TYPE_DATA_UPLD           0x0006
#define host_TYPE_CMD_DNLD            0x0007
#define host_TYPE_CMD_UPLD            0x0008
#define host_TYPE_RXPD_DNLD           0x0009
#define host_TYPE_RXPD_UPLD           0x000a
#define host_TYPE_REG_WRITE           0x000b
#define host_TYPE_REG_READ            0x000c
#define host_TYPE_INT_DNLD            0x000d
#define host_TYPE_INT_UPLD            0x000e


/*
16 bit message codes
*/
typedef enum
{
    host_MSG_NONE = 0x0000,
    host_CMD_CODE_DNLD = 0x0001,
    host_CMD_OP_PARAM_DNLD = 0x0002,
    host_CMD_GET_HW_SPEC = 0x0003,
    host_CMD_EEPROM_UPDATE = 0x0004,
    host_CMD_802_11_RESET = 0x0005,
    host_CMD_802_11_SCAN = 0x0006,
    /*   host_CMD_802_11_QUERY_AP       = 0x0007, */
    /*   host_CMD_802_11_QUERY_IBSS_STA = 0x0008, */
    host_CMD_802_11_QUERY_TRAFFIC = 0x0009,
    host_CMD_802_11_QUERY_STATUS = 0x000a,
    host_CMD_802_11_GET_LOG = 0x000b,
    /*   host_CMD_MAC_TX_ENABLE         = 0x000c, */
    /*   host_CMD_MAC_RX_ENABLE         = 0x000d, */
    /*   host_CMD_MAC_LOOP_BACK_ENABLE  = 0x000e, */
    /*   host_CMD_MAC_INT_ENABLE        = 0x000f, */
    host_CMD_MAC_MULTICAST_ADR = 0x0010,
    host_CMD_802_11_AUTHENTICATE = 0x0011,
    host_CMD_802_11_ASSOCIATE = 0x0012,
    host_CMD_802_11_SET_WEP = 0x0013,
    host_CMD_802_11_GET_STAT = 0x0014,
    host_CMD_802_3_GET_STAT = 0x0015,
    host_CMD_802_11_SNMP_MIB = 0x0016,
    host_CMD_MAC_REG_MAP = 0x0017,
    host_CMD_BBP_REG_MAP = 0x0018,
    host_CMD_RF_REG_MAP = 0x0023,
    host_CMD_MAC_REG_ACCESS = 0x0019,
    host_CMD_BBP_REG_ACCESS = 0x001a,
    host_CMD_RF_REG_ACCESS = 0x001b,
    host_CMD_802_11_RADIO_CONTROL = 0x001c,
    host_CMD_802_11_RF_CHANNEL = 0x001d,
    host_CMD_802_11_RF_TX_POWER = 0x001e,
    host_CMD_802_11_RSSI = 0x001f,
    host_CMD_802_11_RF_ANTENNA = 0x0020,
    host_CMD_802_11_PS_MODE = 0x0021,
    host_CMD_802_11_DATA_RATE = 0x0022,
    host_CMD_802_11_DEAUTHENTICATE = 0x0024,
    host_CMD_802_11_REASSOCIATE = 0x0025,
    host_CMD_802_11_DISASSOCIATE = 0x0026,
    host_CMD_MAC_CONTROL = 0x0028,
    host_CMD_802_11_QUERY_SCAN_RESULT = 0x0029,
    host_RET_GET_HW_SPEC = 0x8003,
    host_RET_EEPROM_UPDATE = 0x8004,
    host_RET_802_11_RESET = 0x8005,
    host_RET_802_11_SCAN = 0x8006,
    /*   host_RET_802_11_AP_INFO        = 0x8007, */
    /*   host_RET_802_11_QUERY_IBSS_STA = 0x8008, */
    host_RET_802_11_QUERY_TRAFFIC = 0x8009,
    host_RET_802_11_STATUS_INFO = 0x800a,
    host_RET_802_11_GET_LOG = 0x800b,
    /*   host_RET_MAC_TX_ENABLE         = 0x800c, */
    /*   host_RET_MAC_RX_ENABLE         = 0x800d, */
    /*   host_RET_MAC_LOOP_BACK_ENABLE  = 0x800e, */
    /*   host_RET_MAC_INT_ENABLE        = 0x800f, */
    host_RET_MAC_MULTICAST_ADR = 0x8010,
    host_RET_802_11_AUTHENTICATE = 0x8011,
    host_RET_802_11_ASSOCIATE = 0x8012,
    host_RET_802_11_SET_WEP = 0x8013,
    host_RET_802_11_STAT = 0x8014,
    host_RET_802_3_STAT = 0x8015,
    host_RET_802_11_SNMP_MIB = 0x8016,
    host_RET_MAC_REG_MAP = 0x8017,
    host_RET_BBP_REG_MAP = 0x8018,
    host_RET_RF_REG_MAP = 0x8023,
    host_RET_MAC_REG_ACCESS = 0x8019,
    host_RET_BBP_REG_ACCESS = 0x801a,
    host_RET_RF_REG_ACCESS = 0x801b,
    host_RET_802_11_RADIO_CONTROL = 0x801c,
    host_RET_802_11_RF_CHANNEL = 0x801d,
    host_RET_802_11_RF_TX_POWER = 0x801e,
    host_RET_802_11_RSSI = 0x801f,
    host_RET_802_11_RF_ANTENNA = 0x8020,
    host_RET_802_11_PS_MODE = 0x8021,
    host_RET_802_11_DATA_RATE = 0x8022,
    host_RET_802_11_DEAUTHENTICATE = 0x8024,
    host_RET_802_11_REASSOCIATE = 0x8025,
    host_RET_802_11_DISASSOCIATE = 0x8026,
    host_RET_MAC_CONTROL = 0x8028,
    host_RET_802_11_QUERY_SCAN_RESULT = 0x8029
} host_Msg_e;


/*
Define result code for each command
*/
typedef enum
{
    host_RESULT_OK = 0x0000,   /*OK*/
    host_RESULT_ERROR = 0x0001,   /*Genenral error*/
    host_RESULT_NOT_SUPPORT = 0x0002,   /*Command is not valid*/
    host_RESULT_PENDING = 0x0003,   /*Command is pending (will be processed)*/
    host_RESULT_BUSY = 0x0004,   /*System is busy (command ignored)*/
    host_RESULT_PARTIAL_DATA = 0x0005  /*Data buffer is not big enough*/
} host_Result_e;


/*
Definition of action or option for each command
*/

/*Define general purpose action */
#define host_ACT_READ     0x0000
#define host_ACT_WRITE    0x0001

#define host_ACT_GET      0x0000
#define host_ACT_SET      0x0001

#define host_ACT_OFF      0x0000
#define host_ACT_ON       0x0001

/*Define action or option for CMD_GET_HW_SPEC*/
/*Define action or option for CMD_EEPROM_UPDATE*/

/*Define action or option for CMD_802_11_AUTHENTICATE*/
#define host_ACT_AUTHENTICATE     0x0001
#define host_ACT_DEAUTHENTICATE   0x0002

/*Define action or option for CMD_802_11_ASSOCIATE*/
#define host_ACT_ASSOCIATE        0x0001
#define host_ACT_DISASSOCIATE     0x0002
#define host_ACT_REASSOCIATE      0x0003

/*Define action or option for CMD_802_11_SET_WEP*/
/*#define host_ACT_ENABLE           0x0001 */
/*#define host_ACT_DISABLE          0x0000 */
#define host_ACT_ADD              0x0002
#define host_ACT_REMOVE           0x0004
#define Host_ACT_USE_DEFAULT      0x0008

#define host_TYPE_WEP_40_BIT      0x0001   /*40 bit*/
#define host_TYPE_WEP_104_BIT     0x0002   /*128 bit*/
#define host_TYPE_WEP_TX_KEY      0x0004   /*TX WEP*/

#define host_NUM_OF_WEP_KEYS      4

#define host_WEP_KEY_INDEX_MASK   0x3fffffff

/*Define action or option for CMD_802_11_RESET*/
#define host_ACT_NOT_REVERT_MIB   0x0001
#define host_ACT_REVERT_MIB       0x0002
#define host_ACT_HALT             0x0003

/*Define action or option for CMD_802_11_SCAN*/
#define host_TYPE_BSS             0x0001
#define host_TYPE_IBSS            0x0002

/*Define action or option for CMD_802_11_QUERY_AP*/
/*Define action or option for CMD_802_11_QUERY_IBSS_STA*/
/*Define action or option for CMD_802_11_QUERY_TRAFFIC*/
/*Define action or option for CMD_802_11_QUERY_STATUS*/
#define host_STATUS_FW_INIT           0x0000
#define host_STATUS_FW_IDLE           0x0001
#define host_STATUS_FW_WORKING        0x0002
#define host_STATUS_FW_ERROR          0x0003
#define host_STATUS_FW_POWER_SAVE     0x0004

#define host_STATUS_MAC_RX_ON         0x0001
#define host_STATUS_MAC_TX_ON         0x0002
#define host_STATUS_MAC_LOOP_BACK_ON  0x0004
#define host_STATUS_MAC_WEP_ENABLE    0x0008
#define host_STATUS_MAC_INT_ENABLE    0x0010

/*Define action or option for CMD_802_11_GET_LOG*/
/*Define action or option for CMD_MAC_TX_ENABLE*/
/*Define action or option for CMD_MAC_RX_ENABLE*/
/*Define action or option for CMD_MAC_LOOP_BACK_ENABLE*/
/*Define action or option for CMD_MAC_INT_ENABLE*/

/*  Define action or option for CMD_MAC_CONTROL */
#define host_ACT_MAC_RX_ON                   0x0001
#define host_ACT_MAC_TX_ON                   0x0002
#define host_ACT_MAC_LOOPBACK_ON             0x0004
#define host_ACT_MAC_WEP_ENABLE              0x0008
#define host_ACT_MAC_INT_ENABLE              0x0010
#define host_ACT_MAC_MULTICAST_ENABLE        0x0020
#define host_ACT_MAC_BROADCAST_ENABLE        0x0040
#define host_ACT_MAC_PROMISCUOUS_ENABLE      0x0080
#define host_ACT_MAC_ALL_MULTICAST_ENABLE    0x0100

/*Define action or option or constant for CMD_MAC_MULTICAST_ADR*/
#define host_SIZE_MAC_ADR        6
#define host_MAX_MCAST_ADRS      32

/*Define action or option for CMD_802_11_GET_STAT*/
/*Define action or option for CMD_802_3_GET_STAT*/

/*Define action or option for CMD_802_11_SNMP_MIB*/
#define host_TYPE_MIB_FLD_BOOLEAN      0x0001  /* BOOLEAN */
#define host_TYPE_MIB_FLD_INTEGER      0x0002  /* 32 byte unsigned integer */
#define host_TYPE_MIB_FLD_COUNTER      0x0003  /* Counter */
#define host_TYPE_MIB_FLD_OCT_STR      0x0004  /* Octet string */
#define host_TYPE_MIB_FLD_DISPLAY_STR  0x0005  /* String */
#define host_TYPE_MIB_FLD_MAC_ADR      0x0006  /* MAC address */
#define host_TYPE_MIB_FLD_IP_ADR       0x0007  /* IP address */
#define host_TYPE_MIB_FLD_WEP          0x0008  /* WEP */

/*Define action or option for CMD_MAC_REG_MAP*/
/*Define action or option for CMD_BBP_REG_MAP*/
/*Define action or option for CMD_RF_REG_MAP*/
/*Define action or option for CMD_MAC_REG_ACCESS*/
/*Define action or option for CMD_BBP_REG_ACCESS*/
/*Define action or option for CMD_RF_REG_ACCESS*/

/*Define action or option for CMD_802_11_RADIO_CONTROL*/
#define host_TYPE_LONG_PREAMBLE    0x0002
#define host_TYPE_SHORT_PREAMBLE   0x0003

/*Define action or option for CMD_802_11_RF_CHANNEL*/
#define host_TYPE_802_11A          0x0001
#define host_TYPE_802_11B          0x0002

/*Define action or option for CMD_802_11_RSSI*/
/*Define action or option for CMD_802_11_RF_TX_POWER*/

/*Define action or option for CMD_802_11_RF_ANTENNA*/
/*Define action or option for CMD_802_11_DATA_RATE*/
#define host_ACT_SET_RX             0x0001
#define host_ACT_SET_TX             0x0002
#define host_ACT_SET_BOTH           0x0003
#define host_ACT_GET_RX             0x0004
#define host_ACT_GET_TX             0x0008
#define host_ACT_GET_BOTH           0x000c

#define host_TYPE_ANTENNA_DIVERSITY   0xffff

/*Define action or option for CMD_802_11_PS_MODE*/
#define host_TYPE_CAM               0x0000
#define host_TYPE_MAX_PSP           0x0001
#define host_TYPE_FAST_PSP          0x0002



/*
Definition of data structure for each command
*/

typedef struct host_MsgHdr_t
{
    UINT16 Msg;          /*Command number - host_Msg_e*/
    UINT16 Size;         /*Size of the data structure*/
    UINT16 SeqNum;       /*Command sequence number*/
    UINT16 Result;       /*Result code - host_Result_e*/
}
PACK_END host_MsgHdr_t;

/*Define data structure for CMD_CODE_DNLD*/
typedef struct host_codeDnld_t
{
    UINT8 Code[4096];
}
PACK_END host_codeDnld_t, *PDS_CODE_DNLD;

/*Define data structure for CMD_OP_PARAM_DNLD*/
typedef struct host_opParamDnld_t
{
    UINT8 OpParam[4096];
}
PACK_END host_opParamDnld_t, *PDS_OP_PARAM_DNLD;

/*Define data structure for CMD_GET_HW_SPEC*/
typedef struct host_getHwSpec_t
{
    UINT16 Veriosn;       /*HW/FW version */
    UINT16 NumOfWCB;         /*Max. number of WCB FW can handle*/
    UINT16 NumOfMCastAdr;    /*Max. number of Multicast address FW can handle*/
    UINT8 PermanentAddr[6]; /* MAC address */
}
PACK_END host_getHwSpec_t, *PDS_GET_HW_SPEC;

/*Define data structure for CMD_EEPROM_UPDATE*/
typedef struct host_eepromUpdate_T
{
    UINT16 Action;           /*Detailed action or option*/
}
PACK_END host_eepromUpdate_T, *PDS_EEPROM_UPDATE;

/*Define data structure for CMD_802_11_RESET*/
typedef struct host_802_11Reset_t
{
    UINT16 Action;           /* ACT_NOT_REVERT_MIB, ACT_REVERT_MIB or ACT_HALT*/
}
PACK_END host_802_11Reset_t, *PDS_802_11_RESET;

/*Define data structure for CMD_802_11_SCAN*/
typedef struct host_802_11Scan_t
{
    UINT16 AutoAssoc;
}
PACK_END host_802_11Scan_t, *PDS_802_11_SCAN;

/*Define return data structure for CMD_802_11_SCAN*/
typedef struct host_Ret_802_11Scan_t
{
    UINT16 BufSize;
}
PACK_END host_Ret_802_11Scan_t, *PDS_RET_802_11_SCAN;

/*Define data structure for CMD_802_11_QUERY_AP*/
typedef struct host_802_11QueryAp_t
{
    /*TBD*/
}
PACK_END host_802_11QueryAp_t, *PDS_802_11_QUERY_AP;

/*Define data structure for CMD_802_11_QUERY_IBSS_STA*/
typedef struct host_802_11QueryIbssSta_t
{
    /*TBD*/
}
PACK_END host_802_11QueryIbssSta_t, *PDS_802_11_QUERY_IBSS_STA;

/*Define data structure for CMD_802_11_QUERY_TRAFFIC*/
typedef struct host_802_11QueryTraffic_t
{
    /*TBD*/
    UINT32 Traffic;       /* Traffic in bps*/
}
PACK_END host_802_11QueryTraffic_t, *PDS_802_11_QUERY_TRAFFIC;

/*Define data structure for CMD_802_11_QUERY_STATUS*/
typedef struct host_802_11QueryStatus_t
{
    UINT16 FWStatus;
    UINT16 MACStatus;
    UINT16 RFStatus;
    UINT16 CurentChannel;     /*1..99*/
    UINT8 APMACAdr[6];       /* Associated AP MAC address*/
    UINT16 Reserved;
    UINT32 MaxLinkSpeed;      /*Allowable max.link speed in unit of 100bps*/
}
PACK_END host_802_11QueryStatus_t, *PDS_802_11_QUERY_STATUS;

/*Define data structure for CMD_802_11_GET_LOG*/
typedef struct host_802_11GetLog_t
{
    UINT8 Log[512];
}
PACK_END host_802_11GetLog_t, *PDS_802_11_GET_LOG;

/*Define data structure for CMD_MAC_TX_ENABLE*/
typedef struct host_macTxEnable_t
{
    UINT16 Action;            /*ACT_ON, ACT_OFF*/
    UINT16 Reserved;
}
PACK_END host_macTxEnable_t, *PDS_MAC_TX_ENABLE;

/*Define data structure for CMD_MAC_RX_ENABLE*/
typedef struct host_macRxEnable_t
{
    UINT16 Action;             /*ACT_ON, ACT_OFF*/
    UINT16 Reserved;
}
PACK_END host_macRxEnable_t, *PDS_MAC_RX_ENABLE;

/*Define data structure for CMD_MAC_LOOP_BACK_ENABLE*/
typedef struct host_macLoopBackEnable_t
{
    UINT16 Action;             /*ACT_ON, ACT_OFF*/
    UINT16 Reserved;
}
PACK_END host_macLoopBackEnable_t, *PDS_MAC_LOOP_BACK_ENABLE;

/*Define data structure for CMD_MAC_INT_ENABLE*/
typedef struct host_macIntEnable_t
{
    UINT16 Action;            /*ACT_ON, ACT_OFF*/
    UINT16 Reserved;
}
PACK_END host_macIntEnable_t, *PDS_MAC_INT_ENABLE;

/*Define data structure for CMD_MAC_MULTICAST_ADR*/
typedef struct host_macMulticastAdr_t
{
    UINT16 Action;
    UINT16 NumOfAdrs;
    UINT8 MACList[host_SIZE_MAC_ADR * host_MAX_MCAST_ADRS];
}
PACK_END host_macMulticastAdr_t, *PDS_MAC_MULTICAST_ADR;

/*Define data structure for CMD_802_11_AUTHENTICATE*/
typedef struct host_802_11Authenticate_t
{
    UINT8 MacAddr[host_SIZE_MAC_ADR];
    UINT8 AuthType;
    UINT16 TimeOut;
    UINT8 Reserved[2];
}
PACK_END host_802_11Authenticate_t, *PDS_802_11_AUTHENTICATE;

/*Define data structure for CMD_802_11_ASSOCIATE*/
typedef struct host_802_11Associate_t
{
    /*TBD*/
}
PACK_END host_802_11Associate_t, *PDS_802_11_ASSOCIATE;

/*Define data structure for CMD_802_11_SET_WEP*/
typedef struct host_802_11SetWep_t
{
    UINT16 Action;          /*ACT_ADD, ACT_REMOVE, ACT_ENABLE*/
    UINT16 TxKeyIndx;
    UINT8 K1WEPType;         /*40, 128bit or TXWEP*/
    UINT8 K2WEPType;         /*40, 128bit or TXWEP*/
    UINT8 K3WEPType;         /*40, 128bit or TXWEP*/
    UINT8 K4WEPType;         /*40, 128bit or TXWEP*/
    UINT8 WEP1[16];
    UINT8 WEP2[16];
    UINT8 WEP3[16];
    UINT8 WEP4[16];
}
PACK_END host_802_11SetWep_t, *PDS_802_11_SET_WEP;

/*Define data structure for CMD_802_3_GET_STAT*/
typedef struct host_802_3GetStat_t
    {}
PACK_END host_802_3GetStat_t, *PDS_802_3_GET_STAT;

/*Define data structure for CMD_802_11_GET_STAT*/
typedef struct host_802_11GetStat_t
{
    UINT32 TXFragmentCnt;
    UINT32 MCastTXFrameCnt;
    UINT32 FailedCnt;
    UINT32 RetryCnt;
    UINT32 MultipleRetryCnt;
    UINT32 RTSSuccessCnt;
    UINT32 RTSFailureCnt;
    UINT32 ACKFailureCnt;
    UINT32 FrameDuplicateCnt;
    UINT32 RXFragmentCnt;
    UINT32 MCastRXFrameCnt;
    UINT32 FCSErrorCnt;
    UINT32 BCastTXFrameCnt;
    UINT32 BCastRXFrameCnt;
    UINT32 TXBeacon;
    UINT32 RXBeacon;
    UINT32 WEPUndecryptable;
}
PACK_END host_802_11GetStat_t, *PDS_802_11_GET_STAT;

/*Define data structure for CMD_802_11_SNMP_MIB*/
typedef struct host_802_11SnmpMib_t
{
    UINT16 Action;
    UINT16 OID;
    UINT16 Size;
    UINT8 Value[128];
}
PACK_END host_802_11SnmpMib_t, *PDS_802_11_SNMP_MIB;

/*Define data structure for CMD_MAC_REG_MAP*/
typedef struct host_macRegMap_t
{
    /*TBD*/
    UINT8 RegMap[128];
}
PACK_END host_macRegMap_t, *PDS_MAC_REG_MAP;

/*Define data structure for CMD_BBP_REG_MAP*/
typedef struct host_bbpRegMap_t
{
    UINT8 RegMap[128];
}
PACK_END host_bbpRegMap_t, *PDS_BBP_REG_MAP;

/*Define data structure for CMD_RF_REG_MAP*/
typedef struct host_rfRegMap_t
{
    UINT8 RegMap[64];
}
PACK_END host_rfRegMap_t, *PDS_RF_REG_MAP;

/*Define data structure for CMD_MAC_REG_ACCESS*/
typedef struct host_macRegAccess_t
{
    UINT16 Action;
    /*TBD*/
    UINT16 Offset;        /*byte offset, aligned on DWORD boundary*/
    UINT32 Value;
}
PACK_END host_macRegAccess_t, *PDS_MAC_REG_ACCESS;

/*Define data structure for CMD_BBP_REG_ACCESS*/
typedef struct host_bbpRegAccess_t
{
    UINT16 Action;
    UINT16 Offset;        /*byte offset*/
    UINT8 Value;
    UINT8 Reserverd[3];
}
PACK_END host_bbpRegAccess_t, *PDS_BBP_REG_ACCESS;

/*Define data structure for CMD_RF_REG_ACCESS*/
typedef struct host_rfRegAccess_t
{
    UINT16 Action;
    UINT16 Offset;        /*byte offset*/
    UINT8 Value;
    UINT8 Reserverd[3];
}
PACK_END host_rfRegAccess_t, *PDS_RF_REG_ACCESS;

/*Define data structure for CMD_802_11_RADIO_CONTROL*/
typedef struct host_802_11RadioCtl_t
{
    UINT16 Action;        /*ON, OFF, long or short preamble*/
    UINT16 Reserverd;
}
PACK_END host_802_11RadioCtl_t, *PDS_802_11_RADIO_CONTROL;

/*Define data structure for CMD_802_11_RF_CHANNEL*/
typedef struct host_802_11RfChnl_t
{
    UINT16 Action;
    UINT16 CurentChannel;
    UINT16 RFType;            /*802.11a or 802.11b*/
    UINT16 Reserved;
    UINT8 ChannelList[32];
}
PACK_END host_802_11RfChnl_t, *PDS_802_11_RF_CHANNEL;

/*Define data structure for CMD_802_11_RSSI*/
typedef struct host_802_11Rssi_t
{
    UINT64 RSSI;              /*RSSI in dBm*/
}
PACK_END host_802_11Rssi_t, *PDS_802_11_RSSI;

/*Define data structure for CMD_802_11_RF_TX_POWER*/
typedef struct host_802_11RfTxPower_t
{
    UINT16 Action;
    UINT16 SupportLevel;      /* 1..8 */
    UINT16 CurrentLevel;      /* 1..8 */
    UINT16 Reserved;
    UINT16 PowerLevelList[8];
}
PACK_END host_802_11RfTxPower_t, *PDS__802_11_RF_TX_POWERDS;

/*Define data structure for CMD_802_11_RF_ANTENNA*/
typedef struct host_802_11RfAntenna_t
{
    UINT16 Action;
    UINT16 AntennaMode;       /*Number of antennas or 0xffff(diversity)*/
}
PACK_END host_802_11RfAntenna_t, *PDS_802_11_RF_ANTENNA;

/*Define data structure for CMD_802_11_PS_MODE*/
typedef struct host_802_11PsMode_t
{
    UINT16 Action;
    UINT16 PowerMode;         /*CAM, Max.PSP or Fast PSP*/
}
PACK_END host_802_11PsMode_t, *PDS_802_11_PS_MODE;

/*Define data structure for CMD_802_11_DATA_RATE*/
typedef struct host_802_11DataRate_t
{
    UINT16 Action;
    UINT16 Reserverd;
    UINT8 DataRate[8];       /*Supported data reate list*/
}
PACK_END host_802_11DataRate_t, *PDS_802_11_DATA_RATE;

/*Define data structure for CMD_MAC_CONTROL*/
typedef struct host_MacControl_t
{
    UINT16 Control;
    UINT16 Reserverd;
}
PACK_END host_MacControl_t, *PDS_MacControl_t;




/*===================================================================================*/
/*  The following sections are the commands for AP only.                             */
/* Most of the host commands above will also be used by AP                           */
/*===================================================================================*/
/* Commands */
#define APCMD_CMD_802_11_QUERY_STATUS    0x0101
#define APCMD_CMD_802_11_GET_AUTH_STA_LIST   0x0102
#define APCMD_CMD_802_11_GET_ASSOC_STA_LIST   0x0103
#define APCMD_CMD_802_11_GET_STA_INFO    0x0104
#define APCMD_CMD_802_11_SSID      0x0105
#define APCMD_CMD_802_11_PRIVACY_OPTION    0x0106
#define APCMD_CMD_802_11_AUTHENTICATION_MODE  0x0107
#define APCMD_CMD_802_11_FRAGMENTATION_THRESHOLD 0x0108
#define APCMD_CMD_802_11_RTS_THRESHOLD    0x0109
#define APCMD_CMD_802_11_DATA_RETRY_LIMIT   0x010A
#define APCMD_CMD_802_11_RTS_RETRY_LIMIT   0x010B
#define APCMD_CMD_802_11_TRANSMIT_RATES    0x010C
#define APCMD_CMD_802_11_SUPPORTED_RATES   0x010D
#define APCMD_CMD_802_11_RADIO_PREAMBLE    0x010E
#define APCMD_CMD_802_11_BEACON_PERIOD    0x010F
#define APCMD_CMD_802_11_DTIM_PERIOD    0x0110
#define APCMD_CMD_802_3_GET_STAT     0x0111
#define APCMD_CMD_802_3_LINK_SPEED     0x0112
#define APCMD_CMD_802_3_IP_ADDRESS     0x0113
#define APCMD_CMD_802_3_SUBNET_MASK     0x0114
#define APCMD_CMD_802_3_GATEWAY      0x0115
#define APCMD_CMD_802_3_DHCP      0x0116
#define APCMD_CMD_802_3_ICMP      0x0117
#define APCMD_CMD_802_11_LINK_TEST     0x0118
#define APCMD_CMD_UPDATE_FIRMWARE     0x0119
#define APCMD_CMD_DIAGNOSTICS      0x011A

/* Response from AP */
#define APCMD_RET_802_11_QUERY_STATUS    0x0101
#define APCMD_RET_802_11_GET_AUTH_STA_LIST   0x0102
#define APCMD_RET_802_11_GET_ASSOC_STA_LIST   0x0103
#define APCMD_RET_802_11_GET_STA_INFO    0x0104
#define APCMD_RET_802_11_SSID      0x0105
#define APCMD_RET_802_11_PRIVACY_OPTION    0x0106
#define APCMD_RET_802_11_AUTHENTICATION_MODE  0x0107
#define APCMD_RET_802_11_FRAGMENTATION_THRESHOLD 0x0108
#define APCMD_RET_802_11_RTS_THRESHOLD    0x0109
#define APCMD_RET_802_11_DATA_RETRY_LIMIT   0x010A
#define APCMD_RET_802_11_RTS_RETRY_LIMIT   0x010B
#define APCMD_RET_802_11_TRANSMIT_RATES    0x010C
#define APCMD_RET_802_11_SUPPORTED_RATES   0x010D
#define APCMD_RET_802_11_RADIO_PREAMBLE    0x010E
#define APCMD_RET_802_11_BEACON_PERIOD    0x010F
#define APCMD_RET_802_11_DTIM_PERIOD    0x0110
#define APCMD_RET_802_3_STATISTICS     0x0111
#define APCMD_RET_802_3_LINK_SPEED     0x0112
#define APCMD_RET_802_3_IP_ADDRESS     0x0113
#define APCMD_RET_802_3_SUBNET_MASK     0x0114
#define APCMD_RET_802_3_GATEWAY      0x0115
#define APCMD_RET_802_3_DHCP      0x0116
#define APCMD_RET_802_3_ICMP      0x0117
#define APCMD_RET_802_11_LINK_TEST     0x0118
#define APCMD_RET_UPDATE_FIRMWARE     0x0119
#define APCMD_RET_DIAGNOSTICS      0x011A


/* Reuse of Host Command */
#define APCMD_CMD_802_11_GET_LOG   HostCmd_CMD_802_11_GET_LOG
#define APCMD_CMD_802_11_DEAUTHENTICATE  HostCmd_CMD_802_11_DEAUTHENTICATE
#define APCMD_CMD_802_11_DISASSOCIATE  HostCmd_CMD_802_11_DISASSOCIATE
#define APCMD_CMD_802_11_SET_WEP   HostCmd_CMD_802_11_SET_WEP
#define APCMD_CMD_802_11_RF_CHANNEL   HostCmd_CMD_802_11_RF_CHANNEL
#define APCMD_CMD_802_11_RF_TX_POWER  HostCmd_CMD_802_11_RF_TX_POWER
#define APCMD_CMD_802_11_RF_ANTENNA   HostCmd_CMD_802_11_RF_ANTENNA

/* Reuse of Host response */
#define APCMD_RET_802_11_GET_LOG   HostCmd_RET_802_11_GET_LOG
#define APCMD_RET_802_11_DEAUTHENTICATE  HostCmd_RET_802_11_DEAUTHENTICATE
#define APCMD_RET_802_11_DISASSOCIATE  HostCmd_RET_802_11_DISASSOCIATE
#define APCMD_RET_802_11_SET_WEP   HostCmd_RET_802_11_SET_WEP
#define APCMd_RET_802_11_RF_CHANNEL   HostCmd_RET_802_11_RF_CHANNEL
#define APCMD_RET_802_11_RF_TX_POWER  HostCmd_RET_802_11_RF_TX_POWER
#define APCMD_RET_802_11_RF_ANTENNA   HostCmd_RET_802_11_RF_ANTENNA 
/*
 SSID Section
*/
#define APCMD_OPEN_SSID_ACCEPTED     0x0001
#define APCMD_OPEN_SSID_NOT_ACCEPTED    0x0000


/*
 WEP Encryption Section
*/
#define APCMD_WEP_ENABLED       0x0001
#define APCMD_WEP_DISABLED       0x0000

/*
 Privacy Option Section
*/
#define APCMD_ENCRYPTION_NOT_REQUIRED    0x0000
#define APCMD_ENCRYPTION_OPTIONAL     0x0001
#define APCMD_ENCRYPTION_REQUIRED     0x0002

/*
 Authentication Mode Section
*/
#define APCMD_AUTHENTICATION_OPEN_SYSTEM   0x0000
#define APCMD_AUTHENTICATION_SHARED_KEY    0x0001
#define APCMD_AUTHENTICATION_BOTH     0x0002

/*
 Radio Preamble Section
*/
#define APCMD_RADIO_PREAMBLE_SHORT     0x0000
#define APCMD_RADIO_PREAMBLE_LONG     0x0001
#define APCMD_RADIO_PREAMBLE_AUTO     0x0002

/*
 DHCP Section
*/
#define APCMD_DHCP_ENABLED       0x0001
#define APCMD_DHCP_DISABLED       0x0000

/*
 ICMP Section
*/
#define APCMD_ICMP_ENABLED       0x0001
#define APCMD_ICMP_DISABLED       0x0000


/*  Define data structure for APCmd_CMD_802_11_QUERY_STATUS */
typedef struct _APCmd_DS_802_11_QUERY_STATUS
{

    UINT16 FWStatus;
    UINT16 MACStatus;
    UINT16 RFStatus;
    UINT16 EthernetPortStatus;
    UINT16 CurentChannel;             /* 1..99 */
    UINT8 APName[32];
    UINT8 APMACAdr[6];               /*  AP MAC address */
    UINT16 NumOfAssociatedSTA;
    UINT32 MaxLinkSpeed;             /* Allowable max.link speed in unit of 1000 bps */
}
APCmd_DS_802_11_QUERY_STATUS, *PAPCmd_DS_802_11_QUERY_STATUS;

/*  Define data structure for
 APCmd_CMD_802_11_GET_AUTH_STA_LIST and APCmd_CMD_802_11_GET_ASSOC_STA_LIST*/
typedef struct _APCmd_CMD_802_11_STA_LIST
{
    UINT16 Index;
    UINT16 AID;
    UINT8 StationName[32];     /* can be NULL */
    UINT8 MACAdr[6];                 /*  MAC address of the station */
    struct _APCmd_CMD_802_11_STA_LIST *Next;     /* Points to next station in the list */
}
APCmd_CMD_802_11_STA_LIST, *PAPCmd_CMD_802_11_STA_LIST;

/*  Define data structure for APCmd_CMD_802_11_GET_AUTH_STAS */
typedef struct _APCmd_CMD_802_11_GET_AUTH_STA_LIST
{

    PAPCmd_CMD_802_11_STA_LIST pStationList;
}
APCmd_CMD_802_11_GET_AUTH_STA_LIST, *PAPCmd_CMD_802_11_GET_AUTH_STA_LIST;

/*  Define data structure for APCmd_CMD_802_11_GET_ASSOC_STAS */
typedef struct _APCmd_CMD_802_11_GET_ASSOC_STA_LIST
{

    PAPCmd_CMD_802_11_STA_LIST pStationList;
}
APCmd_CMD_802_11_GET_ASSOC_STA_LIST, *PAPCmd_CMD_802_11_GET_ASSOC_STA_LIST;

/*  Define data structure for APCmd_CMD_802_11_QUERY_STA_INFO */
typedef struct _APCmd_CMD_802_11_QUERY_STA_INFO
{

    UINT8 MACAdr[6];           /*  MAC address of the station to be queried*/
    UINT8 IPAddress[6];  /*  Info returned by AP, currently use the first 4 bytes */
    UINT16 PowerSaveMode;  /*  Info returned by AP */
    UINT16 WepStatus;   /*  Info returned by AP, see WEP Encryption section */
}
APCmd_CMD_802_11_QUERY_STA_INFO, *PAPCmd_CMD_802_11_QUERY_STA_INFO;

/*  Define data structure for APCmd_CMD_802_11_SSID */
typedef struct _APCmd_CMD_802_11_SSID
{

    UINT16 Action;    /* see general purpose section */
    UINT16 SsidLength;
    UINT8 Ssid[32];
    UINT16 AcceptOpenSsis;  /* 1: Accept, 0: Do not accept */
}
APCmd_CMD_802_11_SSID, *PAPCmd_CMD_802_11_SSID;

/*  Define data structure for APCmd_CMD_802_11_PRIVACY_OPTION */
typedef struct _APCmd_CMD_802_11_PRIVACY_OPTION
{

    UINT16 Action;    /* see general purpose section */
    UINT16 DataEncryption;  /* see privacy option section */
}
APCmd_CMD_802_11_PRIVACY_OPTION, *PAPCmd_CMD_802_11_PRIVACY_OPTION;

/*  Define data structure for APCmd_CMD_802_11_AUTHENTICATION_MODE */
typedef struct _APCmd_CMD_802_11_AUTHENTICATION_MODE
{

    UINT16 Action;    /* see general purpose section */
    UINT16 AuthMode;   /* see authentication mode section */
}
APCmd_CMD_802_11_AUTHENTICATION_MODE, *PAPCmd_CMD_802_11_AUTHENTICATION_MODE;

/*  Define data structure for APCmd_CMD_802_11_FRAGMENTATION_THRESHOLD */
typedef struct _APCmd_CMD_802_11_FRAGMENTATION_THRESHOLD
{

    UINT16 Action;     /* see general purpose section */
    UINT16 FragmentationThreshold;
}
APCmd_CMD_802_11_FRAGMENTATION_THRESHOLD, *PAPCmd_CMD_802_11_FRAGMENTATION_THRESHOLD;

/*  Define data structure for APCmd_CMD_802_11_RTS_THRESHOLD */
typedef struct _APCmd_CMD_802_11_RTS_THRESHOLD
{

    UINT16 Action;     /* see general purpose section */
    UINT16 RtsThreshold;
}
APCmd_CMD_802_11_RTS_THRESHOLD, *PAPCmd_CMD_802_11_RTS_THRESHOLD;

/*  Define data structure for APCmd_CMD_802_11_DATA_RETRY_LIMIT */
typedef struct _APCmd_CMD_802_11_DATA_RETRY_LIMIT
{

    UINT16 Action;     /* see general purpose section */
    UINT16 DataRetryLimit;
}
APCmd_CMD_802_11_DATA_RETRY_LIMIT, *PAPCmd_CMD_802_11_DATA_RETRY_LIMIT;

/*  Define data structure for APCmd_CMD_802_11_RTS_RETRY_LIMIT */
typedef struct _APCmd_CMD_802_11_RTS_RETRY_LIMIT
{

    UINT16 Action;     /* see general purpose section */
    UINT16 RtsRetryLimit;
}
APCmd_CMD_802_11_RTS_RETRY_LIMIT, *PAPCmd_CMD_802_11_RTS_RETRY_LIMIT;

/*  Define data structure for
 APCmd_CMD_802_11_TRANSMIT_RATES and APCmd_CMD_802_11_SUPPORTED_RATES*/
typedef struct _APCmd_DS_802_11_DATA_RATE
{

    UINT16 Action;    /* Ignore if the command is APCmd_CMD_802_11_SUPPORTED_RATE*/
    UINT8 DataRate[8];          /* Transmit or Supported data rate list */
}
APCmd_DS_802_11_DATA_RATE, *PAPCmd_DS_802_11_DATA_RATE;

/*  Define data structure for APCmd_CMD_802_11_RADIO_PREAMBLE */
typedef struct _APCmd_DS_802_11_RADIO_PREAMBLE
{

    UINT16 Action;    /* see general purpose section */
    UINT16 RadioPreamble;  /* see radio preamble section */
}
APCmd_DS_802_11_RADIO_PREAMBLE, *PAPCmd_DS_802_11_RADIO_PREAMBLE;

/*  Define data structure for APCmd_CMD_802_11_BEACON_PERIOD */
typedef struct _APCmd_DS_802_11_BEACON_PERIOD
{

    UINT16 Action;    /* see general purpose section */
    UINT16 BeaconPeriod;     /* in unit of 1024 microseconds (us) */
}
APCmd_DS_802_11_BEACON_PERIOD, *PAPCmd_DS_802_11_BEACON_PERIOD;

/*  Define data structure for APCmd_CMD_802_11_DTIM_PERIOD */
typedef struct _APCmd_DS_802_11_DTIM_PERIOD
{

    UINT16 Action;    /* see general purpose section */
    UINT16 DtimPeriod;   /* in unit of beacon intervals */
}
APCmd_DS_802_11_DTIM_PERIOD, *PAPCmd_DS_802_11_DTIM_PERIOD;

/*  Define data structure for APCmd_CMD_802_3_GET_STAT */
typedef struct _APCmd_DS_802_3_GET_STAT
{
    UINT32 XmitOK;
    UINT32 RcvOK;
    UINT32 XmitError;
    UINT32 RcvError;
    UINT32 XmitMulticast;
    UINT32 RcvMulticast;
    UINT32 XmitUnicast;
    UINT32 RcvUnicast;
    UINT32 RcvNoBuffer;
    UINT32 RcvCRCError;
}
APCmd_DS_802_3_GET_STAT, *PAPCmd_DS_802_3_GET_STAT;

/*  Define data structure for APCmd_CMD_802_3_LINK_SPEED */
typedef struct _APCmd_DS_802_3_LINK_SPEED
{

    UINT32 LinkSpeed; /* in unit of 1000 bps */
}
APCmd_DS_802_3_LINK_SPEED, *PAPCmd_DS_802_3_LINK_SPEED;

/*  Define data structure for APCmd_CMD_802_3_IP_ADDRESS */
/*  This command is for query only if DHCP is enabled */
typedef struct _APCmd_DS_802_3_IP_ADDRESS
{

    UINT16 Action;    /* see general purpose section */
    UINT8 IPAddress[6]; /* currently use the first 4 bytes */
}
APCmd_DS_802_3_IP_ADDRESS, *PAPCmd_DS_802_3_IP_ADDRESS;

/*  Define data structure for APCmd_CMD_802_3_SUBNET_MASK */
/*  This command is for query only if DHCP is enabled */
typedef struct _APCmd_DS_802_3_SUBNET_MASK
{

    UINT16 Action;    /* see general purpose section */
    UINT8 IPAddress[6]; /* currently use the first 4 bytes */
}
APCmd_DS_802_3_SUBNET_MASK, *PAPCmd_DS_802_3_SUBNET_MASK;

/*  Define data structure for APCmd_CMD_802_3_GATEWAY */
/*  This command is for query only if DHCP is enabled */
typedef struct _APCmd_DS_802_3_GATEWAY
{
    UINT16 Action;    /* see general purpose section */
    UINT8 IPAddress[6]; /* currently use the first 4 bytes */
}
APCmd_DS_802_3_GATEWAY, *PAPCmd_DS_802_3_GATEWAY;

/*  Define data structure for APCmd_CMD_802_3_DHCP */
typedef struct _APCmd_DS_802_3_DHCP
{
    UINT16 Action;    /* see general purpose section */
    UINT16 DHCPStatus;   /* see DHCP section */
}
APCmd_DS_802_3_DHCP, *PAPCmd_DS_802_3_DHCP;

/*  Define data structure for APCmd_CMD_802_3_ICMP */
typedef struct _APCmd_DS_802_3_ICMP
{
    UINT16 Action;    /* see general purpose section */
    UINT16 ICMPStatus;   /* see ICMP section */
}
APCmd_DS_802_3_ICMP, *PAPCmd_DS_802_3_ICMP;

/*  Define data structure for APCmd_CMD_802_11_LINK_TEST */
typedef struct _APCmd_CMD_802_11_LINK_TEST
{
    UINT8 IPAddress[6]; /* MAC Address of the target station currently use the first 4 bytes */
    UINT16 PktNum;   /* Packet number to be sent */
    UINT16 PktSize;   /* Size of the packet to be sent */
    UINT16 XmtPkt;   /* Test result, pkt transmitted that received an echo packet */
    UINT16 TotalErr;  /* Test result, pkt failure due to any reason*/
    UINT16 WepErr;   /* Test result, pkt failure due to WEP error */
}
APCmd_CMD_802_11_LINK_TEST, *PAPCmd_CMD_802_11_LINK_TEST;

#endif /* _HOST_H_ */
