/*******************************************************************
*
*    DESCRIPTION: AP Configuration on flash routines
*
*    AUTHOR: cvu@marvell.com
*
*    HISTORY:
*
*    12dec02,cvu,created
*
*******************************************************************/

/** include files **/
#include "wltypes.h"
#include "IEEE_types.h"
#include "apcfg.h"
#include "timer.h"
#include "ap_mib.h"
#include "util.h"
#include "oem.h"
#include "domain.h"
#include "wl_hal.h"
#include "flashdrvapi.h"

#define SAVE_WEP_CONFIG 1
#define WLAN_FILTER_CONFIGURED_KEY	0xdeadbeef

/** local definitions **/

static UINT32 CRC32(unsigned char *buf, int len);
/* default settings */

/** external functions **/

/** external data **/
extern UINT8  WepType[];
extern UINT32 WepTypeAll;
extern MIB_802DOT11 *mibSystem_p;
extern LOGIN Mlogin;

/** internal functions **/
int GetUserConfig(MIB_802DOT11 *pMib);

/** public data **/

/** private data **/
static apfdDriver_t gFlashDrv;
static char userSettings[SIZEOF_MIB_USER_AP];

static int flashDriverInitialized = FALSE;


/** private functions **/

/** public functions **/

void
apfdGetEtherAddr(unsigned char *etherAddr)
{
    MFG_CAL_DATA *mfg;
    int i;
    if (etherAddr == NULL)
    {
        return ;
    }
#ifdef EEPROM
    if (!flashDriverInitialized)
        flashDrvInit();
    gFlashDrv.MAC_Addr(etherAddr);
#else
#ifdef MAC_ADDR_FROM_FLASH
    mfg = (MFG_CAL_DATA *)(MFG_DATA_ADDRESS);

    if ( (mfg->MfgMacAddr[0] == 0xff) &&
         (mfg->MfgMacAddr[1] == 0xff) &&
         (mfg->MfgMacAddr[2] == 0xff) &&
         (mfg->MfgMacAddr[3] == 0xff) &&
         (mfg->MfgMacAddr[4] == 0xff) &&
         (mfg->MfgMacAddr[5] == 0xff) )
    {

        /* Keep default MAC address */
        return ;
    }

    /* Use the MAC address saved on flash */
    for (i = 0; i < 6; ++i)
    {
        etherAddr[i] = mfg->MfgMacAddr[i];
    }
#endif
#endif
}

/**
 * Call OEM supplied function to initialize flash driver object
 */
void
flashDrvInit(void)
{
    if (!flashDriverInitialized)
    {
        apfdDriverInit(&gFlashDrv);
        flashDriverInitialized = TRUE;
    }
}

/**
 * Reset user settings to factory settings
 * 
 * @return TRUE: success
 *         FALSE: failure
 */
int
ResetToFactorySettings(void)
{
    int rc;

    if (!flashDriverInitialized)
    {
        return FALSE;
    }

    rc = (*gFlashDrv.ResetToFactorySettings)();

    GetUserConfig(mibSystem_p);

    return rc;
}

#if defined(WIPOD_NEW_GUI) || defined(AP_PCI)
/**
 * Function to export MIB structure from flash for 
 * new GUI configuration
 * 
 * @return TRUE: success
 *         FALSE: failure
 */
int
GetApConfig(mib_user_AP_t *mibUserPlus)
{
    int ok;

    if (!flashDriverInitialized)
    {
        return FALSE;
    }
    memset(mibUserPlus, 0, SIZEOF_MIB_USER_AP);

    /* Read the user configuration from flash memory */
    ok = (*gFlashDrv.ReadUserSettings)((mib_AP_t *)mibUserPlus);

    if (!ok)
    {
        return FALSE;
    }

    /* Make sure the version is correct */
    if (mibUserPlus->mib_ap.mibVersion != MIB_AP_VERSION)
    {
        return FALSE;
    }
    return TRUE;
}

/**
 * Function to set MIB structure in flash for 
 * new GUI configuration. 
 * 
 * Note: User should always call GetApConfig() first, make modification to related fields
 * leaving other fields unchanged, and then call SetApConfig().
 *
 * @return TRUE: success
 *         FALSE: failure
 */
int
SetApConfig(mib_user_AP_t *mibUserPlus)
{
    int rc;

    /* Make sure the version is correct */
    if (mibUserPlus->mib_ap.mibVersion != MIB_AP_VERSION)
    {
        return FALSE;
    }

    rc = (*gFlashDrv.WriteUserSettings)(mibUserPlus);
#ifdef AP_PCI
    {
        extern WL_SYS_CFG      *pSystemHaltest;
        mib_InitPriv(pSystemHaltest->Mib802dot11);
    }
#endif
    return rc;
}
#endif/*WIPOD_NEW_GUI*/

/**
 * Get user settings from flash and populate the MIB
 * 
 * @return TRUE: success
 *         FALSE: failure
 */
int
GetUserConfig(MIB_802DOT11 *pMib)
{
    MFG_CAL_DATA *mfg;
    mib_AP_t *mibUser;
    mib_user_AP_t *mibUserPlus;
    int i;
    int ok;

    UINT8 minChannel, maxChannel, defaultChannel;
    if (!flashDriverInitialized)
    {
        return FALSE;
    }
#ifdef EEPROM
    mfg = gFlashDrv.CalData();
#else
    mfg = (MFG_CAL_DATA *)(MFG_DATA_ADDRESS);
#endif
    memset(userSettings, 0, SIZEOF_MIB_USER_AP);

    mibUser = (mib_AP_t *)userSettings;
    mibUserPlus = (mib_user_AP_t *)userSettings;
    /* Read the user configuration from flash memory */
    ok = (*gFlashDrv.ReadUserSettings)(mibUser);

//wuming temp

	{
		unsigned char mac[6], mac1[6];
		int i;
		sernet_uplayer_init();
		
		sernet_Set_Eth_Mac(mac);
		for(i=0;i<6;i++)
			mib_currentmacaddress[i] = mac[i];
			
			
			
		sernet_Set_RF_Mac(mac1);
		for(i=0;i<6;i++)
			mib_OpData_p->StaMacAddr[i] = mac1[i];
	}		
			
		




    if (!ok)
    {
        return FALSE;
    }

    /* Make sure the version is correct */
    if (mibUser->mibVersion != MIB_AP_VERSION)
    {
        return FALSE;
    }

    /* Populate the various MIB structures */
    util_CopyList(mib_StaCfg_p->DesiredSsId, mibUser->DesiredSsId, IEEEtypes_SSID_SIZE);
    mib_StaCfg_p->OpRateSet[0] = mibUser->OpRateSet[0];
    mib_StaCfg_p->OpRateSet[1] = mibUser->OpRateSet[1];
    mib_StaCfg_p->OpRateSet[2] = mibUser->OpRateSet[2];
    mib_StaCfg_p->OpRateSet[3] = mibUser->OpRateSet[3];
    mib_StaCfg_p->OpRateSet[4] = mibUser->OpRateSet[4];
    mib_StaCfg_p->OpRateSet[5] = mibUser->OpRateSet[5];
    mib_StaCfg_p->OpRateSet[6] = mibUser->OpRateSet[6];
    mib_StaCfg_p->OpRateSet[7] = mibUser->OpRateSet[7];
    mib_StaCfg_p->OpRateSet[8] = mibUser->OpRateSet[8];
    mib_StaCfg_p->OpRateSet[9] = mibUser->OpRateSet[9];
    mib_StaCfg_p->OpRateSet[10] = mibUser->OpRateSet[10];
    mib_StaCfg_p->OpRateSet[11] = mibUser->OpRateSet[11];
    mib_StaCfg_p->OpRateSet[12] = mibUser->OpRateSet[12];
    mib_StaCfg_p->OpRateSet[13] = mibUser->OpRateSet[13];
    mib_StaCfg_p->BcnPeriod = mibUser->BcnPeriod;
    mib_StaCfg_p->DtimPeriod = mibUser->DtimPeriod;
    mib_OpData_p->RtsThresh = mibUser->RtsThresh;
#if 0 /* Rateadapt on AP32 hardcoded 0x4 */
    mib_OpData_p->ShortRetryLim = mibUser->RtsRetryLim;
#else
    mib_OpData_p->ShortRetryLim = 0x4;
#endif
    mib_OpData_p->FragThresh = mibUser->FragThresh;
    if (domainGetInfo(&minChannel, &maxChannel, &defaultChannel) == OS_SUCCESS)
    {
        if ( (mibUser->CurrRFChan > maxChannel) ||
             (mibUser->CurrRFChan < minChannel) )
            mibUser->CurrRFChan = defaultChannel;
    }
    PhyDSSSTable_p->CurrChan = mibUser->CurrRFChan;

#ifdef SAVE_WEP_CONFIG

    mib_StaCfg_p->PrivOption = mibUser->PrivOption;
    mib_PrivacyTable_p->PrivInvoked = mibUser->PrivInvoked;

    mib_AuthAlg_p->Enable = mibUser->AuthEnable;
    mib_AuthAlg_p->Type = mibUser->AuthType;
    mib_wep = mibUser->mib_wep;

    /* WEP configuration */
    WepType[0] = mibUser->WepType[0];
    WepType[1] = mibUser->WepType[1];
    WepType[2] = mibUser->WepType[2];
    WepType[3] = mibUser->WepType[3];
    
    WepTypeAll = WepType[0] | WepType[1] |WepType[2] | WepType[3];

    memset(mib_WepDefaultKeys_p[0].WepDefaultKeyValue, 0, 13);
    memcpy(mib_WepDefaultKeys_p[0].WepDefaultKeyValue, mibUser->WepDefaultKeyValue1, 13);

    memset(mib_WepDefaultKeys_p[1].WepDefaultKeyValue, 0, 13);
    memcpy(mib_WepDefaultKeys_p[1].WepDefaultKeyValue, mibUser->WepDefaultKeyValue2, 13);

    memset(mib_WepDefaultKeys_p[2].WepDefaultKeyValue, 0, 13);
    memcpy(mib_WepDefaultKeys_p[2].WepDefaultKeyValue, mibUser->WepDefaultKeyValue3, 13);

    memset(mib_WepDefaultKeys_p[3].WepDefaultKeyValue, 0, 13);
    memcpy(mib_WepDefaultKeys_p[3].WepDefaultKeyValue, mibUser->WepDefaultKeyValue4, 13);

    mib_defaultkeyindex = mibUser->DefaultWepKeyindex;
#endif

    mib_PhyTxPwrData_p->CurrTxPwrLevel = mibUser->CurrTxPwrLevel;
    mib_PhyTxPwrData_p->TxPwrLevel1 = mibUser->TxPwrLevel1;
    mib_PhyTxPwrData_p->TxPwrLevel2 = mibUser->TxPwrLevel2;
    mib_PhyTxPwrData_p->TxPwrLevel3 = mibUser->TxPwrLevel3;
    mib_PhyTxPwrData_p->TxPwrLevel4 = mibUser->TxPwrLevel4;
    mib_PhyTxPwrData_p->TxPwrLevel5 = mibUser->TxPwrLevel5;
    mib_PhyTxPwrData_p->TxPwrLevel6 = mibUser->TxPwrLevel6;
    mib_PhyTxPwrData_p->TxPwrLevel7 = mibUser->TxPwrLevel7;
    mib_PhyTxPwrData_p->TxPwrLevel8 = mibUser->TxPwrLevel8;

    mib_txAntenna = mibUser->CurrTxAnt;
    mib_rxAntenna = mibUser->CurrRxAnt;
#if 0
    switch (mfg->Ant & 0xf)
    {
    case 0x00:
        mib_rxAntenna = AP_ANT_A;
        mib_txAntenna = AP_ANT_A;
        break;
    case 0x05:
        mib_rxAntenna = AP_ANT_B;
        mib_txAntenna = AP_ANT_B;
        break;
    case 0x0a:
        mib_rxAntenna = AP_ANT_DIVERSITY;
        mib_txAntenna = AP_ANT_DIVERSITY;
        break;
    case 0x08:
        mib_rxAntenna = AP_ANT_DIVERSITY;
        mib_txAntenna = AP_ANT_A;
        break;
    case 0x09:
        mib_rxAntenna = AP_ANT_DIVERSITY;
        mib_txAntenna = AP_ANT_B;
        break;
    case 0x04:
        mib_rxAntenna = AP_ANT_B;
        mib_txAntenna = AP_ANT_A;
        break;
    case 0x01:
        mib_rxAntenna = AP_ANT_A;
        mib_txAntenna = AP_ANT_B;
        break;
    default:
        break;
    }
#endif
    mib_preAmble = mibUser->PreAmble;

    mib_broadcastssid = mibUser->broadcastssid;

    mib_dhcpenable = mibUser->EnableDHCP;
    mib_DHCP_p->IPAddr = mibUser->IPAddr;
    mib_DHCP_p->SubnetMask = mibUser->SubnetMask;
    mib_DHCP_p->GwyAddr = mibUser->GwyAddr;

    mib_txDataRate = mibUser->txDataRate;

    for (i = 0; i < 32; ++i)
    {
        Mlogin.username[i] = mibUser->username[i];
        Mlogin.password[i] = mibUser->password[i];
    }
    util_CopyList(mib_DeviceName, mibUser->devName, 16);

    /* AP Mode */
    if(mibUser->apmode != 0xff)
    {
        mib_ApMode = mibUser->apmode;
    }
    
	if(mibUserPlus->mib_wlanfilter_cfgkey==WLAN_FILTER_CONFIGURED_KEY)
	{
		mib_wlanfilterno = mibUserPlus->mib_wlanfilter_no;
		mib_wlanfiltermode = mibUserPlus->mib_wlanfilter_mode;
		memcpy(mib_wlanfiltermac, mibUserPlus->mib_wlanfilter_mac, 256*6);
	}

#if defined(WIPOD) && !defined(WIPOD_NEW_GUI)
    /* Read separate WPA settings */
    strcpy(mibUserPlus->mib_WPA_passphrase, mibUser->mib_WPA_passphrase);
    mibUserPlus->mib_WPA_encr_algo = mibUser->mib_WPA_encr_algo;
    mibUserPlus->mib_WPA_cfgkey = mibUser->mib_WPA_cfgkey;
#endif

    if (mibUserPlus->mib_WPA_cfgkey == 0xabcd)
    {
        mib_PrivacyTable_p->RSNEnabled = mibUser->RSNEnabled;
        mib_RSNConfig_p->GroupRekeyTime = mibUser->GrpRekeyTime;
        mib_RSNConfig_p->MulticastCipher[3] = mibUserPlus->mib_WPA_encr_algo;
        mib_RSNConfigUnicastCiphers_p->UnicastCipher[3] = mibUserPlus->mib_WPA_encr_algo;
        strcpy(mib_RSNConfig_p->PSKPassPhrase,mibUserPlus->mib_WPA_passphrase);
    }

#ifdef WDS_FEATURE
    if(mibUserPlus->mib_wds_cfgkey==WDS_CONFIGURED_KEY)
    {
        mib_disableAssoc = mibUserPlus->mib_disable_assoc;
        mib_wdsEnable = mibUserPlus->mib_wds_Enable;
        mib_wdsno = mibUserPlus->mib_wds_no;
        mib_wdsOption = mibUserPlus->mib_wds_option;
        memcpy(mib_wdsmac, mibUserPlus->mib_wds_mac, MAX_WDS_PORT*6);
    }
#endif
    mib_forceProtectiondisable = mibUserPlus->mib_forceProtectiondisable;
    if(mibUserPlus->mib_enableFixedRateTx != 0xff)
    {
        mib_enableFixedRateTx = mibUserPlus->mib_enableFixedRateTx;
        mib_txDataRate = mibUserPlus->mib_txDataRate;
        mib_txDataRateG = mibUserPlus->mib_txDataRateG;
    }

#if defined(WIPOD)
    /* WiPod 5-in-1 */ 
    mib_wipodMode = mibUser->wipodMode;
#endif

#ifdef BURST_MODE
    if(mibUserPlus->mib_burstmode != 0xff)
    {
        mib_BurstMode_p->mib_burstmode = mibUserPlus->mib_burstmode;
        mib_BurstMode_p->mib_burstrate = mibUserPlus->mib_burstrate;
    }
#endif
#ifdef SELF_CTS
    mib_StaCfg_p->SelfCts = mibUserPlus->mib_SelfCts;
#endif
#ifdef TURBO_SETUP
    {
        extern UINT8 mib_TSMode;
        mib_TSMode = mibUserPlus->mib_TSMode; /* 0x00 or 0xff default factory settings, 0x02 Turbo Setup, 0x04 Web GUI*/
    }
#endif
/* if two mac addrs are different, use them */
#if 0
    for (i = 0; i < 6; ++i)
    {
        if (mibUser->EthMACAddr[i] != mibUser->RFMACAddr[i] ) {
            for (i = 0; i < 6; ++i)
            {
                /*mib_currentmacaddress[i] = mibUser->EthMACAddr[i];*/
                mib_OpData_p->StaMacAddr[i] = mibUser->RFMACAddr[i];
            }
            return TRUE;
        }
    }
#endif
    return TRUE;
}

/**
 * Gather the MIB settings and save to flash as user settings
 * 
 * @return TRUE: success
 *         FALSE: error
 */
int
SetUserConfig(void/*MIB_802DOT11 *pMib*/)
{
    mib_AP_t *mibUser;
    mib_user_AP_t *mibUserPlus;
    int i;
    int rc = FALSE;
    int ok;
    unsigned long crc32;

    if (!flashDriverInitialized)
    {
        return FALSE;
    }

    memset(userSettings, 0xff, SIZEOF_MIB_USER_AP);

    mibUser = (mib_AP_t *)userSettings;
    mibUserPlus = (mib_user_AP_t *)userSettings;
    /* Read the user configuration from flash memory*/ 
    ok = (*gFlashDrv.ReadUserSettings)(mibUser);

    if (!ok)
    {
        return FALSE;
    }

    crc32 = CRC32((unsigned char *)mibUserPlus, SIZEOF_MIB_USER_AP);

    mibUser->mibVersion = MIB_AP_VERSION;

    util_CopyList(mibUser->DesiredSsId, mib_StaCfg_p->DesiredSsId, IEEEtypes_SSID_SIZE);
    mibUser->OpRateSet[0] = mib_StaCfg_p->OpRateSet[0];
    mibUser->OpRateSet[1] = mib_StaCfg_p->OpRateSet[1];
    mibUser->OpRateSet[2] = mib_StaCfg_p->OpRateSet[2];
    mibUser->OpRateSet[3] = mib_StaCfg_p->OpRateSet[3];
    mibUser->OpRateSet[4] = mib_StaCfg_p->OpRateSet[4];
    mibUser->OpRateSet[5] = mib_StaCfg_p->OpRateSet[5];
    mibUser->OpRateSet[6] = mib_StaCfg_p->OpRateSet[6];
    mibUser->OpRateSet[7] = mib_StaCfg_p->OpRateSet[7];
    mibUser->OpRateSet[8] = mib_StaCfg_p->OpRateSet[8];
    mibUser->OpRateSet[9] = mib_StaCfg_p->OpRateSet[9];
    mibUser->OpRateSet[10] = mib_StaCfg_p->OpRateSet[10];
    mibUser->OpRateSet[11] = mib_StaCfg_p->OpRateSet[11];
    mibUser->OpRateSet[12] = mib_StaCfg_p->OpRateSet[12];
    mibUser->OpRateSet[13] = mib_StaCfg_p->OpRateSet[13];

    mibUser->BcnPeriod = mib_StaCfg_p->BcnPeriod;
    mibUser->DtimPeriod = mib_StaCfg_p->DtimPeriod;
    mibUser->RtsThresh = mib_OpData_p->RtsThresh;
    mibUser->RtsRetryLim = mib_OpData_p->ShortRetryLim;
    mibUser->FragThresh = mib_OpData_p->FragThresh;
    mibUser->CurrRFChan = PhyDSSSTable_p->CurrChan;

#ifdef SAVE_WEP_CONFIG

    mibUser->WepType[0] = WepType[0];
    mibUser->WepType[1] = WepType[1];
    mibUser->WepType[2] = WepType[2];
    mibUser->WepType[3] = WepType[3];
    mibUser->PrivOption = mib_StaCfg_p->PrivOption;
    mibUser->PrivInvoked = mib_PrivacyTable_p->PrivInvoked;

    mibUser->AuthEnable = mib_AuthAlg_p->Enable;
    mibUser->AuthType = mib_AuthAlg_p->Type;
    mibUser->mib_wep = mib_wep;
    memset(mibUser->WepDefaultKeyValue1, 0, 13);
    memcpy(mibUser->WepDefaultKeyValue1, mib_WepDefaultKeys_p[0].WepDefaultKeyValue, 13);

    memset(mibUser->WepDefaultKeyValue2, 0, 13);
    memcpy(mibUser->WepDefaultKeyValue2, mib_WepDefaultKeys_p[1].WepDefaultKeyValue, 13);

    memset(mibUser->WepDefaultKeyValue3, 0, 13);
    memcpy(mibUser->WepDefaultKeyValue3, mib_WepDefaultKeys_p[2].WepDefaultKeyValue, 13);

    memset(mibUser->WepDefaultKeyValue4, 0, 13);
    memcpy(mibUser->WepDefaultKeyValue4, mib_WepDefaultKeys_p[3].WepDefaultKeyValue, 13);

    mibUser->DefaultWepKeyindex = mib_defaultkeyindex;
#endif

    mibUser->CurrTxPwrLevel = mib_PhyTxPwrData_p->CurrTxPwrLevel;
    mibUser->TxPwrLevel1 = mib_PhyTxPwrData_p->TxPwrLevel1;
    mibUser->TxPwrLevel2 = mib_PhyTxPwrData_p->TxPwrLevel2;
    mibUser->TxPwrLevel3 = mib_PhyTxPwrData_p->TxPwrLevel3;
    mibUser->TxPwrLevel4 = mib_PhyTxPwrData_p->TxPwrLevel4;
    mibUser->TxPwrLevel5 = mib_PhyTxPwrData_p->TxPwrLevel5;
    mibUser->TxPwrLevel6 = mib_PhyTxPwrData_p->TxPwrLevel6;
    mibUser->TxPwrLevel7 = mib_PhyTxPwrData_p->TxPwrLevel7;
    mibUser->TxPwrLevel8 = mib_PhyTxPwrData_p->TxPwrLevel8;

    mibUser->CurrTxAnt = mib_PhyAntData_p->CurrTxAnt;
    mibUser->CurrRxAnt = mib_rxAntenna;

    mibUser->PreAmble = mib_preAmble;

    mibUser->broadcastssid = mib_broadcastssid;

    if (!mib_dhcpenable)
    {
        if (mib_DHCP_p->IPAddr)
        {
            mibUser->IPAddr = mib_DHCP_p->IPAddr;
        }
        if (mib_DHCP_p->SubnetMask)
        {
            mibUser->SubnetMask = mib_DHCP_p->SubnetMask;
        }
        if ( mib_DHCP_p->GwyAddr )
        {
            mibUser->GwyAddr = mib_DHCP_p->GwyAddr;
        }
    }

    mibUser->EnableDHCP = mib_dhcpenable;

    mibUser->txDataRate = mib_txDataRate;

    /* AP Mode */
     mibUser->apmode = mib_ApMode;


    for (i = 0; i < 6; ++i)
    {
        mibUser->EthMACAddr[i] = mib_currentmacaddress[i];
        mibUser->RFMACAddr[i] = mib_OpData_p->StaMacAddr[i];
    }

    for (i = 0; i < 32; ++i)
    {
        mibUser->username[i] = Mlogin.username[i];
        mibUser->password[i] = Mlogin.password[i];
    }
    util_CopyList(mibUser->devName, mib_DeviceName,  16);

	mibUserPlus->mib_wlanfilter_cfgkey = WLAN_FILTER_CONFIGURED_KEY;
	mibUserPlus->mib_wlanfilter_no = mib_wlanfilterno; 
	mibUserPlus->mib_wlanfilter_mode = mib_wlanfiltermode;
    memcpy(mibUserPlus->mib_wlanfilter_mac, mib_wlanfiltermac, 256*6);
    
    /*WPA*/
    mibUser->RSNEnabled = mib_PrivacyTable_p->RSNEnabled;
    mibUser->GrpRekeyTime = mib_RSNConfig_p->GroupRekeyTime;
    strcpy(mibUserPlus->mib_WPA_passphrase, mib_RSNConfig_p->PSKPassPhrase);
    mibUserPlus->mib_WPA_encr_algo = mib_RSNConfigUnicastCiphers_p->UnicastCipher[3];
    mibUserPlus->mib_WPA_cfgkey = 0xabcd;

#ifdef WDS_FEATURE
    mibUserPlus->mib_disable_assoc = mib_disableAssoc;
    mibUserPlus->mib_wds_cfgkey = WDS_CONFIGURED_KEY;
    mibUserPlus->mib_wds_Enable = mib_wdsEnable;
    mibUserPlus->mib_wds_no = mib_wdsno;
    mibUserPlus->mib_wds_option = mib_wdsOption;
    memcpy(mibUserPlus->mib_wds_mac, mib_wdsmac, MAX_WDS_PORT*6);
#endif

    mibUserPlus->mib_forceProtectiondisable = mib_forceProtectiondisable;
    mibUserPlus->mib_enableFixedRateTx = mib_enableFixedRateTx;
    mibUserPlus->mib_txDataRate = mib_txDataRate;
    mibUserPlus->mib_txDataRateG = mib_txDataRateG;

#ifdef BURST_MODE
    mibUserPlus->mib_burstmode = mib_BurstMode_p->mib_burstmode;
    mibUserPlus->mib_burstrate = mib_BurstMode_p->mib_burstrate;
#endif    

#ifdef SELF_CTS
    mibUserPlus->mib_SelfCts = mib_StaCfg_p->SelfCts;
#endif

#ifdef TURBO_SETUP
    {
        extern UINT8 mib_TSMode;
        mibUserPlus->mib_TSMode = mib_TSMode; /* 0x00 or 0xff default factory settings, 0x02 Turbo Setup, 0x04 Web GUI*/
    }
#endif
    /*******************************************
    /* Please add all other MIBs above this line
    /*******************************************/

#if defined(WIPOD)
    /* WiPod 5-in-1 */
    mibUser->wipodMode = mib_wipodMode;

    #if !defined(WIPOD_NEW_GUI)
    /* Save separate WPA settings */
    strcpy(mibUser->mib_WPA_passphrase, mibUserPlus->mib_WPA_passphrase);
    mibUser->mib_WPA_encr_algo = mibUserPlus->mib_WPA_encr_algo;
    mibUser->mib_WPA_cfgkey = mibUserPlus->mib_WPA_cfgkey;
    #endif

    /* Create mirror of common MIBs to WB */
    mibUser ++;
    mibUser->wipodMode = mib_wipodMode;

    if ( !mib_dhcpenable ) {
        if ( mib_DHCP_p->IPAddr ) {
            mibUser->IPAddr = mib_DHCP_p->IPAddr;
        }
        if ( mib_DHCP_p->SubnetMask ) {
            mibUser->SubnetMask = mib_DHCP_p->SubnetMask;
        }
        //if ( mib_DHCP_p->GwyAddr ) {
            mibUser->GwyAddr = mib_DHCP_p->GwyAddr;
        //}
    }
    mibUser->EnableDHCP = mib_dhcpenable;

    for (i = 0; i < 32; ++i) {
        mibUser->username[i] = Mlogin.username[i];
        mibUser->password[i] = Mlogin.password[i];
    }
    util_CopyList(mibUser->devName, mib_DeviceName,  16);
    mibUser --;
#endif /* WIPOD */

    /************************************************
    /* Please do not add all new MIBs below this line
    /************************************************/
    if (crc32 != CRC32(mibUserPlus, SIZEOF_MIB_USER_AP))
    {
        rc = (*gFlashDrv.WriteUserSettings)(mibUser);
    }
    return rc;
}

/*
 ** FirmwareUpgrade
 *
 *  FILENAME: D:\Builds\AP21\802.11_MAC-DRIVER_SW\AP\Src\apflash.c
 *
 *  PARAMETERS:
 *
 *  DESCRIPTION:
 *
 *  RETURNS:
 *
 */

int FirmwareUpgrade(char *pData, unsigned long len)
{
    if (!flashDriverInitialized)
    {
        return FALSE;
    }
    return(*gFlashDrv.UpgradeFirmware)(pData, len);
}
/*
 ** UserConfigModified
 *
 *  FILENAME: D:\Builds\AP21\802.11_MAC-DRIVER_SW\AP\Src\apflash.c
 *
 *  PARAMETERS:
 *
 *  DESCRIPTION: the fuction called by user when the user config data has
 *               been modified
 *
 *  RETURNS:
 *
 */

static int userConfigModifiedCnt = 0;
int UserConfigModified(void)
{
    return(++userConfigModifiedCnt);
}
/*
 ** SaveUserConfig
 *
 *  FILENAME: D:\Builds\AP21\802.11_MAC-DRIVER_SW\AP\Src\apflash.c
 *
 *  PARAMETERS:
 *
 *  DESCRIPTION:
 *
 *  RETURNS:
 *
 */
int SaveUserConfig(void)
{
    /*MIB_802DOT11 *pMib; */
    if (userConfigModifiedCnt > 0)
    {
#ifdef FLASH_CONFIG
        SetUserConfig();
#endif
        userConfigModifiedCnt = 0;
        return TRUE;
    }
    return FALSE;
}
#ifdef ETHERSWITCH_IN_BOOT
void SetIPAddr()
{
    memcpy(&mib_DHCP_p->IPAddr, (volatile char *)0xc000bf7c, 4);
}
#endif

#ifdef EEPROM
MFG_CAL_DATA * GetCalData(void)
{
    return (MFG_CAL_DATA *)gFlashDrv.CalData();
}
#endif


extern unsigned long crc32_table[];
#define SWAP(a,b) do { unsigned char  t = S[a]; S[a] = S[b]; S[b] = t; } while(0)
static unsigned char S[256] = {0};
static UINT32 CRC32(unsigned char *buf, int len)
{

    unsigned long i, j, k, crc;
    unsigned char  *pos;
    pos = buf;

    /* Compute CRC32 */
    crc = ~0;
    i = j = 0;
    for (k = 0; k < len; k++)
    {
        crc = crc32_table[(crc ^ *pos) & 0xff] ^ (crc >> 8);
        i = (i + 1) & 0xff;
        j = (j + S[i]) & 0xff;
        SWAP(i, j);
        *pos++ ^= S[(S[i] + S[j]) & 0xff];
    }
    crc = ~crc;

    return crc;
}
