/******************** (c) Marvell Semiconductor, Inc., 2001 ******************* 
* 
* Purpose: 
*    This file contains the implementations of the function prototypes for the 
*    Bridge Task. 
* 
* Public Procedures: 
*    bridge_Init       Initialzies Bridge Task and related components 
*    bridge_Start      Starts running the Bridge Task 
* 
* Private Procedures: 
*    None  
* 
* Notes: 
*    None. 
* 
*****************************************************************************/ 

/*============================================================================= */
/*                               INCLUDE FILES */
/*============================================================================= */
#include "wltypes.h" 
#include "ap_api.h"
#include <cyg/kernel/kapi.h>
#include <cyg/io/io.h>
#include <cyg/hal/hal_arch.h>
#include "eth_debug.h"
#include "osif.h"
#include "IEEE_types.h"
#include "timer.h"
#include "Hsm.h"
#include "tkip.h"
#include "keyMgmtCommon.h"
#include "keyMgmt.h"
#include "ds.h"
#include "sysinfo.h"
#include "wpa.h"
#include "wl_mib.h"
#include "bufmgt.h"
#include "memmap.h"
#include "wlmac.h"
#include "wl_hal.h"
#include "wl_macros.h"
#include "bridge.h"
#include "dsport.h"

#ifdef FREE_IN_BRIDGE
#include "etherlow.h"
#include "ether.h"
extern UINT32 EtherEventSentFlag;
#endif
#ifdef WDS_FEATURE
#include "wds.h"
#endif

/*============================================================================= */
/*                                DEFINITIONS */
/*============================================================================= */
#define GPIO_WLAN_TX 0
#define GPIO_WLAN_RX 1
#define GPIO_LAN_TX  2
#define GPIO_LAN_RX  3

#undef NO_OF_PKTS
#define NO_OF_PKTS 10


#define MACADDR_CPY(macaddr1,macaddr2) { *(UINT16*)macaddr1 = *(UINT16*)macaddr2; \
                    *(UINT16 *)((UINT16*)macaddr1+1) = *(UINT16 *)((UINT16*)macaddr2+1); \
                    *(UINT16 *)((UINT16*)macaddr1+2) = *(UINT16 *)((UINT16*)macaddr2+2);}


/*============================================================================= */
/*                              TYPE DEFINITIONS */
/*============================================================================= */
/* */

/*============================================================================= */
/*                         IMPORTED PUBLIC VARIABLES */
/*============================================================================= */
extern struct eth_drv_sc *sc1;
/*extern NET_DEV eth_dev; */
extern UINT8 mib_currentmacaddress[];
extern MIB_PRIVACY_TABLE          *mib_PrivacyTable_p;

/*============================================================================ */
/*                          MODULE LEVEL VARIABLES */
/*============================================================================= */
#define sysinfo_TASK_NAME  "Bridge"
#define sysinfo_TASK_STACK_SIZE    8192
#if defined(BRIDGE_POLLING) || defined(BRIDGE_POLLING_1) 
    #define sysinfo_TASK_PRI         30
#else
    #define sysinfo_TASK_PRI         6
#endif
static cyg_thread       Task;
static cyg_handle_t     TaskHdl;
//wuming temp
//static UINT8            Stack[sysinfo_TASK_STACK_SIZE]__attribute__ ((section (".itcmbuf")));
static UINT8            Stack[sysinfo_TASK_STACK_SIZE];

#ifndef NO_ECOS_TCP_IP_STACK
extern void SendToStack(IEEEtypes_8023_Frame_t *frame_p,int length);
unsigned char * rcvdData;
#endif /* ECOS_TCP_IP_STACK */
unsigned int packet_cnt[2];

/*============================================================================= */
/*                   PRIVATE PROCEDURES (ANSI Prototypes) */
/*============================================================================= */

UINT32 ProcessWirelessData(int pkts);
UINT32 ProcessEtherData(int pkts);
#ifndef NO_ECOS_TCP_IP_STACK
UINT32 ProcessLlcData(void);
#endif /* ECOS_TCP_IP_STACK */

static void BridgeTask( cyg_addrword_t Data ); 

UINT32 (*ProcessEtherDataFp)(int pkts)= ProcessEtherData;
UINT32 (*ProcessWirelessDataFp)(int pkts)= ProcessWirelessData;
#ifndef NO_ECOS_TCP_IP_STACK
UINT32 (*ProcessLlcDataFp)(void)= ProcessLlcData;
#endif /* ECOS_TCP_IP_STACK */

void led_Processing(void);
/*============================================================================= */
/*                         CODED PUBLIC PROCEDURES */
/*============================================================================= */
void   MemDma(UINT32 x, UINT32 y, UINT32 z);

/****************************************************************************** 
 * 
 * Name: bridge_Init 
 * 
 * Description: 
 *   This routine is called to initialize the the Bridge Task 
 *   and related components. 
 * 
 * Conditions For Use: 
 *   None. 
 * 
 * Arguments: 
 *   None. 
 * 
 * Return Value: 
 *   Status indicating success or failure 
 * 
 * Notes: 
 *   None. 
 * 
 * 
 *****************************************************************************/ 
extern WL_STATUS bridge_Init( void ) 
{ 
    apio_ErrCode_t errCode;
#ifdef WDS_FEATURE
    DSInit();
#endif 
    errCode = apio_open("eth0",&etherHandle,NULL);
    errCode = apio_open("wl0",&wlanHandle,NULL);

    cyg_thread_create(sysinfo_TASK_PRI,
                      BridgeTask,
                      0,
                      sysinfo_TASK_NAME,
                      (void *)(Stack),
                      sysinfo_TASK_STACK_SIZE,
                      &TaskHdl,
                      &Task);

    return(OS_SUCCESS); 
} 


/****************************************************************************** 
 * 
 * Name: bridge_Start 
 * 
 * Description: 
 *   This routine is called to start the Bridge Task running. 
 * 
 * Conditions For Use: 
 *   Its associated queues has been initialized by calling ds_Init().
 * 
 * Arguments: 
 *   None. 
 * 
 * Return Value: 
 *   None. 
 * 
 * Notes: 
 *   None. 
 * 
 *****************************************************************************/ 
extern void bridge_Start( void ) 
{ 
    cyg_thread_resume(TaskHdl); 
} 

/****************************************************************************** 
 * 
 * Name: ProcessEAPoL 
 * 
 * Description: 
 *   This routine is called to do 802.1x authentication and key managment. 
 * 
 * Conditions For Use: 
 *   Should pass an EAPoL packet to this function.
 * 
 * Arguments: 
 *   apio_bufdescr_t pointer to the incoming packet buffer.
 *   Mac address pointer to the source Mac.  
 * 
 * Return Value: 
 *   apio_bufdescr_t pointer to the outgoing packet buffer.
 * 
 * Notes: 
 *   None. 
 * 
 *****************************************************************************/

apio_bufdescr_t * ProcessEAPoL(apio_bufdescr_t *pBufDescr, IEEEtypes_MacAddr_t *pMacStaAddr)
{
    EAPOL_KeyMsg_t *pEAPoL;
    Msg msg;

    if ( pBufDescr == NULL )
    {/*Sta association event */
        msg.evt = STA_ASSO_EVT;
    }
    else
    {

        pEAPoL = (EAPOL_KeyMsg_t *)pBufDescr->framePtr;
        if ( pEAPoL->hdr_8021x.pckt_type == 0x03 )
        {/*key data */
            msg.evt = MSGRECVD_EVT;
            msg.body = pEAPoL;
        }
        else
        {/*reject all other EAPoL packet for now.... */
            /*    apio_free(WlanHandle,pBufDescr); */
            return NULL;
        }

    }
    ProcessKeyMgmtData(pBufDescr, pMacStaAddr, &msg);
    return NULL;
}

/*============================================================================= */
/*                         CODED PRIVATE PROCEDURES */
/*============================================================================= */

/****************************************************************************** 
 * 
 * Name: BridgeTask 
 * 
 * Description: 
 *   This routine is the task that runs continuously, waking to process 
 *   time outs and messages received from the different message queues. 
 *   It commences when the bridge_Start() routine is invoked. 
 * 
 * Conditions For Use: 
 *   None
 * 
 * Arguments: 
 *   Arg1 (i  ): Data - Data passed to the task; not actually used - just 
 *                      included to conform to the structure the OS 
 *                      expects when creating a task 
 * 
 * Return Value: 
 *   None. 
 * 
 * Notes: 
 *   None. 
 * 
 *****************************************************************************/ 
 void CheckStaAssocState3()
  {
            DistTaskMsg_t *pDistMsg;

            if ( (pDistMsg = (DistTaskMsg_t *)os_QueueReadNoBlock(sysinfo_WPA_Q)) != NULL )
            {
                if ( pDistMsg->MsgType == STA_ASSOMSGRECVD )
                {
                    StaAssocStateMsg_t *staAssocMsg_p;

                    staAssocMsg_p = &pDistMsg->msg.StaAssocStateMsg;
                    /* call key mgmt function */
                    /*pBufDescrEAPoL = ProcessEAPoL( NULL, &staAssocMsg_p->staMACAddr); */
                    ProcessEAPoL( NULL, &staAssocMsg_p->staMACAddr);
                }
                else if ( pDistMsg->MsgType == TIMERMSGRECVD )
                {
                    distQ_TimerMsg_t *distTimerMsg_p = &pDistMsg->msg.distQ_TimerMsg;
                    keyMgmt_Timeout(distTimerMsg_p);
                }
                msi_wl_FreePktBuf((char *)pDistMsg);
            }
        } 
 
#ifdef BRIDGE_POLLING_1
UINT8 bridge_polling = 1;
int setBridgeTaskToIntDriven(void)
{
    bridge_polling = 0;
    cyg_thread_set_priority(TaskHdl, 6);
}
#endif
static void BridgeTask( cyg_addrword_t Data ) 
{ 
    UINT32 EventsTriggered; 
    UINT32 noPkts, noPkts1;
    static UINT32 count = 0;

    if ( mib_PrivacyTable_p->RSNEnabled )
    {
        KeyMgmtInit();
    }
    while(1)
    {
#ifdef BRIDGE_POLLING_1
    while (bridge_polling) 
    {
        noPkts = (*ProcessWirelessDataFp)(NO_OF_PKTS);
        if(noPkts)            
            apctlSwLedEnable(GPIO_WLAN_RX);
        noPkts1 = (*ProcessEtherDataFp)(NO_OF_PKTS);
        if(noPkts1)    
            apctlSwLedEnable(GPIO_LAN_RX);
        noPkts += noPkts1;
        #ifndef NO_ECOS_TCP_IP_STACK
        noPkts += (*ProcessLlcDataFp)();
        #endif /* ECOS_TCP_IP_STACK */

        {
            DistTaskMsg_t *pDistMsg;

            if ( (pDistMsg = (DistTaskMsg_t *)os_QueueReadNoBlock(sysinfo_WPA_Q)) != NULL )
            {
                if ( pDistMsg->MsgType == STA_ASSOMSGRECVD )
                {
                    StaAssocStateMsg_t *staAssocMsg_p;

                    staAssocMsg_p = &pDistMsg->msg.StaAssocStateMsg;
                    /* call key mgmt function */
                    /*pBufDescrEAPoL = ProcessEAPoL( NULL, &staAssocMsg_p->staMACAddr); */
                    ProcessEAPoL( NULL, &staAssocMsg_p->staMACAddr);
                }
                else if ( pDistMsg->MsgType == TIMERMSGRECVD )
                {
                    distQ_TimerMsg_t *distTimerMsg_p = &pDistMsg->msg.distQ_TimerMsg;
                    keyMgmt_Timeout(distTimerMsg_p);
                }
                msi_wl_FreePktBuf((char *)pDistMsg);
            }
        }
#ifdef FREE_IN_BRIDGE
        ProcessTxDone();
        msi_wl_Eth_free_tx_queue (&eth_dev, 0);
        EtherEventSentFlag &= ~(0x8);
        GT_REG_WRITE (ETHER0_INT_CAUSE_REG + ETH_ADDR_GAP * ((ETHERNET_PRIV *)eth_dev.PrivData)->port ,
                      ~(unsigned int)(0xc));
#endif
        if(!noPkts)
        {
            if(count++ > 1000)
            {
                count = 0;
                apctlSwLedDisable(GPIO_WLAN_TX);
                apctlSwLedDisable(GPIO_WLAN_RX);
                apctlSwLedDisable(GPIO_LAN_TX );
                apctlSwLedDisable(GPIO_LAN_RX );
            }
        }
    }
#endif

    /*while ( 1 )*/
    {
#ifndef BRIDGE_POLLING
        EventsTriggered =  apio_select();
        /*------------------------------*/ 
        /* Check for an error condition */ 
        /*------------------------------*/ 
        if ( EventsTriggered == 0 )
        {
            /* Do some error handling */
        }
        else
        {
            /*-------------------------------------------------*/ 
            /* Check for data on the 802.11 Data */ 
            /*-------------------------------------------------*/
            if ( EventsTriggered & APIO_POLL_RADIO_DATA_RDY )
            {
                apctlSwLedEnable(GPIO_WLAN_RX);
#endif                
                noPkts = (*ProcessWirelessDataFp)(NO_OF_PKTS);
                WL_WRITE_WORD(MIR_INTR_STATUS, ~DATA_BUF_RDY);

#ifndef BRIDGE_POLLING
                if ( noPkts >= NO_OF_PKTS )
                {
                    ds_SendEvent(APIO_POLL_RADIO_DATA_RDY);

                }

            }
            else
            {
                apctlSwLedDisable(GPIO_WLAN_RX);
            }
            if ( EventsTriggered & APIO_POLL_ETH_DATA_RDY )
            {
                apctlSwLedEnable(GPIO_LAN_RX);
#endif                
                noPkts = (*ProcessEtherDataFp)(NO_OF_PKTS);
#ifndef BRIDGE_POLLING                
                if ( noPkts >= NO_OF_PKTS )
                {
                    ds_SendEvent(APIO_POLL_ETH_DATA_RDY);
                }

            }
            else
            {
                apctlSwLedDisable(GPIO_LAN_RX);
            }
            /*-------------------------------------------------*/ 
            /* Check for data on local TCP/IP stack */ 
            /*-------------------------------------------------*/
            if ( EventsTriggered & APIO_POLL_PROT_STACK_DATA_RDY )
            {
#endif
#ifndef NO_ECOS_TCP_IP_STACK
                (*ProcessLlcDataFp)();
#endif /* ECOS_TCP_IP_STACK */
#ifndef BRIDGE_POLLING
            }
            if ( EventsTriggered & APIO_POLL_WPA_MSG )
#endif            
            {
                DistTaskMsg_t *pDistMsg;

                if ( (pDistMsg = (DistTaskMsg_t *)os_QueueReadNoBlock(sysinfo_WPA_Q)) != NULL )
                {
                    if ( pDistMsg->MsgType == STA_ASSOMSGRECVD )
                    {
                        StaAssocStateMsg_t *staAssocMsg_p;

                        staAssocMsg_p = &pDistMsg->msg.StaAssocStateMsg;
                        /* call key mgmt function */
                        /*pBufDescrEAPoL = ProcessEAPoL( NULL, &staAssocMsg_p->staMACAddr); */
                        ProcessEAPoL( NULL, &staAssocMsg_p->staMACAddr);
                    }
                    else if ( pDistMsg->MsgType == TIMERMSGRECVD )
                    {
                        distQ_TimerMsg_t *distTimerMsg_p = &pDistMsg->msg.distQ_TimerMsg;
                        keyMgmt_Timeout(distTimerMsg_p);
                    }
                    msi_wl_FreePktBuf((char *)pDistMsg);
                }
            }
#ifndef BRIDGE_POLLING
        } 
#endif
    }
    }
} 

/****************************************************************************** 
 * 
 * Name: ProcessWirelessData
 * 
 * Description: 

 * 
 * Conditions For Use: 
 *   None
 * 
 * Arguments: 
 *   Arg1 (i  ): 
 * 
 * Return Value: 
 *   None. 
 * 
 * Notes: 
 *   None. 
 * 
 *****************************************************************************/ 
UINT32 ProcessWirelessData(int pkts)
{

    ether_hdr_t *EtherHdr_p;

    UINT32 Len;

    UINT32 noPkts;
    UINT8 RepeatId[6];
    UINT8 noPkts2eth = 0;
    apio_bufdescr_t BufDescr;

    noPkts = 0;

    while ( apio_wlanRead(&BufDescr) )
    {
        EtherHdr_p = (ether_hdr_t *)BufDescr.framePtr;
        Len = BufDescr.frameLen;
        if ( IS_GROUP((UINT8 *)&EtherHdr_p->da ) )
        {
#ifndef NO_ECOS_TCP_IP_STACK
            /* send the pkt to protocol stack */
            /* rcvdData is used by the w8100_eth_x functions */
            SendToStack(EtherHdr_p, Len);
#endif /* ECOS_TCP_IP_STACK */
            /* send the packet over the ethernet */
            /*apio_write(etherHandle,&BufDescr);*/
            noPkts2eth++;
            dsPort_Write(&BufDescr);
        }

        else if ( !memcmp(&EtherHdr_p->da , mib_currentmacaddress,
                          sizeof(IEEEtypes_MacAddr_t)) )
        {
            if ( EtherHdr_p->type == 0x8E88 )
            {/*EAPoL packet */
                ProcessEAPoL(&BufDescr, &EtherHdr_p->sa);
            }
            else
            {
#ifndef NO_ECOS_TCP_IP_STACK
                SendToStack(EtherHdr_p, Len);
#endif /* ECOS_TCP_IP_STACK */
            }
            /* free the packet */
            msi_wl_FreePktBuf((&BufDescr)->framePtr);

        }
        else
        {

            /* send the packet over the ethernet */
            noPkts2eth++;
            dsPort_Write(&BufDescr);
        }
        if ( ++noPkts >= pkts )  break;

    } /* End While GetWireless Data */
    if ( noPkts2eth )
    {
        apctlSwLedEnable(GPIO_LAN_TX);
    }
    return(noPkts);
}

#ifdef WDS_FEATURE
UINT32 ProcessWirelessDataWds(int pkts)
{

    ether_hdr_t *EtherHdr_p;

    UINT32 Len;

    UINT32 noPkts;
    UINT32 noPkts2eth= 0;
    UINT32 noPkts2wlan = 0;
    apio_bufdescr_t BufDescr;

    noPkts = 0;
    while ( apio_wlanRead(&BufDescr) )
    {
        EtherHdr_p = (ether_hdr_t *)BufDescr.framePtr;
        Len = BufDescr.frameLen;
        if ( EtherHdr_p->type == 0x8E88 )
        {/*EAPoL packet */
            ProcessEAPoL(&BufDescr, &EtherHdr_p->sa);
            /* free the packet */
            msi_wl_FreePktBuf((&BufDescr)->framePtr);
            break;
        }
        if(BufDescr.reserved)
        {
            noPkts2wlan++;
            DSForwardPacket(WDS_Port_CB[BufDescr.reserved & 0x7f].wdsxHandle,&BufDescr);
        }
        else
        {
            noPkts2eth++;
            DSForwardPacket(wlanHandle,&BufDescr);
        }
        if ( ++noPkts >= pkts )  break;
    }
    if ( noPkts2eth )
    {
        apctlSwLedEnable(GPIO_LAN_TX);
    }
    return(noPkts);
}
#endif
/****************************************************************************** 
 * 
 * Name: ProcessEtherData
 * 
 * Description: 

 * 
 * Conditions For Use: 
 *   None
 * 
 * Arguments: 
 *   Arg1 (i  ): 
 * 
 * Return Value: 
 *   None. 
 * 
 * Notes: 
 *   None. 
 * 
 *****************************************************************************/ 
UINT32 ProcessEtherData(int pkts)
{
    IEEEtypes_8023_Frame_t  *DataEtherFrame_p; 
    BOOLEAN BcastFlag;
    UINT32 Len;
    UINT32 noPkts;
    UINT16 ethType;
    UINT32 noPkts2wlan = 0;
    apio_bufdescr_t wBufDescr,rBufDescr;

    /* Retrieve data from the receive buffers one by one
     * and process them. The Frame has ethernet header and the
     * destination address is checked to see how the frame has
     * to be routed */
    noPkts = 0;
    /* retrieve the frame */
    while ( dsPort_Read(&rBufDescr) )
    {
        DataEtherFrame_p = (IEEEtypes_8023_Frame_t*)rBufDescr.framePtr;
        Len = rBufDescr.frameLen;
        BcastFlag = FALSE;
        if ( (ethType=(DataEtherFrame_p->Hdr.type << 8) + (DataEtherFrame_p->Hdr.type >> 8))
             == DBG_DEFAULT_TYPE )
        {
            etherBugRecv((UINT8 *)DataEtherFrame_p, Len);
        }

        if ( IS_GROUP((UINT8 *)&DataEtherFrame_p->Hdr.da) )
        {
            BcastFlag = TRUE;
            Check_ArpRequest_for_Handover(DataEtherFrame_p);
        }

        if ( (!memcmp(DataEtherFrame_p->Hdr.da, mib_currentmacaddress,
                      sizeof(IEEEtypes_MacAddr_t))) || BcastFlag )
        {
#ifndef NO_ECOS_TCP_IP_STACK
            /* send the pkt to protocol stack */
            /* rcvdData is used by the w8100_eth_x functions */
            SendToStack(DataEtherFrame_p, Len);
#endif /* ECOS_TCP_IP_STACK */
            if ( !BcastFlag )
            {
                wBufDescr.frameLen = Len;
                wBufDescr.framePtr = (UINT8 *)DataEtherFrame_p;
                msi_wl_FreePktBuf((&wBufDescr)->framePtr);
                if ( ++noPkts >= pkts )  break;
                continue;
            }
        }
        wBufDescr.frameLen = Len;
        wBufDescr.framePtr = (UINT8*)DataEtherFrame_p;
        noPkts2wlan++;
        apio_wlanWrite(&wBufDescr);
        if ( ++noPkts >= pkts )  break;
    }
    if ( noPkts2wlan )
    {
        apctlSwLedEnable(GPIO_WLAN_TX);
    }
    return(noPkts);
}

#ifdef WDS_FEATURE
UINT32 ProcessEtherDataWds(int pkts)
{
    IEEEtypes_8023_Frame_t  *DataEtherFrame_p; 
    BOOLEAN BcastFlag;
    UINT32 Len;
    UINT32 noPkts;
    UINT16 ethType;

    apio_bufdescr_t wBufDescr,rBufDescr;

    /* Retrieve data from the receive buffers one by one
     * and process them. The Frame has ethernet header and the
     * destination address is checked to see how the frame has
     * to be routed */
    noPkts = 0;
    /* retrieve the frame */
    while ( dsPort_Read(&rBufDescr) )
    {
        DataEtherFrame_p = (IEEEtypes_8023_Frame_t*)rBufDescr.framePtr;
        Len = rBufDescr.frameLen;
        BcastFlag = FALSE;
        if ( (ethType=(DataEtherFrame_p->Hdr.type << 8) + (DataEtherFrame_p->Hdr.type >> 8))
             == DBG_DEFAULT_TYPE )
        {
            etherBugRecv((UINT8 *)DataEtherFrame_p, Len);
        }
#if 0
        if ( IS_GROUP((UINT8 *)&DataEtherFrame_p->Hdr.da) )
        {
            BcastFlag = TRUE;
            Check_ArpRequest_for_Handover(DataEtherFrame_p);
        }
#endif
        DSForwardPacket(etherHandle,&rBufDescr);
        if ( ++noPkts >= pkts )  break;

    } /* End While GetWireless Data */
    if ( noPkts )
    {
        apctlSwLedEnable(GPIO_LAN_TX);
    }
    return(noPkts);
}
#endif // WDS_FEATURE
