/*
*                Copyright 2003, Marvell Semiconductor, Inc.
* This code contains confidential information of Marvell Semiconductor, Inc.
* No rights are granted herein under any patent, mask work right or copyright
* of Marvell or any third party.
* Marvell reserves the right at its sole discretion to request that this code
* be immediately returned to Marvell. This code is provided "as is".
* Marvell makes no warranties, express, implied or otherwise, regarding its
* accuracy, completeness or performance.
*/

/*!
 * \file    isr.c
 * \brief   eCos interrupt service routines
 *
 * $Revision: 1.27 $
 * $Date: 2004/05/18 19:24:04 $
 */

/*============================================================================= */
/*                               INCLUDE FILES */
/*============================================================================= */
#include "wltypes.h"
#include "buildModes.h"
#include "IEEE_types.h"
#include "osif.h"
#include "sysinfo.h"
#include "bcngen.h"
#include "tkip.h"
#include "macmgmtap.h"
#include "rx80211.h"
#include "tx80211.h"
#include "txsched.h"
#include "etherlow.h"
#include "ether.h"
#include "ds.h"
#include "oem.h"
#include "bufmgt.h"
#include "eth_debug.h"
#include "memmap.h"
#include "intr.h"
#include "wl_mib.h"
#include "wl_hal.h"
#include "idma.h"
#include "wl_macros.h"
#include "ap_api.h"
#include "stats.h"
#include "isr.h"

#ifdef BRIDGE_POLLING_1
extern UINT8 bridge_polling;
#endif
extern QHANDLE ethRxQ, ethTxQ, wlRxQ, wlTxQ;
extern void Reset(void);
#ifndef NO_ECOS_TCP_IP_STACK
extern void initiate_dhcp(int force);
#endif /* ECOS_TCP_IP_STACK */
extern void apEvtRun(UINT16 evtCode, UINT8 *body, UINT8 status);

#define ERRORS (TX_UNDERFLOW | TX_WDOG_TIMEOUT | RX_FRAG_ERR | RX_MEM_ERR \
	            | PTR_ERR | RX_OVERFLOW)

#define FEMAC_INTR    9
#define WLMAC_INTR    10
#define IDMA0_INTR    15 


/* Ethernet Interrupt cause register bit defines. */

#define ETH_CAUSE_RX_BUFFER_RETURN  (1<<0)  /* Rx Buffer Return             */
#define ETH_CAUSE_RSRVD_L01         (1<<1)
#define ETH_CAUSE_TX_BUFFER_HIGH    (1<<2)  /* Tx Buffer for High prio Q    */
#define ETH_CAUSE_TX_BUFFER_LOW     (1<<3)  /* Tx Buffer for low  prio Q    */
#define ETH_CAUSE_RSRVD_L04         (1<<4)
#define ETH_CAUSE_RSRVD_L05         (1<<5)
#define ETH_CAUSE_TX_END_HIGH       (1<<6)  /* Tx End for High Prio Q       */
#define ETH_CAUSE_TX_END_LOW        (1<<7)  /* Tx End for Low  Prio Q       */
#define ETH_CAUSE_RX_ERROR          (1<<8)  /* Rx Resource Err              */ 
#define ETH_CAUSE_RSRVD_L09         (1<<9)
#define ETH_CAUSE_TX_ERROR_HIGH     (1<<10) /* Tx Resource Err for High Prio Q */
#define ETH_CAUSE_TX_ERROR_LOW      (1<<11) /* Tx Resource Err for Low Prio Q  */
#define ETH_CAUSE_RX_OVERRUN        (1<<12) /* Rx Overrun                   */
#define ETH_CAUSE_TX_UNDERRUN       (1<<13) /* Tx Underrun                  */
#define ETH_CAUSE_RSRVD_L14         (1<<14)
#define ETH_CAUSE_RSRVD_L15         (1<<15)
#define ETH_CAUSE_RX_BUFFER_QUEUE_0 (1<<16) /* Rx Buffer Ret in Prio Q[0]   */
#define ETH_CAUSE_RX_BUFFER_QUEUE_1 (1<<17) /* Rx Buffer Ret in Prio Q[1]   */
#define ETH_CAUSE_RX_BUFFER_QUEUE_2 (1<<18) /* Rx Buffer Ret in Prio Q[2]   */
#define ETH_CAUSE_RX_BUFFER_QUEUE_3 (1<<19) /* Rx Buffer Ret in Prio Q[3]   */
#define ETH_CAUSE_RX_ERROR_QUEUE_0  (1<<20) /* Rx Resource Err in Prio Q[0] */
#define ETH_CAUSE_RX_ERROR_QUEUE_1  (1<<21) /* Rx Resource Err in Prio Q[1] */
#define ETH_CAUSE_RX_ERROR_QUEUE_2  (1<<22) /* Rx Resource Err in Prio Q[2] */
#define ETH_CAUSE_RX_ERROR_QUEUE_3  (1<<23) /* Rx Resource Err in Prio Q[3] */
#define ETH_CAUSE_RSRVD_L24         (1<<24)
#define ETH_CAUSE_RSRVD_L25         (1<<25)
#define ETH_CAUSE_RSRVD_L26         (1<<26)
#define ETH_CAUSE_RSRVD_L27         (1<<27)
#define ETH_CAUSE_MII_PHY_STC       (1<<28) /* MII PHY Status Change */
#define ETH_CAUSE_SMI_DONE          (1<<29) /* SMI Command Done      */
#define ETH_CAUSE_RSRVD_L30         (1<<30)
#define ETH_CAUSE_INT_SUM           (1<<31) /* Ethernet Interrupt Summary */

/*============================================================================= */
/*                              TYPE DEFINITIONS */
/*============================================================================= */
typedef struct Isr_t
{
#ifdef USE_NEW_OSIF
    INTERRUPT_HANDLE handle;
#endif
    UINT32 (*isrFunc_p)();
    void (*dsrFunc_p)();
}
Isr_t;

/*============================================================================= */
/*                   PRIVATE PROCEDURES (ANSI Prototypes) */
/*============================================================================= */
#ifdef ENABLE_GPIO_INT
UINT32 isr_gpio(UINT32, UINT32);
void dsr_gpio(UINT32, UINT32, UINT32);
#endif

UINT32 wlandsrcnt = {0};
UINT8 resetmode = 0;
/*============================================================================= */
/*                          MODULE LEVEL VARIABLES */
/*============================================================================= */

unsigned *irqvec = (unsigned *)0x18;
void PwrModeChangeIsr(void);
extern void RxEtherIsr(void);
extern void TxEtherIsr(void);

struct Debug
{
    UINT32 Tsf_lo;
    UINT32 Tsf_hi;
    UINT32 IsrStatus;
    UINT32 Rsrv;
}
DebugBuf[100] ; /*__attribute__ ((section (".sqbuf"))); */

volatile UINT8 dma0_busy = 0;
volatile UINT8 dma1_busy = 0;
volatile UINT32 txwep_busy = 0;
/*UINT32 EventSentFlag = 0; */
volatile UINT32 EtherEventSentFlag = 0;

UINT8 stateCnt = 0;
UINT32 TxState[5] ;
UINT32 watchdogCount;
#ifdef DEBUG
UINT32 IsrCallCnt[32];
#endif
extern UINT32 staFreeze;

UINT8 ether_port_down = 1;

/*============================================================================= */
/*                         CODED PUBLIC PROCEDURES */
/*============================================================================= */


UINT32 isr_wlMac(UINT32 vector, UINT32 data)
{
    /* */
    /* Disable WLMAC interrupt */
    /* */
    WL_WRITE_WORD(WL_INTR_CLEAR, INTR_WLMACIRQ);


    /* Cause DSR to be run */
    return(CYG_ISR_CALL_DSR);
}

/******************************************************************************
 *
 * Name: dsr__wlmac
 *
 * Description:
 *   This routine is to handle wireless mac interrupts
 *
 * Conditions For Use:
 *
 * Arguments:
 *   Arg1 (i  ): data - not used
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *****************************************************************************/
static void dsr_wlMac(UINT32 vector, UINT32 cnt, UINT32 data)
{
    UINT8 rx_error = 0;
    UINT32 val;
#ifdef ENABLE_WDS
    static int tbttflag = 0;
    static   int curtbttvalue,prevtbttvalue ,nexttbttvalue=0;
#endif
#ifdef DEBUG

    wlandsrcnt++;
    IsrCallCnt[WLMAC_INTR]++;
#endif

    /* Read the interrupt status, this also mask the unit level interrupt */
    WL_READ_WORD(MIR_INTR_STATUS, val);

    /* clearing the status bit are doen individually, after doing
     * required processing */
#if 0
    if ( val & ( TX_DONE | TXWEP_DONE) )
    {
        if ( val & TX_DONE )
        {
            /*msi_wl_PktXmtd(wlTxQ, 0 frame->wlanTxHdr.QosControl.QosControl, 1); */
#ifdef USE_NEW_OSIF
            os_EventTrigger(tx11EH, tx80211_FRAME_TX_DONE);
#else
            os_EventTrigger(sysinfo_802_11_TX_Q_EVENT, tx80211_FRAME_TX_DONE);
#endif

        }
        if ( val&TXWEP_DONE )
        {
            txwep_busy = 0;
        }
    }
#endif //--Rahul

    if ( val & TX_DONE )
    {
        /*msi_wl_PktXmtd(wlTxQ, 0 frame->wlanTxHdr.QosControl.QosControl, 1); */
        //wuming for linker 
        os_EventTrigger(sysinfo_802_11_TX_Q_EVENT, tx80211_FRAME_TX_DONE);
#ifdef FREE_IN_BRIDGE
        if(!bridge_polling)
#endif
#ifdef USE_NEW_OSIF
        os_EventTrigger(tx11EH, tx80211_FRAME_TX_DONE);
#else
        os_EventTrigger(sysinfo_802_11_TX_Q_EVENT, tx80211_FRAME_TX_DONE);
#endif

    }

    if ( val & (MGMT_BUF_RDY | CTRL_MSG_RDY) )
    {
#ifdef USE_PKT_Q
        wlan_rx_pkts(QUEUE_MLME, 1);
#else

        /*if ( val & MGMT_BUF_RDY )
            WL_WRITE_WORD(MIR_INTR_STATUS, ~MGMT_BUF_RDY);
        else
            WL_WRITE_WORD(MIR_INTR_STATUS, ~CTRL_MSG_RDY);*/
#ifdef USE_NEW_OSIF
        os_EventTrigger(mgmtEH, macMgmtMain_802_11_MGMT_MSG_RCVD);
#else
        os_EventTrigger(sysinfo_MAC_MGMT_MAIN_Q_EVENT, macMgmtMain_802_11_MGMT_MSG_RCVD);
#endif
#endif

    }
    if ( val & DATA_BUF_RDY )
    {
#ifdef USE_PKT_Q
        if ( wlan_rx_pkts(QUEUE_DATA_0, 16) )
        {
            WL_WRITE_WORD(MIR_INTR_STATUS, ~DATA_BUF_RDY);
        }
        else
        {
            wlan_data_ready(wlRxQ, QUEUE_DATA_0, 16);
        }
#else
#ifdef USE_NEW_OSIF
        os_EventTrigger(dsEH, ds_802_11_DATA_RCVD);
#else
        os_EventTrigger(sysinfo_DS_EVENT, ds_802_11_DATA_RCVD);
#endif
#endif
    }
#if 0 //Rahul
    if ( val & ( CFP_END | CFP_START | BCN_FREE) )
    {
        if ( val & CFP_END )
        {
#ifdef USE_NEW_OSIF
            os_EventTrigger(tx11EH, tx80211_CFP_END_RCVD);
#else
            os_EventTrigger(sysinfo_802_11_TX_Q_EVENT, tx80211_CFP_END_RCVD);
#endif
        }
        if ( val & CFP_START )
        {
            WL_WRITE_WORD(MIR_INTR_STATUS, ~CFP_START);
        }
        if ( val & BCN_FREE )
        {
            bcngen_BeaconFreeIsr();
#ifdef ENABLE_WDS
            if ( !WL_REGS32(MTD_BCNTX_CNT) && tbttflag )
            {

                /* 10% is reduced to account for the processing delay ,
                 * the current bcn tx time is passed and the hw has skipped the bcn tx,
                 * and the sw tries to set the bcn tx as early as possible by decreasing the
                 * acc interval value by 95% of the bcn interval, which will result in 
                 * the bcn tx delayed by 5% of the bcn interval.
                 */
#if 1
                if ( WL_REGS32(TX_ACC_INTVL_HI) == WL_REGS32( TX_TSF_HI ) )
                {
                    nexttbttvalue = (WL_REGS32( TX_ACC_INTVL_LO)-
                                     ((WL_REGS32(TX_BCN_PERIOD)*1024) -(WL_REGS32(TX_BCN_PERIOD)*102)));
                    if ( nexttbttvalue  > WL_REGS32(TX_TSF_LO) )
                    {
                        WL_WRITE_WORD(TX_ACC_INTVL_LO ,nexttbttvalue);
                        WL_WRITE_WORD(TX_ACC_INTVL_HI, WL_REGS32(TX_ACC_INTVL_HI));
                        WL_WRITE_WORD(TX_TIMER_COMMIT,0x2);
                    }
                }
#else
                /* another way on doing this, not tested throughly */
                tsflo = WL_REGS32(TX_TSF_LO);
                tsfhi         = WL_REGS32(TX_TSF_HI);
                curtbttvalue =  tsflo & 0x80000000;
                nexttbttvalue =  tsflo + 5000;

                WL_WRITE_WORD(TX_ACC_INTVL_LO,nexttbttvalue );

                if ( curtbttvalue && !(nexttbttvalue & 0x80000000) )
                {
                    WL_WRITE_WORD(TX_ACC_INTVL_HI, tsfhi+1);

                }
                else
                {
                    WL_WRITE_WORD(TX_ACC_INTVL_HI, tsfhi);
                }
                WL_WRITE_WORD(TX_TIMER_COMMIT,0x2);
#endif


            }
            tbttflag = 0;

#endif
            WL_WRITE_WORD(MIR_INTR_STATUS, ~BCN_FREE);
        }

    }
#endif  //--Rahul

    if ( val & BCN_FREE )
    {
        bcngen_BeaconFreeIsr();
#ifdef ENABLE_WDS
        if ( !WL_REGS32(MTD_BCNTX_CNT) && tbttflag )
        {

            /* 10% is reduced to account for the processing delay ,
             * the current bcn tx time is passed and the hw has skipped the bcn tx,
             * and the sw tries to set the bcn tx as early as possible by decreasing the
             * acc interval value by 95% of the bcn interval, which will result in 
             * the bcn tx delayed by 5% of the bcn interval.
             */
#if 1
            if ( WL_REGS32(TX_ACC_INTVL_HI) == WL_REGS32( TX_TSF_HI ) )
            {
                nexttbttvalue = (WL_REGS32( TX_ACC_INTVL_LO)-
                                 ((WL_REGS32(TX_BCN_PERIOD)*1024) -(WL_REGS32(TX_BCN_PERIOD)*102)));
                if ( nexttbttvalue  > WL_REGS32(TX_TSF_LO) )
                {
                    WL_WRITE_WORD(TX_ACC_INTVL_LO ,nexttbttvalue);
                    WL_WRITE_WORD(TX_ACC_INTVL_HI, WL_REGS32(TX_ACC_INTVL_HI));
                    WL_WRITE_WORD(TX_TIMER_COMMIT,0x2);
                }
            }
#else
            /* another way on doing this, not tested throughly */
            tsflo = WL_REGS32(TX_TSF_LO);
            tsfhi         = WL_REGS32(TX_TSF_HI);
            curtbttvalue =  tsflo & 0x80000000;
            nexttbttvalue =  tsflo + 5000;

            WL_WRITE_WORD(TX_ACC_INTVL_LO,nexttbttvalue );

            if ( curtbttvalue && !(nexttbttvalue & 0x80000000) )
            {
                WL_WRITE_WORD(TX_ACC_INTVL_HI, tsfhi+1);

            }
            else
            {
                WL_WRITE_WORD(TX_ACC_INTVL_HI, tsfhi);
            }
            WL_WRITE_WORD(TX_TIMER_COMMIT,0x2);
#endif


        }
        tbttflag = 0;

#endif
        WL_WRITE_WORD(MIR_INTR_STATUS, ~BCN_FREE);
    }

#ifdef ENABLE_WDS
    if ( val & BCN_DATA_RDY )
    {
        tbttflag = 1;
        WL_WRITE_WORD(MIR_INTR_STATUS, ~BCN_DATA_RDY);
    }
#endif
#if 0
    if ( val & POWER_CHNG )
    {
        PwrModeChangeIsr();
    }
#endif

    if ( val & ERRORS )
    {
        Stats.MacErrCnt++;
        if ( val & RX_MEM_ERR )
        {
            rx_error++;
            Stats.MacRxMemErrCnt++;
            WL_WRITE_WORD(MIR_INTR_STATUS, ~RX_MEM_ERR);
        }
        if ( val & RX_OVERFLOW )
        {
            rx_error++;
            Stats.MacRxOverFlowCnt++;
            WL_WRITE_WORD(MIR_INTR_STATUS, ~RX_OVERFLOW );
        }
        if ( val & PTR_ERR )
        {
            rx_error++;
            Stats.MacPtrErrCnt++;
            WL_WRITE_WORD(MIR_INTR_STATUS, ~PTR_ERR);
        }
        if ( val & RX_FRAG_ERR )
        {
            rx_error++;
            Stats.MacRxFragErrCnt++;
            WL_WRITE_WORD(MIR_INTR_STATUS, ~RX_FRAG_ERR);
        }
        if (rx_error)
        {

#ifdef USE_NEW_OSIF
            os_EventTrigger(dsEH, ds_802_11_DATA_RCVD);
#else
            os_EventTrigger(sysinfo_DS_EVENT, ds_802_11_DATA_RCVD);
#endif
        }

        if ( val & TX_UNDERFLOW )
        {
            extern int MfgTestModeOn;
            WL_WRITE_WORD(MIR_INTR_STATUS, ~TX_UNDERFLOW);
            Stats.MacTxUnderFlowCnt++;
            switch (resetmode) {
            case 1:
                if ((!(WL_REGS32(MPP_FORCEPLL) & 0x8)) && (!MfgTestModeOn))
                {
                    msi_wl_DisableReceiver();
                    WlanEnabled = FALSE;
                    msi_wl_WriteBBP(0x03, 0xff);
                    WL_WRITE_WORD(MPP_FORCEPLL, 0x9);
                    SendResetCmd(1);
                }
                break;
            case 2:
                break;
            default:
                if ((!(WL_REGS32(MPP_FORCEPLL) & 0x8)) && (!MfgTestModeOn))
                {
                    msi_wl_WriteBBP(0x03, 0xff);
                    WL_WRITE_WORD(MPP_FORCEPLL, 0x9);
                    SendResetCmd(1);
                }
                break;

            }
        }

        if ( val & TX_WDOG_TIMEOUT )
        {
            extern int MfgTestModeOn;
            Stats.MacTxWatchDogTimeouts++;
#if 1                    
            switch (resetmode) {
            case 1:
                if ((!(WL_REGS32(MPP_FORCEPLL) & 0x8)) && (!MfgTestModeOn))
                {
                    msi_wl_DisableReceiver();
                    WlanEnabled = FALSE;
                    msi_wl_WriteBBP(0x03, 0xff);
                    WL_WRITE_WORD(MPP_FORCEPLL, 0x9);
                    SendResetCmd(1);
                }
                break;
            case 2:
                break;
            default:
                if ((!(WL_REGS32(MPP_FORCEPLL) & 0x8)) && (!MfgTestModeOn))
                {
                    msi_wl_WriteBBP(0x03, 0xff);
                    WL_WRITE_WORD(MPP_FORCEPLL, 0x9);
                    SendResetCmd(1);
                }
                break;

            }
#else
            /* Reset watchdog timer to 4 sec */
            WL_WRITE_WORD(MTD_TX_WATCHDOG, 0x11000);

            WL_WRITE_WORD(MIR_INTR_STATUS, ~TX_WDOG_TIMEOUT);
#endif
        }
    }  /* end if ERRORs */
    /* Re-enable WLMAC interrupt */
    WL_REGS32_SETBITS(WL_INTR_SET, INTR_WLMACIRQ);
}
/******************************************************************************
 *
 * Name: isr_feMac
 *
 * Description:
 *   This routine is to handle ethernet mac interrupts
 *
 * Conditions For Use:
 *
 * Arguments:
 *   Arg1 (i  ): data - not used
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *****************************************************************************/
UINT32 isr_feMac(UINT32 vector, UINT32 data)
{
    /* */
    /* Disable ETHER MAC interrupt */
    /* */
    WL_WRITE_WORD(WL_INTR_CLEAR, INTR_FEIRQ);


    /* Cause DSR to be run */
    return(CYG_ISR_CALL_DSR);
}
/******************************************************************************
 *
 * Name: dsr_feMac
 *
 * Description:
 *   This routine is to handle ether mac interrupts
 *
 * Conditions For Use:
 *
 * Arguments:
 *   Arg1 (i  ): data - not used
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *****************************************************************************/
void dsr_feMac(UINT32 vector, UINT32 cnt, UINT32 data)
{
    unsigned long eth_int_cause;
    ETHERNET_PRIV *ethernet_priv;
    NET_DEV *dev;

    dev = &eth_dev;
    ethernet_priv = dev->PrivData;

    /* interrupts are cleared only in the task, after processing packets
     * So, to avoid generating unnecessary events EtherEventSentFlag is used*/
    GT_REG_READ (ETHER0_INT_CAUSE_REG + ETH_ADDR_GAP * ethernet_priv->port , &eth_int_cause);
    if ( eth_int_cause & 0xc )
    {
        if ( eth_int_cause & 0x8 )
        {
            /* Free queue 0 , which is TxBufferHigh */
#if 1
            if ( !(EtherEventSentFlag & 0x8) )
            {
                EtherEventSentFlag |= 0x8;
                	
    #ifdef FREE_IN_TXTASK
    		os_EventTrigger(sysinfo_802_11_TX_Q_EVENT,tx80211_ETHER_TX_DONE);
        #ifdef FREE_IN_BRIDGE
                if(!bridge_polling)
        #endif
        #ifdef USE_NEW_OSIF
                os_EventTrigger(tx11EH,tx80211_ETHER_TX_DONE);
        #else
                os_EventTrigger(sysinfo_802_11_TX_Q_EVENT,tx80211_ETHER_TX_DONE);
        #endif
    #else
        #ifdef USE_NEW_OSIF
                os_EventTrigger(dsEH, ds_802_3_TX_DONE);
        #else
                os_EventTrigger(sysinfo_DS_EVENT, ds_802_3_TX_DONE);
        #endif
    #endif
            }
#else
            {
                extern NET_DEV eth_dev;

                ETHERNET_PRIV *ether_priv;
                ether_priv = (ETHERNET_PRIV *)eth_dev.PrivData;
                msi_wl_Eth_free_tx_queue (&eth_dev, 0);
                GT_REG_WRITE (ETHER0_INT_CAUSE_REG + ETH_ADDR_GAP * ether_priv->port ,
                              ~(unsigned int)(0xc));
            }
#endif
        }
    }

    if ( eth_int_cause & 0x101 )
    {
        /*RxBuffer , RxResource Error*/
        if ( !(EtherEventSentFlag & 0x100) )
        {
            EtherEventSentFlag |= 0x100;
#ifdef USE_NEW_OSIF
            os_EventTrigger(dsEH, ds_802_3_DATA_RCVD);
#else
            os_EventTrigger(sysinfo_DS_EVENT, ds_802_3_DATA_RCVD);
#endif
            if ( eth_int_cause & 0x100 )
            {
                Stats.EthRxResourceErrCnt++;
            }
        }
    }

    if ( eth_int_cause & 0x10000000 )
    {
        /* MII PHY status changed */
        unsigned int port_status;
        GT_REG_READ (ETHER0_PORT_STATUS_REG + ETH_ADDR_GAP * ethernet_priv->port , &port_status);
        /* Check Link status on Ethernet0 */
        if ( port_status & 0x8 )
        {
			#ifndef NO_ECOS_TCP_IP_STACK
            initiate_dhcp(1);
			#endif /* ECOS_TCP_IP_STACK */
#ifndef AP31E
#if !defined(AP21N) && !defined(AP21M)

            apctlGPIOWrite(10, 0);
#endif
            ether_port_down = 0;
        }
        else
        {
#if !defined(AP21N) && !defined(AP21M)
            apctlGPIOWrite(10, 1);
#endif
            ether_port_down = 1;
#endif
        }

        GT_REG_WRITE (ETHER0_INT_CAUSE_REG + ETH_ADDR_GAP * ethernet_priv->port , ~(unsigned int)(0x10000000));
    }

    GT_REG_WRITE (ETHER0_INT_CAUSE_REG  + ETH_ADDR_GAP * ethernet_priv->port , ~(unsigned int)(0x80000000));

    /* Re-enable FE MAC interrupt */
    WL_REGS32_SETBITS(WL_INTR_SET, INTR_FEIRQ);
}

/******************************************************************************
 *
 * Name: w81isr_host
 *
 * Description:
 *   This routine is to handle interrupts from host
 *
 * Conditions For Use:
 *
 * Arguments:
 *   Arg1 (i  ): data - not used
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 *****************************************************************************/
#ifdef AP_PCI
UINT32 isr_host(UINT32 vector, UINT32 data)
{
   // Disable host interrupt
   WL_WRITE_WORD(WL_INTR_CLEAR, INTR_HIIRQ);
   // Cause DSR to be run   
   return CYG_ISR_CALL_DSR;
}

/******************************************************************************
 *
 * Name: w81dsr_host
 *
 * Description:
 *   This routine performs post interrupts processing for host events
 *
 * Conditions For Use:
 *
 * Arguments:
 *   Arg1 (i  ): data - not used
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 *****************************************************************************/
void dsr_host(UINT32 vector, UINT32 cnt, UINT32 data)
{
   hwi_pci_isr();
   // Re-enable the interrupt
   WL_REGS32_SETBITS(WL_INTR_SET, INTR_HIIRQ);
}
#endif
/******************************************************************************
 *
 * Name: isr_iDma0
 *
 * Description:
 *   This routine performs iDAM interrupts post interrupts processing
 *
 * Conditions For Use:
 *
 * Arguments:
 *   Arg1 (i  ): data - not used
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 *****************************************************************************/
UINT32 isr_iDma0(UINT32 vector, UINT32 data)
{

    /* Disable iDMA0 interrupt */
    WL_WRITE_WORD(WL_INTR_CLEAR, INTR_DMA0IRQ);


    /* Cause DSR to be run */
    return(CYG_ISR_CALL_DSR);
}

/******************************************************************************
 *
 * Name: dsr_iDma0
 *
 * Description:
 *   This routine is to handle iDAM interrupts
 *
 * Conditions For Use:
 *
 * Arguments:
 *   Arg1 (i  ): data - not used
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 *****************************************************************************/
void dsr_iDma0(UINT32 vector, UINT32 cnt, UINT32 data)
{
    UINT32 val;
#ifdef DEBUG

    IsrCallCnt[IDMA0_INTR]++;
#endif

    val = WL_REGS32(WL_IDMA_ISR0);
    WL_REGS32(WL_IDMA_CH0_CTRL) = 0x0;
    /*write to clear & enable interrupt */
    WL_REGS32(WL_IDMA_ISR0) = 0;

    dma0_busy = 0;



    /* Re-enable iDMA0 interrupt */
    WL_REGS32_SETBITS(WL_INTR_SET, INTR_DMA0IRQ);
}

/******************************************************************************
 *
 * Name: isr_iDma1
 *
 * Description:
 *   This routine performs iDAM interrupts post interrupts processing
 *
 * Conditions For Use:
 *
 * Arguments:
 *   Arg1 (i  ): data - not used
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 *****************************************************************************/
UINT32 isr_iDma1(UINT32 vector, UINT32 data)
{


    /* Disable iDMA0 interrupt */
    WL_WRITE_WORD(WL_INTR_CLEAR, INTR_DMA1IRQ);

    /* Cause DSR to be run */
    return(CYG_ISR_CALL_DSR);
}

/******************************************************************************
 *
 * Name: dsr_iDma1
 *
 * Description:
 *   This routine is to handle iDAM interrupts
 *
 * Conditions For Use:
 *
 * Arguments:
 *   Arg1 (i  ): data - not used
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 *****************************************************************************/
void dsr_iDma1(UINT32 vector, UINT32 cnt, UINT32 data)
{
    UINT32 val;

    val = WL_REGS32(WL_IDMA_ISR1);
    WL_REGS32(WL_IDMA_CH1_CTRL) = 0x0;

    /*write to clear & enable interrupt */
    WL_REGS32(WL_IDMA_ISR1) = 0;

    dma1_busy = 0;


    /* Re-enable iDMA0 interrupt */
    WL_REGS32_SETBITS(WL_INTR_SET, INTR_DMA1IRQ);
}
/******************************************************************************
 *
 * Name: isr_Timer1
 *
 * Description:
 *   
 *
 * Conditions For Use:
 *
 * Arguments:
 *   Arg1 (i  ): data - not used
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 *****************************************************************************/
UINT32 isr_timer1(UINT32 vector, UINT32 data)
{


    /* Disable Timer interrupt */
    WL_WRITE_WORD(WL_INTR_CLEAR, INTR_TIMER1);
    ds_SendEvent(ds_POLLING_TIMEOUT_RCVD);

    /* Cause DSR to be run */
    return(CYG_ISR_CALL_DSR);
}
/******************************************************************************
 *
 * Name: dsr_Timer1
 *
 * Description:
 *   This routine is to handle Timer interrupt
 *
 * Conditions For Use:
 *
 * Arguments:
 *   Arg1 (i  ): data - not used
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 *****************************************************************************/
void dsr_timer1(UINT32 vector, UINT32 cnt, UINT32 data)
{

    /* Re-enable Timer interrupt */
    /*   WL_REGS32_SETBITS(WL_INTR_SET, INTR_TIMER1); */
}
/******************************************************************************
 *
 * Name: isr_Timer2
 *
 * Description:
 *   
 *
 * Conditions For Use:
 *
 * Arguments:
 *   Arg1 (i  ): data - not used
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 *****************************************************************************/
UINT32 isr_timer2(UINT32 vector, UINT32 data)
{


    /* Disable Timer interrupt */
    WL_WRITE_WORD(WL_INTR_CLEAR, INTR_TIMER2);
    ds_SendEvent(ds_POLLING_TIMEOUT_RCVD);

    /* Cause DSR to be run */
    return(CYG_ISR_CALL_DSR);
}
/******************************************************************************
 *
 * Name: dsr_Timer2
 *
 * Description:
 *   This routine is to handle Timer interrupt
 *
 * Conditions For Use:
 *
 * Arguments:
 *   Arg1 (i  ): data - not used
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 *****************************************************************************/
void dsr_timer2(UINT32 vector, UINT32 cnt, UINT32 data)
{




    /* Re-enable Timer interrupt */
    /*   WL_REGS32_SETBITS(WL_INTR_SET, INTR_TIMER2); */
}
/******************************************************************************
 *
 * Name: isr_Timer3
 *
 * Description:
 *   
 *
 * Conditions For Use:
 *
 * Arguments:
 *   Arg1 (i  ): data - not used
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 *****************************************************************************/
UINT32 isr_timer3(UINT32 vector, UINT32 data)
{


    /* Disable Timer interrupt */
    WL_WRITE_WORD(WL_INTR_CLEAR, INTR_TIMER3);



    return(CYG_ISR_CALL_DSR);
}
/******************************************************************************
 *
 * Name: dsr_Timer3
 *
 * Description:
 *   This routine is to handle Timer interrupt
 *
 * Conditions For Use:
 *
 * Arguments:
 *   Arg1 (i  ): data - not used
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 *****************************************************************************/
void dsr_timer3(UINT32 vector, UINT32 cnt, UINT32 data)
{


    ds_SendEvent(ds_POLLING_TIMEOUT_RCVD);

    /* Re-enable Timer interrupt */
    WL_REGS32_SETBITS(WL_INTR_SET, INTR_TIMER3);
}
/******************************************************************************
 *
 * Name: isr_Timer4
 *
 * Description:
 *   
 *
 * Conditions For Use:
 *
 * Arguments:
 *   Arg1 (i  ): data - not used
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 *****************************************************************************/
UINT32 isr_timer4(UINT32 vector, UINT32 data)
{


    /* Disable Timer interrupt */
    WL_WRITE_WORD(WL_INTR_CLEAR, INTR_TIMER4);
    ds_SendEvent(ds_POLLING_TIMEOUT_RCVD);

    /* Cause DSR to be run */
    return(CYG_ISR_CALL_DSR);
}
/******************************************************************************
 *
 * Name: dsr_Timer4
 *
 * Description:
 *   This routine is to handle Timer interrupt
 *
 * Conditions For Use:
 *
 * Arguments:
 *   Arg1 (i  ): data - not used
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 *****************************************************************************/
void dsr_timer4(UINT32 vector, UINT32 cnt, UINT32 data)
{
    /* Re-enable Timer interrupt */
    /*   WL_REGS32_SETBITS(WL_INTR_SET, INTR_TIMER4); */
}
#ifdef ENABLE_GPIO_INT

    #define GPIO_REG(offset) (WL_LBU_GPU_CFG_BASE | offset)

    #define GPIO_REG_SELECT_0_7     WL_LBU_GPU_CFG_BASE
    #define GPIO_REG_SELECT_8_15    GPIO_REG(0x04)
    #define GPIO_REG_OUTPUT_ENABLE  GPIO_REG(0x08)
    #define GPIO_REG_OUTPUTS        GPIO_REG(0x0C)
    #define GPIO_REG_INPUTS         GPIO_REG(0x10)
    #define GPIO_REG_IER			GPIO_REG(0x14)
    #define GPIO_REG_IMR			GPIO_REG(0x18)
    #define GPIO_REG_RSR            GPIO_REG(0x1c)
    #define GPIO_REG_ISR			GPIO_REG(0x20)

UINT32 isr_gpio(UINT32 vector, UINT32 data)
{
    UINT16 val;
    WL_READ_WORD(GPIO_REG_ISR, val);
    WL_WRITE_WORD(WL_INTR_CLEAR, INTR_GPIO);

    WL_WRITE_WORD(GPIO_REG_RSR, (GPIO_PIN_SW_RESET)<<3);
    WL_WRITE_WORD(GPIO_REG_ISR, 0);

    WL_REGS32_SETBITS(WL_INTR_SET, INTR_GPIO);
    return(CYG_ISR_CALL_DSR);
}

void dsr_gpio(UINT32 vector, UINT32 cnt, UINT32 data)
{
    UINT16 val;
    WL_READ_WORD(GPIO_REG_INPUTS, val);
    if ( val & GPIO_PIN_SW_RESET )
        WL_WRITE_WORD(GPIO_REG_IER, (GPIO_PIN_SW_RESET)<<3);
    else
        WL_WRITE_WORD(GPIO_REG_IER, 0);
    apEvtRun(APEVT_RST_BTN_PRESSED, 0, 0);
}
#endif

UINT32 isr_default0(UINT32 vector, UINT32 cnt)
{
    return(CYG_ISR_HANDLED); /*Spurious interrupt */

}


/*
 * Put this arrya at the end of the file so that we do not have to
 * declare a lot of function prototypes
 */

static Isr_t IsrTable[18] =
{
#ifdef USE_NEW_OSIF
    {0, isr_default0, NULL},
    {0, NULL, NULL},
    {0, NULL, NULL},
    {0, NULL, NULL},

    {0, NULL, NULL},
    {0, NULL, NULL},  /*OS Timer Tick */
    {0, isr_timer3, dsr_timer3},
    {0, NULL, NULL},

    {0, NULL, NULL},
    {0, isr_feMac, dsr_feMac},
    {0, isr_wlMac, dsr_wlMac},
    {0, NULL, NULL},

#ifdef ENABLE_GPIO_INT
    {0, isr_gpio, dsr_gpio},
#else
    {0, NULL, NULL},
#endif
    {0, NULL, NULL},
    {0, NULL, NULL},
    {0, isr_iDma0, dsr_iDma0},

    {0, isr_iDma1, dsr_iDma1},
    {0, NULL, NULL}
#else
    {isr_default0, NULL},
    {NULL, NULL},
    {NULL, NULL},
    {NULL, NULL},

    {NULL, NULL},
    {NULL, NULL},  /*OS Timer Tick */
    {isr_timer3, dsr_timer3},
    {NULL, NULL},
#ifdef AP_PCI
    {isr_host, dsr_host},
#else
    {NULL, NULL},
#endif
    {isr_feMac, dsr_feMac},
    {isr_wlMac, dsr_wlMac},
    {NULL, NULL},

#ifdef ENABLE_GPIO_INT
    {isr_gpio, dsr_gpio},
#else
    {NULL, NULL},
#endif
    {NULL, NULL},
    {NULL, NULL},
    {isr_iDma0, dsr_iDma0},

    {isr_iDma1, dsr_iDma1},
    {NULL, NULL}
#endif
};

/******************************************************************************
 *
 * Name: IsrCreate
 *
 * Description:
 *   This routine is called to created ISR objects
 *
 * Conditions For Use:
 *   
 * Arguments:
 *   None.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *    If there is a ISR Then 
 *       Call os_InterruptCreate to create ISR object
 *    End If
 * END PDL
 *
 *****************************************************************************/
void IsrCreate(void)
{
    UINT32 i, j = 0;

    for ( i = 0; i < 16; i++ )
    {
        if ( IsrTable[i].isrFunc_p != 0 )
        {
#ifdef USE_NEW_OSIF
            os_InterruptCreate(&IsrTable[i].handle,
#else
            os_InterruptCreate(j,
#endif
                               i,
                               j,
                               (void *)IsrTable[i].isrFunc_p,
                               (void *)IsrTable[i].dsrFunc_p);
            j++;
        }
    }
}

/******************************************************************************
 *
 * Name: IsrAttach
 *
 * Description:
 *   This routine provides the mechanism to trigger an event indicating to
 *
 * Conditions For Use:
 *   The task and its associated queues has been initialized by calling
 *   macMgmtMain_Init().
 *
 * Arguments:
 *   Arg1 (i  ): Event - value indicating which queue has received a message
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 *    If Event is one of the defined events for this task Then 
 *       Call os_EventTrigger to trigger the received event
 *       Return OS_SUCCESS
 *    Else
 *       Return OS_FAIL
 *    End If
 * END PDL
 *
 *****************************************************************************/
void IsrAttach(void)
{
    UINT32 i, j = 0;

    for ( i = 0; i < 16; i++ )
    {
        if ( IsrTable[i].isrFunc_p != 0 )
        {
#ifdef USE_NEW_OSIF
            os_InterruptAttach(IsrTable[i].handle);
#else
            os_InterruptAttach(j);
#endif
            j++;
        }
    }
}

