/******************** (c) Marvell Semiconductor, Inc., 2001 *******************
*
* $Header: E:\\APCVSDEPOT/wlan/prod/ap/ap31/bridge/src/mib.c,v 1.19 2004/05/12 19:55:42 jshen Exp $
*
* Purpose:
*    This file provides declarations and initialization of MIB variables.
*
* Public Procedures:
*    None.
*
* Private Procedures:
*    None.
*
* Notes:
*    None.
*
*****************************************************************************/
#include "wltypes.h"
#include "buildModes.h"
#include "IEEE_types.h"
#include "timer.h"
#include "wl_mib.h"
#include "wl_hal.h"
#include "ap_mib.h"
#include "util.h"
#include "api2.h"
#include "ap_api.h"
#include "oem.h"
#include "apflash.h"

#ifdef ENABLE_WEP

    #define WEP_ENABLE      TRUE
    #define WEP_TYPE        AUTH_SHARED_KEY
    #define MIB_WEP         TRUE
    #define PRIV_OPTION     TRUE
    #define PRIV_INVOKED    TRUE

#else

    #define WEP_ENABLE      FALSE
    #define WEP_TYPE        AUTH_OPEN_SYSTEM
    #define MIB_WEP         FALSE
    #define PRIV_OPTION     FALSE
    #define PRIV_INVOKED    FALSE

#endif

extern void patch_devName(void);


/* Imported functions */
extern void apfdGetEtherAddr(unsigned char *etherAddr);


MIB_802DOT11 *mibSystem_p;

MIB_DHCP      mib_DHCP;

#ifdef OS_VxW
MIB_DHCP      mib_DHCP_Eth;
MIB_DHCP      mib_DHCP_WAN;
#endif

unsigned char enaddr[] = {0x00, 0x50, 0x43, 0x02, 0x00, 0xf0};

MIB_STA_CFG                *mib_StaCfg_p;
MIB_OP_DATA                *mib_OpData_p;
MIB_AUTH_ALG               *mib_AuthAlg_p;
MIB_PHY_DSSS_TABLE         *PhyDSSSTable_p;
MIB_PRIVACY_TABLE          *mib_PrivacyTable_p;
MIB_PHY_SUPP_DATA_RATES_TX *mib_PhySuppTxDataRates_p;
MIB_WEP_DEFAULT_KEYS       *mib_WepDefaultKeys_p;
MIB_PHY_TX_POWER_TABLE     *mib_PhyTxPwrData_p;
MIB_PHY_ANT_LIST           *mib_PhyAntList_p;
MIB_PHY_ANT_TABLE          *mib_PhyAntData_p;
MIB_COUNTERS               *mib_Counters_p;
MIB_DHCP                   *mib_DHCP_p;
#ifdef OS_VxW
MIB_DHCP                   *mib_DHCP_Eth_p;
MIB_DHCP                   *mib_DHCP_WAN_p;
#endif
MIB_RSNCONFIG              *mib_RSNConfig_p;
MIB_RSNCONFIG_UNICAST_CIPHERS   *mib_RSNConfigUnicastCiphers_p;
MIB_RSNCONFIG_AUTH_SUITES       *mib_RSNConfigAuthSuites_p;
MIB_RSNSTATS               *mib_RSNStats_p;
#ifdef BURST_MODE
MIB_BURST_MODE             *mib_BurstMode_p;
#endif

extern char * getWirelsSSID();
extern BOOLEAN mib_InitPriv(MIB_802DOT11 *mib)
{ 

    UINT32 i;

    /* Initialize global system mib pointer to mib structure. 
       Part of legacy code.                                   
    */
    mibSystem_p = mib;
    /* Initialize system mib pointers.  These are used by 
       modules for accessing mib configuration.  These
       are global pointers.  It may be a better idea to 
       assign a private mib pointers only to those mib 
       structures that each module will access and then 
       make it part of that module initialization.                           
    */ 
    mib_StaCfg_p             = &mib->StationConfig;
    mib_OpData_p             = &mib->OperationTable;
    mib_AuthAlg_p            = &mib->AuthAlg;
    PhyDSSSTable_p           = &mib->PhyDSSSTable;
    mib_PrivacyTable_p       = &mib->Privacy;
    mib_PhySuppTxDataRates_p = mib->SuppDataRatesTx;
    mib_WepDefaultKeys_p     = mib->WepDefaultKeys;
    mib_PhyTxPwrData_p       = &mib->PhyPowerTable;
    mib_PhyAntList_p         = &mib->AntennasListTable;
    mib_PhyAntData_p         = &mib->PhyAntTable;
    mib_Counters_p           = &mib->CountersTable;
    /* mib_DHCP is not part of the MIB_802DOT11 mib. */
    mib_DHCP_p               = &mib_DHCP;
#ifdef OS_VxW
    mib_DHCP_Eth_p           = &mib_DHCP_Eth;
    mib_DHCP_WAN_p           = &mib_DHCP_WAN;
#endif
    mib_RSNConfig_p            = &mib->RSNConfig;
    mib_RSNConfigUnicastCiphers_p = &mib->UnicastCiphers;
    mib_RSNConfigAuthSuites_p  = &mib->RSNConfigAuthSuites;
    mib_RSNStats_p             = &mib->RSNStats;
#ifdef BURST_MODE
    mib_BurstMode_p            = &mib->BurstMode; 

    mib_BurstMode_p->mib_burstmode = FALSE;
    mib_BurstMode_p->mib_burstrate = 0;
#endif
    /* These are global mib settings which are not currently part of
    the mib.  This is part of the legacy code we may want to check
    if these should be part of the HAL mib structures.            */
    mib_wep             = MIB_WEP;
    mib_preAmble        = PREAMBLE_AUTO_SELECT; /*Default, auto */
    mib_rxAntenna       = RX_ANTENNA;
    mib_txAntenna       = TX_ANTENNA;
    mib_broadcastssid   = TRUE;
    mib_defaultkeyindex = 0;

    mib_dhcpenable      = FALSE;
    mib_dhcpstatechange = TRUE;
    mib_shortSlotTime   = TRUE;

    /* default in auto state */
    mib_txDataRate      = 3;
    mib_enableFixedRateTx = 0;
    mib_txDataRateG = 12;    /* Register index of the G rate, 54 M default */
    mib_ApMode = AP_MODE_MIXED;
    mib_ErpProtEnabled = TRUE;
    mib_wlanfilterno = 0;
    mib_wlanfiltermode = 0;

    mib_DeviceName[0] = 0xff;

    mib_PrivacyTable_p->RSNEnabled = FALSE;
    mib_RSNConfig_p->GroupRekeyTime = 24*60*60; /*once per day*/
    /* Get the MAC address from flash */
    apfdGetEtherAddr(enaddr);

    for (i = 0; i < 6; ++i)
    {
        mib_OpData_p->StaMacAddr[i] = enaddr[i];
        mib_currentmacaddress[i]    = enaddr[i];
    }
#ifdef AP_PCI
    mib_OpData_p->StaMacAddr[5] = enaddr[5]+1;
#endif
    //util_CopyList(mib_StaCfg_p->DesiredSsId, OEMSSID, IEEEtypes_SSID_SIZE);
    util_CopyList(mib_StaCfg_p->DesiredSsId, getWirelsSSID(), IEEEtypes_SSID_SIZE);

#ifdef FLASH_CONFIG

    /* Read user settings from flash */
    GetUserConfig(mibSystem_p);
#endif

    if(mib_DeviceName[0] == 0xff)
    {
        util_CopyList(mib_DeviceName, "AP31", 16);
        patch_devName();
    }

    return(OS_SUCCESS);
}
extern void patch_devName(void)
{
    int i;
    UINT8 *tmp = mib_DeviceName;
    for (i = 0; i < 16, *(tmp+i); i++);
    i = (i > 9) ? 10: i;
    sprintf(&mib_DeviceName[i],"%02x%02x%02x",
            enaddr[3], enaddr[4], enaddr[5]);
}

BOOLEAN isStrictShareMode(void) 
{
    return(!mib_wep);
}
BOOLEAN isAuthAlgTypeMatch(UINT8 x) 
{
    return(x == mib_AuthAlg_p->Type);
}
BOOLEAN isWepRequired(void) 
{
    return(mib_StaCfg_p->PrivOption);
}

#ifdef MACACCESSTABLE
BOOLEAN isInMacAccessList(IEEEtypes_MacAddr_t *destAddr_p)
{
    UINT8 i;

#ifdef WDS_FEATURE
    if ( mib_wdsEnable && mib_disableAssoc ) {
        return(FALSE);
    }
#endif /* WDS_FEATURE */

    if ( mib_wlanfiltermode == WLAN_FILTER_DISABLE )
        return(TRUE);

#if !defined(WIPOD_NEW_GUI)
    if ( mib_wlanfilterno < 1 )  /*MacAccess Table is not enable */
        return(TRUE);
#endif

    for ( i = 0;i < mib_wlanfilterno;i++ )
    {
        if ( memcmp(destAddr_p, mib_wlanfiltermac + i*6, 6) == 0 )
        {
            /*found entry */
            if ( mib_wlanfiltermode == WLAN_FILTER_BLOCK )
                return(FALSE);
            return(TRUE);
        }
    }
    /* not found */
    if ( mib_wlanfiltermode == WLAN_FILTER_BLOCK )
        return(TRUE);

    return(FALSE);

}
#endif

