/******************** (c) Marvell Semiconductor, Inc., 2001 *******************
*
* $Header: E:\\APCVSDEPOT/wlan/prod/ap/ap31/bridge/src/w81init.c,v 1.35 2004/06/15 20:22:12 jshen Exp $
*
* Purpose:
*    This file contains the implementations of the function prototypes given
*    in the associated header file for the OS wrapper.
*
* Public Procedures:
*		WL_Init()
*
* Private Procedures:
*		WL_hwInit()
*		WL_swInit()
*		WL_TaskInit()
*
* Notes:
*    None.
*
*****************************************************************************/ 
/*============================================================================= */
/*                               INCLUDE FILES */
/*============================================================================= */
#include "buildModes.h"
#include "osif.h"
#include "sysinfo.h" 
#include "w81init.h"

#include "IEEE_types.h"
#include "timer.h"
#include "wl_mib.h"
#include "wl_hal.h"
#include "pool.h"
#include "bcngen.h"
#include "ds.h"
#include "tkip.h"
#include "rx80211.h"  /* Added for hal debug. */
#include "timer.h"
#include "tx80211.h"  /* Added for hal debug. */

#include "oem.h"

#include "wltypes.h"
#include "bufmgt.h"
#include "intr.h"
#include "wl_macros.h"
#include "ap_mib.h"
#include "psQue.h"
#include "bufmgt.h"
UINT8 system_Init(WL_SYS_CFG *sSysCfg_p);    

extern void hw_Init(void);
extern void init_static_network_interfaces(int);
extern void web_Start( void ) ;
extern WL_STATUS web_Init( void ) ;
extern WL_STATUS bridge_Init( void ) ;
extern void bridge_Start( void ) ;
extern void iCacheEnable(void);
extern void flashDrvInit(void);

extern WL_STATUS idle_Init( void );
extern WL_STATUS dhcpTask_Init( void );
extern void dhcp_Start( void );
extern void idle_Start( void );
extern int create_ds_pools();


/*============================================================================= */
/*                                DEFINITIONS */
/*============================================================================= */

#define CLK_MASK   0x000E
#define CLK_132MHZ  0x4
#define CLK_88MHZ   0x2


#define PACKET_NUMBER 160


UINT8 mem[PACKET_NUMBER * POOL_BUFFER_SIZE];
/*============================================================================= */
/*                              TYPE DEFINITIONS */
/*============================================================================= */
/* None. */


/*============================================================================= */
/*                         IMPORTED PUBLIC VARIABLES */
/*============================================================================= */
#ifndef NO_ECOS_TCP_IP_STACK
extern cyg_sem_t ip_ready;
#endif /* ECOS_TCP_IP_STACK */

UINT8   *ethAllocBuf(UINT32 align, UINT32 size);
BOOLEAN ethFreeBuf(void *dataPtr);

UINT8* allocMemTest0(UINT32 allign, UINT32 size);

/*============================================================================= */
/*                          MODULE LEVEL VARIABLES */
/*============================================================================= */

MFG_CAL_DATA *calData; 

extern void hw_Init(void);
extern unsigned char enaddr[]; 
/*============================================================================= */
/*                         CODED PUBLIC PROCEDURES */
/*============================================================================= */


void TimerSetUp(void (*timerCbFunc)(void))
{
    #define MILLISECONDS_PER_TICK 10
    cyg_handle_t hSysClk;
    cyg_handle_t hCounter;

    static cyg_handle_t hAlarm;    
    static unsigned int alarmData;    
    static cyg_alarm timerAlarm;
    int interval = 90 / MILLISECONDS_PER_TICK;   /* 0.1sec*/
    
    /* Attach the timer to the real-time clock */
    hSysClk = cyg_real_time_clock();

    cyg_clock_to_counter(hSysClk, &hCounter);

    cyg_alarm_create(hCounter, (cyg_alarm_t *)timerCbFunc,
                     (cyg_addrword_t) &alarmData,
                     &hAlarm, &timerAlarm);

    /* This creates a periodic timer */
    cyg_alarm_initialize(hAlarm, cyg_current_time() + interval, interval);
}

/******************************************************************************
 *
 * Name: system_Init
 *
 * Description:
 *   This routine is the entry point for initialization
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   
 *
 * Return Value:
 *   status
 *
 * Notes:
 *   None.
 *
 * PDL:
 *    
 * END PDL
 *
 ****************************************************************************/
/* Need to declare eth_dev structure here.  Needed to support
   current state of legacy code.  I will change this later.
*/
NET_DEV      eth_dev;
UINT8 system_Init(WL_SYS_CFG *sSysCfg_p)
{
    NET_DEV      *wlMac_p;
    NET_DEV      *eth_dev_p;

    UINT8 status;
    UINT32 Misc1Reg;

    sSysCfg_p->EthNetDev   = &eth_dev;

    /* Need to replace sSysCfg_p->EthNetDev with eth_dev.  
       Current legacy code uses this global device 
       structure to access ethernet driver.  Temporary, 
       just needed to support legacy code. 
    */ 
    eth_dev_p = &eth_dev;
    wlMac_p   = sSysCfg_p->WlNetDev;

    WL_WRITE_WORD(0x8000200c, 0xffffffff);
    
    hal_SetClock(CPU_MHZ_160, CPU_MHZ_160); 

    msi_wl_InitPadResetPeriph();

    /* Configure wireless device driver. */
    wlMac_p->allocBuf   = NULL;
    wlMac_p->allocMem   = allocMemTest0;
    wlMac_p->flags      = 0;
    wlMac_p->freeBuf    = NULL;
    wlMac_p->NetDevType = NET_DEV_W8310;
    /* Example wireless phy address. */
    wlMac_p->PhyAddr[0] = mib_OpData_p->StaMacAddr[0];
    wlMac_p->PhyAddr[1] = mib_OpData_p->StaMacAddr[1];
    wlMac_p->PhyAddr[2] = mib_OpData_p->StaMacAddr[2];
    wlMac_p->PhyAddr[3] = mib_OpData_p->StaMacAddr[3];
    wlMac_p->PhyAddr[4] = mib_OpData_p->StaMacAddr[4];
    wlMac_p->PhyAddr[5] = mib_OpData_p->StaMacAddr[5];
    wlMac_p->PrivData   = NULL;

    /* Initialize wireless MAC. */
    {
        extern void MemDma(UINT32 x, UINT32 y, UINT32 z);
        extern void MemDma1(UINT32 x, UINT32 y, UINT32 z);
        extern void MemDmaWPA(UINT32 x, UINT32 y, UINT32 z);
        extern void MemDma1WPA(UINT32 x, UINT32 y, UINT32 z);
        WLAN_CONTROL wlan_ctrl;
        wlan_ctrl.rxBeaconCopy = NULL;

        if (mib_PrivacyTable_p->RSNEnabled)
        {
            wlan_ctrl.rxDataCopy = MemDmaWPA;
            wlan_ctrl.rxDataCopy1 = MemDma1WPA;
        }
        else
        {
            wlan_ctrl.rxDataCopy = MemDma;
            wlan_ctrl.rxDataCopy1 = MemDma1;
        }
        wlan_ctrl.rxMgmtCopy = NULL;
        msi_wl_InitWLMAC(WL_OP_MODE_AP, wlMac_p, &wlan_ctrl);
    }

    hw_Init();
    msi_wl_StatsReset();
    /* Configure ethernet driver */
    /* ethAllocBuf and etheFreeBuf are user defined. */
    eth_dev_p->allocBuf   = ethAllocBuf;
    eth_dev_p->freeBuf    = ethFreeBuf;
    /* Allocate memory is intended for static memory allocation, once
       allocated it is not freed.  Intended for internal device driver
       memory. The ethernet driver has the option of using memory 
       defined by the user internally. */
    eth_dev_p->allocMem   = NULL;
    eth_dev_p->PrivData   = NULL;
    eth_dev_p->NetDevType = NET_DEV_FAST_ETH;
    eth_dev_p->flags      = 0;
    /* Example ethernet phy address. */
    eth_dev_p->PhyAddr[0] = mib_currentmacaddress[0];
    eth_dev_p->PhyAddr[1] = mib_currentmacaddress[1];
    eth_dev_p->PhyAddr[2] = mib_currentmacaddress[2];
    eth_dev_p->PhyAddr[3] = mib_currentmacaddress[3];
    eth_dev_p->PhyAddr[4] = mib_currentmacaddress[4];
    eth_dev_p->PhyAddr[5] = mib_currentmacaddress[5];
#ifdef ETHERSWITCH
    eth_dev_p->flags = 0x2;
#endif
#ifdef ETHERSWITCH_IN_BOOT
    eth_dev_p->flags = 0x3;
#endif

    /* Initialize ethernet port. */
    msi_wl_EthInit(eth_dev_p);
   


    /* Initialize Base Band Pass Processor. */
    /*msi_wl_InitBBP(calData); */
    /* Initialize RF Phy. */
    /*msi_wl_InitPHY(); */
    BBPInit();
    RFInit();
#ifdef AP_PGRINE /* Bill Modify and JL Write it */
    msi_wl_CheckRFReady();
#endif
#ifdef RF_POWERDOWN_BOOT
    msi_wl_PowerdownRF();
#endif

#ifdef AP_PCI
    hwi_pci_Init();
    msi_wl_EnableInt(INTR_DEFAULT_EN_HOST_IF);
#else
#ifdef ENABLE_GPIO_INT
    msi_wl_EnableInt(INTR_DEFAULT_ENABLE);
#else
    msi_wl_EnableInt(INTR_DEFAULT_DIS_GPIO_ENABLE);
#endif
#endif
    TimerSetUp(TimerTick);  /* 0.1s per tick */

    return(OS_SUCCESS);
}

/*============================================================================= */
/*                         CODED PRIVATE PROCEDURES */
/*============================================================================= */




extern UINT8 *buf_alloc(void *handle, int size);
extern void buf_free(void *handle, UINT8 *p);
QHANDLE ethRxQ, ethTxQ, wlRxQ, wlTxQ, mgtCmdQ;
#ifndef ONE_POOL
extern DEV_CTRL wlan_dev_ctrl;
#endif
void
wlan_data_ready(QHANDLE handle, UINT8 Qid, UINT8 pkts)
{
#ifndef ONE_POOL
    switch ( Qid )
    {
    case QUEUE_DATA_0:
#ifdef USE_NEW_OSIF
        os_EventTrigger(dsEH, ds_802_11_DATA_RCVD);
#else
        os_EventTrigger(sysinfo_DS_EVENT, ds_802_11_DATA_RCVD);
#endif
        break;
    case QUEUE_MLME:
#ifdef USE_NEW_OSIF
        os_EventTrigger(mgmtEH, macMgmtMain_802_11_MGMT_MSG_RCVD);
#else
        os_EventTrigger(sysinfo_MAC_MGMT_MAIN_Q_EVENT, macMgmtMain_802_11_MGMT_MSG_RCVD);
#endif
        break;
    }
#endif
}

void
wlan_tx_done(QHANDLE handle, UINT8 Qid, UINT8 pkts)
{
#ifdef USE_NEW_OSIF
    os_EventTrigger(tx11EH, tx80211_FRAME_TX_DONE);
#else
    os_EventTrigger(sysinfo_802_11_TX_Q_EVENT, tx80211_FRAME_TX_DONE);
#endif
}

POOL pool_hd;

int q_Init()
{
    int i=0;
#ifndef ONE_POOL
    
    PKT_Q_CFG cfg, cfg1, cfg2, cfg3, cfg4;
#endif
    poolCreate(&pool_hd, mem, PACKET_NUMBER, POOL_BUFFER_SIZE);

#ifndef ONE_POOL
    memset(&cfg, 0, sizeof(cfg));
    cfg.type = QUEUE_RING;
    cfg.bd_align = 16;
    cfg.buf_align = 16;
    cfg.hdr_size = ROUND4(sizeof(ether_hdr_t)); /* for 4-byte Rx alignment */
    cfg.usr_data_size = 0;  /* 128 */
    cfg.buf_size = 2532;
/*    cfg.post_put = my_post_put; */
    cfg.info_size = 64;
/*    cfg.post_get = my_post_get; */
    cfg.def_pri_put = QUEUE_PRI_LOW;
    cfg.def_pri_get = QUEUE_PRI_HIGH;
    /*   for (i=0; i < QUEUE_MAX; i++) */
    cfg.length[0] = 0;    /* Lengths of each queue */

    cfg.mem_handle = (void *)&pool_hd;
    cfg.alloc = poolGet;
    cfg.free = poolPut;
    ethRxQ = msi_wl_CreatePktQ(&cfg, FALSE);

    i++;
    memset(&cfg1, 0, sizeof(cfg1));
    cfg1.type = QUEUE_LIST;
    cfg1.bd_align = 16;
    cfg1.buf_align = 16;
    cfg1.hdr_size = sizeof(ether_hdr_t);
    cfg1.usr_data_size = 0;  /* 128 */
    cfg1.buf_size = 2532;
/*    cfg.post_put = my_post_put; */
    cfg1.info_size = 64;
/*    cfg.post_get = my_post_get; */
    cfg1.def_pri_put = QUEUE_PRI_LOW;
    cfg1.def_pri_get = QUEUE_PRI_HIGH;
/*    for (i=0; i < QUEUE_MAX; i++) */
    cfg1.length[0] = 0;    /* Lengths of each queue */

    cfg1.mem_handle = (void *)&pool_hd;
    cfg1.alloc = poolGet;
    cfg1.free = poolPut;
    ethTxQ = msi_wl_CreatePktQ(&cfg1, FALSE);

    i++;

/********************************************************************/


#if 1

    memset(&cfg2, 0, sizeof(cfg));
    cfg2.type = QUEUE_RING;
    cfg2.bd_align = 16;
    cfg2.buf_align = 16;
    cfg2.usr_data_size = 0;
    cfg2.info_size = 64;/*sizeof(WLAN_RX_FRAME); */
    cfg2.hdr_size = sizeof(IEEEtypes_GenHdr_t);
    cfg2.buf_size = 2500;
    /*cfg.length[QUEUE_DATA_0] = RX_BUF_NUM-1; */ /*    Lengths of each queue */
    cfg2.length[QUEUE_DATA_0] = 48;    /* Lengths of each queue */
    cfg2.length[QUEUE_MLME] = 8;/*TX_BUF_NUM;  */ /*    Lengths of each queue */
    cfg2.data_ready = wlan_data_ready;

    /*cfg.mem_handle = (void *)wl_rx_pool_init(); */
    cfg2.mem_handle = (void *)&pool_hd;
    cfg2.alloc = poolGet;
    cfg2.free = poolPut;

    wlRxQ = msi_wl_CreatePktQ(&cfg2, TRUE);

    memset(&cfg3, 0, sizeof(cfg));
    cfg3.type = QUEUE_RING;
    cfg3.bd_align = 16;
    cfg3.buf_align = 16;
    cfg3.usr_data_size = 0;  /* 128 */
    cfg3.info_size = 64;/*sizeof(WLAN_TX_FRAME); */
    cfg3.hdr_size = sizeof(IEEEtypes_GenHdr_t);
    cfg3.buf_size = 2500;
    /*cfg.length[QUEUE_DATA_0] = TX_BUF_NUM; */ /*   Lengths of each queue */
    /*cfg.length[QUEUE_MLME] = TX_BUF_NUM;   */ /*  Lengths of each queue */
    cfg3.length[QUEUE_DATA_0] = 32;/*TX_BUF_NUM;    Lengths of each queue */
    cfg3.length[QUEUE_MLME] = 8;/*TX_BUF_NUM;      Lengths of each queue */

    cfg3.tx_done = wlan_tx_done;

    /*cfg.mem_handle = (void *)wl_tx_pool_init(); */
    cfg3.mem_handle = (void *)&pool_hd;
    cfg3.alloc = poolGet;
    cfg3.free = poolPut;

    wlTxQ = msi_wl_CreatePktQ(&cfg3, FALSE);
    msi_wl_BindPktQ(&wlan_dev_ctrl, wlRxQ, wlTxQ);
#endif

#if 1
    memset(&cfg4, 0, sizeof(cfg));
    cfg4.type = QUEUE_RING;
    cfg4.bd_align = 16;
    cfg4.buf_align = 16;
    cfg4.usr_data_size = 0;
    cfg4.info_size = 64;
    cfg4.hdr_size = 0;/*sizeof(IEEEtypes_GenHdr_t); */
    cfg4.buf_size = 700;
    cfg4.length[QUEUE_DATA_0] = 8;    /* Lengths of each queue */
    /*cfg.data_ready = wlan_data_ready; */

    cfg4.mem_handle = (void *)&pool_hd;
    cfg4.alloc = poolGet;
    cfg4.free = poolPut;

    mgtCmdQ = msi_wl_CreatePktQ(&cfg4, FALSE);
#endif
#endif

}


/******************************************************************************
 *
 * Name: wlan_swinit
 *
 * Description:
 *   
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *    
 *
 * Return Value:
 *   status
 *
 * Notes:
 *   None.
 *
 * PDL:
 *    
 * END PDL
 *
 ****************************************************************************/
WL_SYS_CFG      *pSystemHaltest;
#ifdef USE_NEW_OSIF
OS_SYS osSys;
extern void *malloc (size_t size);
extern void free (void *ptr);
#endif

int AP_Init(void)
{
#ifdef USE_NEW_OSIF
    osSys.allocMem = malloc;
    osSys.freeMem = free;
    os_OsInit(&osSys);
#else
    os_OsInit();
#endif
    q_Init();

#ifdef FLASH_CONFIG
    flashDrvInit();
#endif

    /* msi_wl_CreateSysCfg works fine. Still use mib_InitPriv for
       setting some mib variables which are not part of the new
       structure.  Part of legacy code.  
     */
#ifdef EEPROM
    calData = GetCalData();
#else
    calData = (MFG_CAL_DATA *)MFG_DATA_ADDRESS;
#endif
    if(NoCalibration())
        calData = NULL;  
    pSystemHaltest = msi_wl_CreateSysCfg(WL_OP_MODE_AP, calData);
#ifdef ETHERSWITCH_IN_BOOT
    pSystemHaltest->flag = 0x01;
#else
    pSystemHaltest->flag = 0x00;
#endif
    mib_InitPriv(pSystemHaltest->Mib802dot11);
    system_Init(pSystemHaltest);

	#ifndef NO_ECOS_TCP_IP_STACK
    cyg_semaphore_init(&ip_ready, 0);
	#endif /* ECOS_TCP_IP_STACK */

    extStaDb_Init(MAX_STNS);
    macMgmtAp_Init(MAX_STNS, mib_OpData_p->StaMacAddr);
    smeMain_Init();
    ds_Init();
    tx80211_Init();
    txsched_Init();
    idle_Init();
    
    macMgmtAp_Start();
    smeMain_Start();
    tx80211_Start();
    /*bcngen_Init();*/

#if defined(WDS_FEATURE)
    AddWdsMacs();
#endif

#ifndef NO_ECOS_TCP_IP_STACK
#ifdef TASKDHCP
    dhcpTask_Init();
#else

    init_static_network_interfaces(1);
#endif

#ifndef EXCLUDE_WEB
    web_Init();
#endif
#ifdef TASKDHCP

    dhcp_Start();
#endif

#ifndef EXCLUDE_WEB
    web_Start();
#endif
#endif /* ECOS_TCP_IP_STACK */
#ifdef ICACHE_ENABLE

    iCacheEnable();
#endif
    idle_Start();

#ifdef ENABLE_RATE_ADAPTATION_BASEBAND
    SetupAdaptRow();
#endif
    MaxResourceCnt = pool_hd.nFree__;
    return(1);
}

extern UINT32 *pRxFrag0Buf;
extern UINT32 *pRxFrag1Buf;
extern UINT32 *pRxFrag2Buf;
extern txBcnInfo_t *PrbRspTxInfo;

extern UINT32 TxQSpace[14 * 64+ 1024]__attribute__ ((section (".sqbuf")));
extern txInfo_t *BcnTxInfo;
extern IEEEtypes_Bcn_t *BcnBuffer_p;

extern UINT32 *pRxDataNFragBuf;
extern UINT32 *pRxMgmtNFragBuf;
extern UINT32 *pRxBcnBuf;


UINT32 flag[10] = {1,1,1,1,1,1,1,1,1,1};

UINT8* allocMemTest0(UINT32 allign, UINT32 size)
{
    if ( (size == tx_PTR_ARRAY_SIZE) && flag[0] )
    {
        return((UINT8*) ((UINT32)(&TxQSpace[0] + 1024) & 0xfffff000));
    }

    if ( (size == tx_BCN_SIZE) && flag[1] )
    {
        return(UINT8 *) BcnTxInfo;
    }

    if ( (size == rx_BCN_BUF_SIZE) && flag[2] )
    {
        return(UINT8 *) pRxBcnBuf;
    }

    if ( (size == RX_AP_MGMT_NFRAG_BUFSIZE) && flag[3] )
    {
        return(UINT8 *) pRxMgmtNFragBuf;
    }

    if ( (size == RX_AP_DATA_NFRAG_BUFSIZE) && flag[4] )
    {
        return(UINT8 *) pRxDataNFragBuf;
    }

    if ( (size == sizeof(txBcnInfo_t)) && flag[5] )
    {
        return(UINT8 *) PrbRspTxInfo;
    }

    if ( (size == RX_AP_FRAG0_BUFSIZE) && flag[6] )
    {
        return(UINT8 *) pRxFrag0Buf;
    }

    if ( (size == RX_AP_FRAG1_BUFSIZE) && flag[7] )
    {
        return(UINT8 *) pRxFrag1Buf;
    }

    if ( (size == RX_AP_FRAG2_BUFSIZE) && flag[8] )
    {
        return(UINT8 *) pRxFrag2Buf;
    }

    return(NULL);

}

