#ifndef _IEEE_TYPES_H_
#define _IEEE_TYPES_H_
/*
 *                Copyright 2003, Marvell Semiconductor, Inc.
 * This code contains confidential information of Marvell Semiconductor, Inc.
 * No rights are granted herein under any patent, mask work right or copyright
 * of Marvell or any third party.
 * Marvell reserves the right at its sole discretion to request that this code
 * be immediately returned to Marvell. This code is provided "as is".
 * Marvell makes no warranties, express, implied or otherwise, regarding its
 * accuracy, completeness or performance.
 */ 
/*!
 * \file    IEEE_types.h
 * \brief   This file contains definitions relating to messages specified in the
 *          IEEE 802.11 spec
 */

//=============================================================================
//                               INCLUDE FILES
//=============================================================================

//=============================================================================
//                            PUBLIC DEFINITIONS
//=============================================================================

/*--------------------------------------------------------------*/
/* Reason Codes - these codes are used in management message    */
/* frame bodies to indicate why an action is taking place (such */
/* as a disassociation or deauthentication).                    */
/*--------------------------------------------------------------*/
#define IEEEtypes_REASON_RSVD                    0
#define IEEEtypes_REASON_UNSPEC                  1
#define IEEEtypes_REASON_PRIOR_AUTH_INVALID      2
#define IEEEtypes_REASON_DEAUTH_LEAVING          3
#define IEEEtypes_REASON_DISASSOC_INACTIVE       4
#define IEEEtypes_REASON_DISASSOC_AP_BUSY        5
#define IEEEtypes_REASON_CLASS2_NONAUTH          6
#define IEEEtypes_REASON_CLASS3_NONASSOC         7
#define IEEEtypes_REASON_DISASSOC_STA_HASLEFT    8
#define IEEEtypes_REASON_CANT_ASSOC_NONAUTH      9

#ifdef WPA
#define IEEEtypes_REASON_INVALID_IE              13
#define IEEEtypes_REASON_MIC_FAILURE             14
#define IEEEtypes_REASON_4WAY_HANDSHK_TIMEOUT    15
#define IEEEtypes_REASON_GRP_KEY_UPD_TIMEOUT     16
#define IEEEtypes_REASON_IE_4WAY_DIFF            17
#define IEEEtypes_REASON_INVALID_MCAST_CIPHER    18
#define IEEEtypes_REASON_INVALID_UNICAST_CIPHER  19
#define IEEEtypes_REASON_INVALID_AKMP            20
#define IEEEtypes_REASON_UNSUPT_RSNE_VER         21
#define IEEEtypes_REASON_INVALID_RSNE_CAP        22
#define IEEEtypes_REASON_8021X_AUTH_FAIL         23
#endif

/*------------------------------------------------------------*/
/* Status Codes - these codes are used in management message  */
/* frame bodies to indicate the results of an operation (such */
/* as association, reassociation, and authentication).        */
/*------------------------------------------------------------*/
#define IEEEtypes_STATUS_SUCCESS                 0
#define IEEEtypes_STATUS_UNSPEC_FAILURE          1
#define IEEEtypes_STATUS_CAPS_UNSUPPORTED        10
#define IEEEtypes_STATUS_REASSOC_NO_ASSOC        11
#define IEEEtypes_STATUS_ASSOC_DENIED_UNSPEC     12
#define IEEEtypes_STATUS_UNSUPPORTED_AUTHALG     13
#define IEEEtypes_STATUS_RX_AUTH_NOSEQ           14
#define IEEEtypes_STATUS_CHALLENGE_FAIL          15
#define IEEEtypes_STATUS_AUTH_TIMEOUT            16
#define IEEEtypes_STATUS_ASSOC_DENIED_BUSY       17
#define IEEEtypes_STATUS_ASSOC_DENIED_RATES      18

//
// 802.11b additions
//
#define IEEEtypes_STATUS_ASSOC_DENIED_NOSHORT    19
#define IEEEtypes_STATUS_ASSOC_DENIED_NOPBCC     20
#define IEEEtypes_STATUS_ASSOC_DENIED_NOAGILITY  21
#ifdef ERP 
//
// 802.11g additions
//
#define IEEEtypes_STATUS_ASSOC_DENIED_NOSHORTSLOTTIME 25
#define IEEEtypes_STATUS_ASSOC_DENIED_NODSSSOFDM      26
#endif 
/*--------------------------------------------*/
/* Various sizes used in IEEE 802.11 messages */
/*--------------------------------------------*/
#define IEEEtypes_ADDRESS_SIZE         6
#define IEEEtypes_BITMAP_SIZE          251
#define IEEEtypes_CHALLENGE_TEXT_SIZE  253
#define IEEEtypes_CHALLENGE_TEXT_LEN   128
#define IEEEtypes_MAX_DATA_RATES       8
#define IEEEtypes_MAX_DATA_BODY_LEN    2312
#define IEEEtypes_MAX_MGMT_BODY_LEN    2312
#define IEEEtypes_SSID_SIZE            32
#define IEEEtypes_TIME_STAMP_SIZE      8
#define IEEEtypes_MAX_CHANNELS         14
#define IEEEtypes_MAX_BSS_DESCRIPTS    16
#define W81_80211_HEADER_SIZE          32
#ifdef ERP
#define IEEEtypes_MAX_DATA_RATES_G     14
#endif 
/*---------------------------------------------------------------------*/
/* Define masks used to extract fields from the capability information */
/* structure in a beacon message.                                      */
/*---------------------------------------------------------------------*/
#define IEEEtypes_CAP_INFO_ESS             1
#define IEEEtypes_CAP_INFO_IBSS            2
#define IEEEtypes_CAP_INFO_CF_POLLABLE     4
#define IEEEtypes_CAP_INFO_CF_POLL_RQST    8
#define IEEEtypes_CAP_INFO_PRIVACY         16
#define IEEEtypes_CAP_INFO_SHORT_PREAMB    32
#define IEEEtypes_CAP_INFO_PBCC            64
#define IEEEtypes_CAP_INFO_CHANGE_AGILITY  128
#ifdef ERP
#define IEEEtypes_CAP_INFO_SHORT_SLOT_TIME  0x0400
#define IEEEtypes_CAP_INFO_DSSS_OFDM        0x2000
#endif

/*---------------------------*/
/* Miscellaneous definitions */
/*---------------------------*/
#define IEEEtypes_PROTOCOL_VERSION     0

#define IEEEtypes_BASIC_RATE_FLAG 0x80
//
// Used to determine which rates in a list are designated as basic rates
//

#define IEEEtypes_BASIC_RATE_MASK 0x7F
//
// Used to mask off the basic rate flag, if one exists, for given
// data rates
//

#define IEEEtypes_RATE_MIN 2
//
// The minimum allowable data rate in units of kb/s
//

#define IEEEtypes_RATE_MAX 127
//
// The maximum allowable data rate in units of kb/s
//

#define IEEEtypes_TIME_UNIT 1024
//
// The number of microseconds in 1 time unit, as specified in the
// 802.11 spec
//


//=============================================================================
//                          PUBLIC TYPE DEFINITIONS
//=============================================================================

/*---------------------------------------------------------------------------*/
/*                 Enumerated types used in 802.11 messages                  */
/*---------------------------------------------------------------------------*/
typedef enum
{
    IEEE_TYPE_MANAGEMENT = 0,
    IEEE_TYPE_CONTROL,
    IEEE_TYPE_DATA
} IEEEtypes_MsgType_e;
//
// The 3 possible types of 802.11 messages
//

typedef enum
{
    IEEE_MSG_ASSOCIATE_RQST = 0,
    IEEE_MSG_ASSOCIATE_RSP,
    IEEE_MSG_REASSOCIATE_RQST,
    IEEE_MSG_REASSOCIATE_RSP,
    IEEE_MSG_PROBE_RQST,
    IEEE_MSG_PROBE_RSP,
    IEEE_MSG_BEACON = 8,
    IEEE_MSG_ATIM,
    IEEE_MSG_DISASSOCIATE,
    IEEE_MSG_AUTHENTICATE,
    IEEE_MSG_DEAUTHENTICATE
} IEEEtypes_MgmtSubType_e;
//
// The possible types of management messages
//

typedef enum
{
    PS_POLL = 10,
    RTS,
    CTS,
    ACK,
    CF_END,
    CF_END_CF_ACK
} IEEEtypes_CtlSubType_e;
//
// The possible types of control messages
//

typedef enum
{
    DATA = 0,
    DATA_CF_ACK,
    DATA_CF_POLL,
    DATA_CF_ACK_CF_POLL,
    NULL_DATA,
    CF_ACK,
    CF_POLL,
    CF_ACK_CF_POLL
} IEEEtypes_DataSubType_e;
//
// The possible types of data messages
//

typedef enum
{
    SME_CMD_ASSOCIATE,
    SME_CMD_AUTHENTICATE,
    SME_CMD_DEAUTHENTICATE,
    SME_CMD_DISASSOCIATE,
    SME_CMD_JOIN,
    SME_CMD_REASSOCIATE,
    SME_CMD_RESET,
    SME_CMD_SCAN,
    SME_CMD_START
} IEEEtypes_SmeCmd_e;
//
// The possible types of commands sent from the SME
//

typedef enum
{
    SME_NOTIFY_PWR_MGMT_CFRM,
    SME_NOTIFY_SCAN_CFRM,
    SME_NOTIFY_JOIN_CFRM,
    SME_NOTIFY_AUTH_CFRM,
    SME_NOTIFY_AUTH_IND,
    SME_NOTIFY_DEAUTH_CFRM,
    SME_NOTIFY_DEAUTH_IND,
    SME_NOTIFY_ASSOC_CFRM,
    SME_NOTIFY_ASSOC_IND,
    SME_NOTIFY_REASSOC_CFRM,
    SME_NOTIFY_REASSOC_IND,
    SME_NOTIFY_DISASSOC_CFRM,
    SME_NOTIFY_DISASSOC_IND,
    SME_NOTIFY_RESET_CFRM,
    SME_NOTIFY_START_CFRM
} IEEEtypes_SmeNotify_e;
//
// The possible types of notifications sent from the SME
//

typedef enum
{
    CBP_CMD_ASSOCIATE = SME_CMD_ASSOCIATE,
    CBP_CMD_AUTHENTICATE = SME_CMD_AUTHENTICATE,
    CBP_CMD_DEAUTHENTICATE = SME_CMD_DEAUTHENTICATE,
    CBP_CMD_DISASSOCIATE = SME_CMD_DISASSOCIATE,
    CBP_CMD_JOIN = SME_CMD_JOIN,
    CBP_CMD_REASSOCIATE = SME_CMD_REASSOCIATE,
    CBP_CMD_RESET = SME_CMD_RESET,
    CBP_CMD_SCAN = SME_CMD_SCAN,
    CBP_CMD_START = SME_CMD_START,
    CBP_CMD_MIB_READ,
    CBP_CMD_MIB_WRITE,
    CBP_CMD_REGISTER_READ,
    CBP_CMD_REGISTER_WRITE,
    CBP_CMD_SIGNAL_STRENGTH,
    CBP_CMD_STATION_INFO_READ,
    CBP_CMD_STATION_INFO_WRITE,
    CBP_CMD_STATION_LIST,
    CBP_CMD_STATISTICS
} IEEEtypes_CbProcessorCmd_e;
//
// The possible types of commands sent from the CB Processor
//

typedef enum
{
    BSS_INFRASTRUCTURE = 1,
    BSS_INDEPENDENT,
    BSS_ANY
} IEEEtypes_Bss_e;
//
// The possible types of Basic Service Sets
//

typedef enum
{
    SSID = 0,
    SUPPORTED_RATES,
    FH_PARAM_SET,
    DS_PARAM_SET,
    CF_PARAM_SET,
    TIM,
    IBSS_PARAM_SET,
    CHALLENGE_TEXT = 16,
#ifdef ERP
    ERP_INFO = 42,
    EXT_SUPPORTED_RATES = 50,
#endif

#ifdef WPA
                          RSN_IE = 221,
#endif
} IEEEtypes_ElementId_e;
//
// Variable length mandatory fields or optional frame body components
// within management messages are called information elements; these
// elements all have associated with them an Element ID. IDs 7 to 15
// are reserved; IDs 17 to 31 are reserved for challenge text; IDs
// 32 to 255 are reserved.
//

typedef enum
{
    PWR_MODE_ACTIVE = 1,
    PWR_MODE_PWR_SAVE
} IEEEtypes_PwrMgmtMode_e;
//
// The possible power management modes
//

typedef enum
{
    SCAN_ACTIVE,
    SCAN_PASSIVE
} IEEEtypes_Scan_e;
//
// The possible methods to scan for APs
//

typedef enum
{
    AUTH_OPEN_SYSTEM = 0,
    AUTH_SHARED_KEY,
    AUTH_OPEN_OR_SHARED_KEY,
    AUTH_NOT_SUPPORTED
} IEEEtypes_AuthType_e;
//
// The possible types of authentication
//

typedef enum
{
    SCAN_RESULT_SUCCESS,
    SCAN_RESULT_INVALID_PARAMETERS
} IEEEtypes_ScanResult_e;
//
// The possible responses to a request to scan
//

typedef enum
{
    JOIN_RESULT_SUCCESS,
    JOIN_RESULT_INVALID_PARAMETERS,
    JOIN_RESULT_TIMEOUT
} IEEEtypes_JoinResult_e;
//
// The possible responses to a request to join a BSS
//

typedef enum
{
    AUTH_RESULT_SUCCESS,
    AUTH_RESULT_INVALID_PARAMETERS,
    AUTH_RESULT_TIMEOUT,
    AUTH_RESULT_TOO_MANY_SIMULTANEOUS_RQSTS,
    AUTH_RESULT_REFUSED
} IEEEtypes_AuthResult_e;
//
// The possible results to a request to authenticate
//

typedef enum
{
    DEAUTH_RESULT_SUCCESS,
    DEAUTH_RESULT_INVALID_PARAMETERS,
    DEAUTH_RESULT_TOO_MANY_SIMULTANEOUS_RQSTS
} IEEEtypes_DeauthResult_e;
//
// The possible results to a request to deauthenticate
//

typedef enum
{
    ASSOC_RESULT_SUCCESS,
    ASSOC_RESULT_INVALID_PARAMETERS,
    ASSOC_RESULT_TIMEOUT,
    ASSOC_RESULT_REFUSED
} IEEEtypes_AssocResult_e;
//
// The possible results to a request to associate
//

typedef enum
{
    REASSOC_RESULT_SUCCESS,
    REASSOC_RESULT_INVALID_PARAMETERS,
    REASSOC_RESULT_TIMEOUT,
    REASSOC_RESULT_REFUSED
} IEEEtypes_ReassocResult_e;
//
// The possible results to a request to reassociate
//

typedef enum
{
    DISASSOC_RESULT_SUCCESS,
    DISASSOC_RESULT_INVALID_PARAMETERS,
    DISASSOC_RESULT_TIMEOUT,
    DISASSOC_RESULT_REFUSED
} IEEEtypes_DisassocResult_e;
//
// The possible results to a request to disassociate
//

typedef enum
{
    PWR_MGMT_RESULT_SUCCESS,
    PWR_MGMT_RESULT_INVALID_PARAMETERS,
    PWR_MGMT_RESULT_NOT_SUPPORTED
} IEEEtypes_PwrMgmtResult_e;
//
// The possible results to a request to change the power management mode
//

typedef enum
{
    RESET_RESULT_SUCCESS
} IEEEtypes_ResetResult_e;
//
// The possible results to a request to reset
//

typedef enum
{
    START_RESULT_SUCCESS,
    START_RESULT_INVALID_PARAMETERS,
    START_RESULT_BSS_ALREADY_STARTED_OR_JOINED
} IEEEtypes_StartResult_e;
//
// The possible results to a request to start
//

typedef enum
{
    STATE_IDLE,
    STATE_SCANNING,
    STATE_JOINING,
    STATE_JOINED,
    STATE_AUTHENTICATED_WITH_AP,
    STATE_ASSOCIATING,
    STATE_ASSOCIATED,
    STATE_REASSOCIATING,
    STATE_RESTORING_FROM_SCAN,
    STATE_IBSS_STARTED,
} IEEEtypes_MacMgmtStates_e;
//
// The possible states the MAC Management Service Task can be in
//


/*---------------------------------------------------------------------------*/
/*           Types Used In IEEE 802.11 MAC Message Data Structures           */
/*---------------------------------------------------------------------------*/
typedef uint8 IEEEtypes_Len_t;
//
// Length type
//

typedef uint8 IEEEtypes_Addr_t;
//
// Address type
//

typedef IEEEtypes_Addr_t IEEEtypes_MacAddr_t[IEEEtypes_ADDRESS_SIZE];
//
// MAC address type
//

typedef uint8 IEEEtypes_DataRate_t;
//
// Type used to specify the supported data rates
//

typedef uint8 IEEEtypes_SsId_t[IEEEtypes_SSID_SIZE];
//
// SS ID type
//

/*---------------------------------------------------------------------------*/
/*                 IEEE 802.11 MAC Message Data Structures                   */
/*                                                                           */
/* Each IEEE 802.11 MAC message includes a MAC header, a frame body (which   */
/* can be empty), and a frame check sequence field. This section gives the   */
/* structures that used for the MAC message headers and frame bodies that    */
/* can exist in the three types of MAC messages - 1) Control messages,       */
/* 2) Data messages, and 3) Management messages.                             */
/*---------------------------------------------------------------------------*/
typedef struct IEEEtypes_FrameCtl_t
{
    uint16 ProtocolVersion:2;
    uint16 Type:2;
    uint16 Subtype:4;
    uint16 ToDs:1;
    uint16 FromDs:1;
    uint16 MoreFrag:1;
    uint16 Retry:1;
    uint16 PwrMgmt:1;
    uint16 MoreData:1;
    uint16 Wep:1;
    uint16 Order:1;
}
PACK IEEEtypes_FrameCtl_t;
//
// The frame control field in the header of a MAC message
//

typedef struct IEEEtypes_GenHdr_t
{
    uint16 FrmBodyLen;
    IEEEtypes_FrameCtl_t FrmCtl;
    uint16 DurationId;
    IEEEtypes_MacAddr_t Addr1;
    IEEEtypes_MacAddr_t Addr2;
    IEEEtypes_MacAddr_t Addr3;
    uint16 SeqCtl;
    IEEEtypes_MacAddr_t Addr4;
}
PACK IEEEtypes_GenHdr_t;
//
// The general header for MAC messages
//

typedef struct IEEEtypes_MgmtHdr_t
{
    uint16 FrmBodyLen;
    IEEEtypes_FrameCtl_t FrmCtl;
    uint16 Duration;
    IEEEtypes_MacAddr_t DestAddr;
    IEEEtypes_MacAddr_t SrcAddr;
    IEEEtypes_MacAddr_t BssId;
    uint16 SeqCtl;
    IEEEtypes_MacAddr_t Rsrvd;
}
PACK IEEEtypes_MgmtHdr_t;
//
// The header for MAC management messages
//

typedef struct IEEEtypes_PsPollHdr_t
{
    uint16 FrmBodyLen;
    IEEEtypes_FrameCtl_t FrmCtl;
    uint16 Aid;
    IEEEtypes_MacAddr_t BssId;
    IEEEtypes_MacAddr_t TxAddr;
}
PACK IEEEtypes_PsPollHdr_t;
//
// The header for power-save poll messages (the only control message
// processed by the MAC software)
//

typedef struct IEEEtypes_CtlFrame_t
{
    IEEEtypes_PsPollHdr_t Hdr;
    uint32 FCS;
}
PACK IEEEtypes_CtlFrame_t;
//
// The structure for control frames (of which only the power-save
// poll is processed by the MAC software)
//

typedef struct IEEEtypes_DataFrame_t
{
    IEEEtypes_GenHdr_t Hdr;
    uint8 FrmBody[IEEEtypes_MAX_DATA_BODY_LEN];
    uint32 FCS;
}
PACK IEEEtypes_DataFrame_t;
//
// The structure for data frames
//


/*-------------------------------------------------*/
/* Management Frame Body Components - Fixed Fields */
/*-------------------------------------------------*/
typedef uint16 IEEEtypes_AId_t;
//
// Association ID assigned by an AP during the association process
//

typedef uint16 IEEEtypes_AuthAlg_t;
//
// Number indicating the authentication algorithm used (it can take
// on the values given by IEEEtypes_AuthType_e):
//    0 = Open system
//    1 = Shared key
//    All other values reserved
//

typedef uint16 IEEEtypes_AuthTransSeq_t;
//
// Authentication transaction sequence number that indicates the current
// state of progress through a multistep transaction
//

typedef uint16 IEEEtypes_BcnInterval_t;
//
// Beacon interval that represents the number of time units between
// target beacon transmission times
//

typedef uint8 IEEEtypes_DtimPeriod_t;
//
// Interval that represents the number of time units between DTIMs.
//

typedef struct IEEEtypes_CapInfo_t
{
    unsigned Ess:1;
    unsigned Ibss:1;
    unsigned CfPollable:1;
    unsigned CfPollRqst:1;
    unsigned Privacy:1;
    unsigned ShortPreamble:1;
    unsigned Pbcc:1;
    unsigned ChanAgility:1 ;
#ifndef ERP

    unsigned Rsrvd:8;
#else

    unsigned Rsrvd1:2;
    unsigned ShortSlotTime:1;
    unsigned Rsrvd2:2;
    unsigned DsssOfdm:1;
    unsigned Rsrvd3:2;
#endif
}
PACK IEEEtypes_CapInfo_t;
//
// Capability information used to indicate requested or advertised
// capabilities
//

typedef uint16 IEEEtypes_ListenInterval_t;
//
// Listen interval to indicate to an AP how often a STA wakes to listen
// to beacon management frames
//

typedef uint16 IEEEtypes_ReasonCode_t;
//
// Reason code to indicate the reason that an unsolicited notification
// management frame of type Disassociation or Deauthentication was
// generated
//

typedef uint16 IEEEtypes_StatusCode_t;
//
// Status code used in a response management frame to indicate the
// success or failure of a requested operation
//

typedef uint8 IEEEtypes_TimeStamp_t[IEEEtypes_TIME_STAMP_SIZE];
//
// Time stamp
//
#ifdef CONDOR
typedef struct IEEEtypes_QosCtl_t
{
    uint16 QosControl;
}
PACK IEEEtypes_QosCtl_t;
#endif 
/*-------------------------------------------------------*/
/* Management Frame Body Components - Information Fields */
/*-------------------------------------------------------*/
typedef struct IEEEtypes_InfoElementHdr_t
{
    IEEEtypes_ElementId_e ElementId;
    IEEEtypes_Len_t Len;
}
PACK IEEEtypes_InfoElementHdr_t;

typedef struct IEEEtypes_SsIdElement_t
{
    IEEEtypes_ElementId_e ElementId;
    IEEEtypes_Len_t Len;
    IEEEtypes_SsId_t SsId;
}
PACK IEEEtypes_SsIdElement_t;
//
// SSID element that idicates the identity of an ESS or IBSS
//

typedef struct IEEEtypes_SuppRatesElement_t
{
    IEEEtypes_ElementId_e ElementId;
    IEEEtypes_Len_t Len;
    IEEEtypes_DataRate_t Rates[IEEEtypes_MAX_DATA_RATES];
}
PACK IEEEtypes_SuppRatesElement_t;
//
// Supported rates element that specifies the rates in the operational
// rate set in the MLME join request and the MLME start request
//

typedef struct IEEEtypes_FhParamSet_t
{
    IEEEtypes_ElementId_e ElementId;
    IEEEtypes_Len_t Len;
    uint16 DwellTime;
    uint8 HopSet;
    uint8 HopPattern;
    uint8 HopIndex;
}
PACK IEEEtypes_FhParamSet_t;
//
// FH parameter set that conatins the set of parameters necessary to
// allow sychronization for stations using a frequency hopping PHY
//

typedef struct IEEEtypes_DsParamSet_t
{
    IEEEtypes_ElementId_e ElementId;
    IEEEtypes_Len_t Len;
    uint8 CurrentChan;
}
PACK IEEEtypes_DsParamSet_t;
//
// DS parameter set that contains information to allow channel number
// identification for stations using a direct sequence spread spectrum PHY
//

typedef struct IEEEtypes_CfParamSet_t
{
    IEEEtypes_ElementId_e ElementId;
    IEEEtypes_Len_t Len;
    uint8 CfpCnt;
    uint8 CfpPeriod;
    uint16 CfpMaxDuration;
    uint16 CfpDurationRemaining;
}
PACK IEEEtypes_CfParamSet_t;
//
// CF parameter set that contains the set of parameters necessary to
// support the PCF
//

typedef struct IEEEtypes_Tim_t
{
    IEEEtypes_ElementId_e ElementId;
    IEEEtypes_Len_t Len;
    uint8 DtimCnt;
    uint8 DtimPeriod;
    uint8 BitmapCtl;
    uint8 PartialVirtualBitmap[IEEEtypes_BITMAP_SIZE];
}
PACK IEEEtypes_Tim_t;
//
// TIM, which contains:
// 1) DTIM count - how many beacons (including the current beacon
//    frame) appear before the next DTIM; a count of 0 indicates the
//    current TIM is the DTIM
//
// 2) DTIM period - indicates the number of beacon intervals between
//    successive DTIMs
//
// 3) Bitmap control - contains the traffic indicator bit associated
//    with association ID 0 - this is set to 1 for TIM elements with a
//    a value of 0 in the DTIM count field when one or more broadcast
//    or multicast frames are buffered at the AP. The remaining bits
//    of the field form the bitmap offset
//
// 4) Partial virtual bitmap - indicates which stations have messages
//    buffered at the AP, for which the AP is prepared to deliver at
//    the time the beacon frame is transmitted

typedef struct IEEEtypes_IbssParamSet_t
{
    IEEEtypes_ElementId_e ElementId;
    IEEEtypes_Len_t Len;
    uint16 AtimWindow;
}
PACK IEEEtypes_IbssParamSet_t;
//
// IBSS parameters necessary to support an IBSS
//

typedef struct IEEEtypes_ChallengeText_t
{
    IEEEtypes_ElementId_e ElementId;
    IEEEtypes_Len_t Len;
    uint8 Text[IEEEtypes_CHALLENGE_TEXT_SIZE];
}
PACK IEEEtypes_ChallengeText_t;
//
// The challenge text used in authentication exchanges
//

/*-------------------------*/
/* Management Frame Bodies */
/*-------------------------*/
typedef union IEEEtypes_PhyParamSet_t
{
    IEEEtypes_FhParamSet_t FhParamSet;
    IEEEtypes_DsParamSet_t DsParamSet;
} PACK IEEEtypes_PhyParamSet_t;
//
// The parameter set relevant to the PHY
//

typedef union IEEEtypes_SsParamSet_t
{
    IEEEtypes_CfParamSet_t CfParamSet;
    IEEEtypes_IbssParamSet_t IbssParamSet;
} PACK IEEEtypes_SsParamSet_t;
//
// Service set parameters - for a BSS supporting, PCF, the
// CF parameter set is used; for an independent BSS, the IBSS
// parameter set is used.
//
#ifdef ERP
typedef struct IEEEtypes_ExtSuppRatesElement_t
{
    IEEEtypes_ElementId_e ElementId;
    IEEEtypes_Len_t Len;
    IEEEtypes_DataRate_t Rates[IEEEtypes_MAX_DATA_RATES];
}
PACK IEEEtypes_ExtSuppRatesElement_t;

typedef struct IEEEtypes_ERPInfo_t
{
    uint8 NonERPPresent:1;
    uint8 UseProtection:1;
    uint8 BarkerPreamble:1;
    uint8 reserved:5;
}
PACK IEEEtypes_ERPInfo_t;

typedef struct IEEEtypes_ERPInfoElement_t
{
    IEEEtypes_ElementId_e ElementId;
    IEEEtypes_Len_t Len;
    IEEEtypes_ERPInfo_t ERPInfo;
}
PACK IEEEtypes_ERPInfoElement_t;
#endif

#ifdef WPA

// Authentication Key Mgmt Suite Selector
#define IEEEtypes_RSN_AUTH_KEY_SUITE_RSVD      0
#define IEEEtypes_RSN_AUTH_KEY_SUITE_8021X     1
#define IEEEtypes_RSN_AUTH_KEY_SUITE_PSK       2

// Cipher Suite Selector
#define IEEEtypes_RSN_CIPHER_SUITE_NONE     0
#define IEEEtypes_RSN_CIPHER_SUITE_WEP40    1
#define IEEEtypes_RSN_CIPHER_SUITE_TKIP     2
#define IEEEtypes_RSN_CIPHER_SUITE_WRAP     3
#define IEEEtypes_RSN_CIPHER_SUITE_CCMP     4
#define IEEEtypes_RSN_CIPHER_SUITE_WEP104   5

#define MAX_SIZE_RSN_IE_BUF 32  // number of bytes

typedef struct IEEEtypes_RSN_IE_t
{
    uint8 ElemId;
    uint8 Len;
    uint8   OuiType[4];    //00:50:f2:01
    uint8   Ver[2];
    uint8 GrpKeyCipher[4];
    uint8   PwsKeyCnt[2];
    uint8 PwsKeyCipherList[4];
    uint8   AuthKeyCnt[2];
    uint8 AuthKeyList[4];
    //uint8   RsnCap[2];
} PACK IEEEtypes_RSN_IE_t;
#endif


typedef struct IEEEtypes_Bcn_t
{
    IEEEtypes_TimeStamp_t TimeStamp;
    IEEEtypes_BcnInterval_t BcnInterval;
    IEEEtypes_CapInfo_t CapInfo;
    IEEEtypes_SsIdElement_t SsId;
    IEEEtypes_SuppRatesElement_t SuppRates;
    IEEEtypes_PhyParamSet_t PhyParamSet;
    IEEEtypes_SsParamSet_t SsParamSet;
#ifdef WPA  // put RSN IE before TIM due to HW limitation
    IEEEtypes_RSN_IE_t RsnIE;
#endif
    IEEEtypes_Tim_t Tim;
#ifdef ERP

    IEEEtypes_ERPInfoElement_t ERPInfo;
    IEEEtypes_ExtSuppRatesElement_t ExtSuppRates;
#endif
}
PACK IEEEtypes_Bcn_t;
//
// Beacon message body
//

typedef struct IEEEtypes_DisAssoc_t
{
    IEEEtypes_ReasonCode_t ReasonCode;
}
PACK IEEEtypes_DisAssoc_t;
//
// Disassociation message body
//

typedef struct IEEEtypes_AssocRqst_t
{
    IEEEtypes_CapInfo_t CapInfo;
    IEEEtypes_ListenInterval_t ListenInterval;
    IEEEtypes_SsIdElement_t SsId;
    IEEEtypes_SuppRatesElement_t SuppRates;
#ifdef ERP

    IEEEtypes_ExtSuppRatesElement_t ExtSuppRates;
#endif

#ifdef WPA

    IEEEtypes_RSN_IE_t RsnIE;
#endif
}
PACK IEEEtypes_AssocRqst_t;
//
// Association request message body
//

typedef struct IEEEtypes_AssocRsp_t
{
    IEEEtypes_CapInfo_t CapInfo;
    IEEEtypes_StatusCode_t StatusCode;
    IEEEtypes_AId_t AId;
    IEEEtypes_SuppRatesElement_t SuppRates;
#ifdef ERP

    IEEEtypes_ExtSuppRatesElement_t ExtSuppRates;
#endif
}
PACK IEEEtypes_AssocRsp_t;
//
// Association response message body
//

typedef struct IEEEtypes_ReassocRqst_t
{
    IEEEtypes_CapInfo_t CapInfo;
    IEEEtypes_ListenInterval_t ListenInterval;
    IEEEtypes_MacAddr_t CurrentApAddr;
    IEEEtypes_SsIdElement_t SsId;
    IEEEtypes_SuppRatesElement_t SuppRates;
#ifdef WPA
    IEEEtypes_RSN_IE_t RsnIE;
#endif
#ifdef ERP

    IEEEtypes_ExtSuppRatesElement_t ExtSuppRates;
#endif

}
PACK IEEEtypes_ReassocRqst_t;
//
// Reassociation request message body
//

typedef struct IEEEtypes_ReassocRsp_t
{
    IEEEtypes_CapInfo_t CapInfo;
    IEEEtypes_StatusCode_t StatusCode;
    IEEEtypes_AId_t AId;
    IEEEtypes_SuppRatesElement_t SuppRates;
#ifdef ERP

    IEEEtypes_ExtSuppRatesElement_t ExtSuppRates;
#endif

}
PACK IEEEtypes_ReassocRsp_t;
//
// Reassociation response message body
//

typedef struct IEEEtypes_ProbeRqst_t
{
    IEEEtypes_SsIdElement_t SsId;
    IEEEtypes_SuppRatesElement_t SuppRates;
#ifdef ERP

    IEEEtypes_ExtSuppRatesElement_t ExtSuppRates;
#endif

}
PACK IEEEtypes_ProbeRqst_t;
//
// Probe request message body
//

typedef struct IEEEtypes_ProbeRsp_t
{
    IEEEtypes_TimeStamp_t TimeStamp;
    IEEEtypes_BcnInterval_t BcnInterval;
    IEEEtypes_CapInfo_t CapInfo;
    IEEEtypes_SsIdElement_t SsId;
    IEEEtypes_SuppRatesElement_t SuppRates;
    IEEEtypes_PhyParamSet_t PhyParamSet;
    IEEEtypes_SsParamSet_t SsParamSet;
    IEEEtypes_Tim_t Tim;
#ifdef WPA

    IEEEtypes_RSN_IE_t RsnIE;
#endif
#ifdef ERP

    IEEEtypes_ERPInfoElement_t ERPInfo;
    IEEEtypes_ExtSuppRatesElement_t ExtSuppRates;
#endif

}
PACK IEEEtypes_ProbeRsp_t;
//
// Probe response message body
//

typedef struct IEEEtypes_Auth_t
{
    IEEEtypes_AuthAlg_t AuthAlg;
    IEEEtypes_AuthTransSeq_t AuthTransSeq;
    IEEEtypes_StatusCode_t StatusCode;
    IEEEtypes_ChallengeText_t ChallengeText;
}
PACK IEEEtypes_Auth_t;
//
// Authentication message body
//

typedef struct IEEEtypes_Deauth_t
{
    IEEEtypes_ReasonCode_t ReasonCode;
}
PACK IEEEtypes_Deauth_t;
//
// Deauthentication message body
//

/*---------------------------------------------------------------------------*/
/*              IEEE 802.11 MLME SAP Interface Data Structures               */
/*                                                                           */
/* According to IEEE 802.11, services are provided by the MLME to the SME.   */
/* In the current architecture, the services are provided to the SME by the  */
/* MAC Management Service Task. This section describes data structures       */
/* needed for these services.                                                */
/*---------------------------------------------------------------------------*/

/*---------------------*/
/* BSS Description Set */
/*---------------------*/
typedef struct IEEEtypes_BssDesc_t
{
    IEEEtypes_MacAddr_t BssId;
    IEEEtypes_SsId_t SsId;
    IEEEtypes_Bss_e BssType;
    IEEEtypes_BcnInterval_t BcnPeriod;
    IEEEtypes_DtimPeriod_t DtimPeriod;
    IEEEtypes_TimeStamp_t Tstamp;
    IEEEtypes_TimeStamp_t StartTs;
    IEEEtypes_PhyParamSet_t PhyParamSet;
    IEEEtypes_SsParamSet_t SsParamSet;
    IEEEtypes_CapInfo_t Cap;
    IEEEtypes_DataRate_t DataRates[IEEEtypes_MAX_DATA_RATES];
}
PACK IEEEtypes_BssDesc_t;
//
// A description of a BSS, providing the following:
// BssId:        The ID of the BSS
// SsId:         The SSID of the BSS
// BssType:      The type of the BSS (INFRASTRUCTURE or INDEPENDENT)
// BcnPeriod:    The beacon period (in time units)
// DtimPeriod:   The DTIM period (in beacon periods)
// Tstamp:       Timestamp of a received frame from the BSS; this is an 8
//                  byte string from a probe response or beacon
// StartTs:      The value of a station's timing synchronization function
//                  at the start of reception of the first octet of the
//                  timestamp field of a received frame (probe response or
//                  beacon) from a BSS; this is an 8 byte string
// PhyParamSet:  The parameter set relevant to the PHY (empty if not
//                  needed by the PHY)
// SsParamSet:   The service set parameters. These can consist of either
//                  the parameter set for CF periods or for an IBSS.
// Cap:          The advertised capabilities of the BSS
// DataRates:    The set of data rates that must be supported by all
//                  stations (the BSS basic rate set)
//

typedef struct IEEEtypes_BssDescSet_t
{
    uint8 NumSets;
    IEEEtypes_BssDesc_t BssDesc[IEEEtypes_MAX_BSS_DESCRIPTS];
}
PACK IEEEtypes_BssDescSet_t;
//
// The set of BSS descriptions
//

/*-------------------*/
/* MLME SAP Messages */
/*-------------------*/
typedef struct IEEEtypes_PwrMgmtCmd_t
{
    IEEEtypes_PwrMgmtMode_e PwrMgmtMode;
    Boolean WakeUp;
    Boolean RcvDTIMs;
}
PACK IEEEtypes_PwrMgmtCmd_t;
//
// Power management request message from the SME
//

typedef struct IEEEtypes_PwrMgmtCfrm_t
{
    IEEEtypes_PwrMgmtResult_e Result;
}
PACK IEEEtypes_PwrMgmtCfrm_t;
//
// Power management confirm message sent from the MLME as a result
// of a power management request; it is sent after the change has
// taken place
//

typedef struct IEEEtypes_ScanCmd_t
{
    IEEEtypes_Bss_e BssType;
    IEEEtypes_MacAddr_t BssId;
    IEEEtypes_SsId_t SsId;
    IEEEtypes_Scan_e ScanType;
    uint16 ProbeDelay;
    uint8 ChanList[IEEEtypes_MAX_CHANNELS];
    //uint8                Reserved;
    uint16 MinChanTime;
    uint16 MaxChanTime;
}
PACK IEEEtypes_ScanCmd_t;
//
// Scan request message from the SME to determine if there are BSSs
// that can be joined
//
// Note: The "Reserved" field is inserted for alignment for
// commands coming from the host
//

typedef struct IEEEtypes_ScanCfrm_t
{
    IEEEtypes_BssDescSet_t BssDescSet;
    IEEEtypes_ScanResult_e Result;
}
PACK IEEEtypes_ScanCfrm_t;
//
// Scan confirm message sent from the MLME as a result of a scan request;
// it reports the results of the scan
//

typedef struct IEEEtypes_JoinCmd_t
{
    IEEEtypes_BssDesc_t BssDesc;
    uint16 FailTimeout;
    uint16 ProbeDelay;
    IEEEtypes_DataRate_t OpRateSet[IEEEtypes_MAX_DATA_RATES];
}
PACK IEEEtypes_JoinCmd_t;
//
// Join request message from the SME to establish synchronization with
// a BSS
//

typedef struct IEEEtypes_JoinCfrm_t
{
    IEEEtypes_JoinResult_e Result;
}
PACK IEEEtypes_JoinCfrm_t;
//
// Join confirm message sent from the MLME as a result of a join request;
// it reports the result of the join
//

typedef struct IEEEtypes_AuthCmd_t
{
    IEEEtypes_MacAddr_t PeerStaAddr;
    uint16 FailTimeout;
    IEEEtypes_AuthType_e AuthType;
}
PACK IEEEtypes_AuthCmd_t;
//
// Authenticate request message sent from the SME to establish
// authentication with a specified peer MAC entity
//

typedef struct IEEEtypes_AuthCfrm_t
{
    IEEEtypes_MacAddr_t PeerStaAddr;
    IEEEtypes_AuthType_e AuthType;
    IEEEtypes_AuthResult_e Result;
}
PACK IEEEtypes_AuthCfrm_t;
//
// Authenticate confirm message sent from the MLME as a result of an
// authenticate request; it reports the result of the authentication
//

typedef struct IEEEtypes_AuthInd_t
{
    IEEEtypes_MacAddr_t PeerStaAddr;
    IEEEtypes_AuthType_e AuthType;
}
PACK IEEEtypes_AuthInd_t;
//
// Authenticate indication message sent from the MLME to report
// authentication with a peer MAC entity that resulted from an
// authentication procedure that was initiated by that MAC entity
//

typedef struct IEEEtypes_DeauthCmd_t
{
    IEEEtypes_MacAddr_t PeerStaAddr;
    IEEEtypes_ReasonCode_t Reason;
}
PACK IEEEtypes_DeauthCmd_t;
//
// Deauthenticate request message sent from the SME to invalidate
// authentication with a specified peer MAC entity
//

typedef struct IEEEtypes_DeauthCfrm_t
{
    IEEEtypes_MacAddr_t PeerStaAddr;
    IEEEtypes_DeauthResult_e Result;
}
PACK IEEEtypes_DeauthCfrm_t;
//
// Deauthenticate confirm message sent from the MLME as a result of a
// deauthenticate request message; it reports the result of the
// deauthentication
//

typedef struct IEEEtypes_DeauthInd_t
{
    IEEEtypes_MacAddr_t PeerStaAddr;
    IEEEtypes_ReasonCode_t Reason;
}
PACK IEEEtypes_DeauthInd_t;
//
// Deauthentication indication message sent from the MLME to report
// invalidation of an authentication with a peer MAC entity; the message
// is generated as a result of an invalidation of the authentication
//

typedef struct IEEEtypes_AssocCmd_t
{
    IEEEtypes_MacAddr_t PeerStaAddr;
    uint16 FailTimeout;
    IEEEtypes_CapInfo_t CapInfo;
    IEEEtypes_ListenInterval_t ListenInterval;
}
PACK IEEEtypes_AssocCmd_t;
//
// Association request message sent from the SME to establish an
// association with an AP
//

typedef struct IEEEtypes_AssocCfrm_t
{
    IEEEtypes_AssocResult_e Result;
}
PACK IEEEtypes_AssocCfrm_t;
//
// Association confirm message sent from the MLME as a result of an
// association request message; it reports the result of the assoication
//

typedef struct IEEEtypes_AssocInd_t
{
    IEEEtypes_MacAddr_t PeerStaAddr;
}
PACK IEEEtypes_AssocInd_t;
//
// Association indication message sent from the MLME to report an
// association with a specified peer MAC entity acting as an AP; the
// indication is the result of an association procedure that was
// initiated by the peer MAC entity
//

typedef struct IEEEtypes_ReassocCmd_t
{
    IEEEtypes_MacAddr_t NewApAddr;
    uint16 FailTimeout;
    IEEEtypes_CapInfo_t CapInfo;
    IEEEtypes_ListenInterval_t ListenInterval;
}
PACK IEEEtypes_ReassocCmd_t;
//
// Reassociation request message sent from the SME to change association
// to a specified new peer MAC entity acting as an AP
//

typedef struct IEEEtypes_ReassocCfrm_t
{
    IEEEtypes_ReassocResult_e Result;
}
PACK IEEEtypes_ReassocCfrm_t;
//
// Reassociation confirm message sent from the MLME as the result of a
// reassociate request message; it reports the result of the reassociation
//

typedef struct IEEEtypes_ReassocInd_t
{
    IEEEtypes_MacAddr_t PeerStaAddr;
}
PACK IEEEtypes_ReassocInd_t;
//
// Reassociate indication message sent from the MLME to report a
// reassociation with a specified peer MAC entity; the
// indication is the result of a reassociation procedure that was
// initiated by the peer MAC entity
//

typedef struct IEEEtypes_DisassocCmd_t
{
    IEEEtypes_MacAddr_t PeerStaAddr;
    IEEEtypes_ReasonCode_t Reason;
}
PACK IEEEtypes_DisassocCmd_t;
//
// Disassociate request message sent from the SME to establish
// disassociation with an AP
//

typedef struct IEEEtypes_DisassocCfrm_t
{
    IEEEtypes_DisassocResult_e Result;
}
PACK IEEEtypes_DisassocCfrm_t;
//
// Disassociate confirm message sent from the MLME as a result of a
// disassociate request message; it reports the result of the
// disassociation
//

typedef struct IEEEtypes_DisassocInd_t
{
    IEEEtypes_MacAddr_t PeerStaAddr;
    IEEEtypes_ReasonCode_t Reason;
}
PACK IEEEtypes_DisassocInd_t;
//
// Disassociate indication message sent from the MLME to report the
// invalidation of an association relationship with a peer MAC entity;
// the message is generated as a result of an invalidation of an
// association relationship
//

typedef struct IEEEtypes_ResetCmd_t
{
    IEEEtypes_MacAddr_t StaAddr;
    Boolean SetDefaultMIB;
}
PACK IEEEtypes_ResetCmd_t;
//
// Reset request message sent from the SME to reset the MAC to initial
// conditions; the reset must be used prior to a start command
//

typedef struct IEEEtypes_ResetCfrm_t
{
    IEEEtypes_ResetResult_e Result;
}
PACK IEEEtypes_ResetCfrm_t;
//
// Reset confirm message sent from the MLME as a result of a reset
// request message; it reports the result of the reset
//

typedef struct IEEEtypes_StartCmd_t
{
    IEEEtypes_SsId_t SsId;
    IEEEtypes_Bss_e BssType;
    IEEEtypes_BcnInterval_t BcnPeriod;
    IEEEtypes_DtimPeriod_t DtimPeriod;
    IEEEtypes_SsParamSet_t SsParamSet;
    IEEEtypes_PhyParamSet_t PhyParamSet;
    uint16 ProbeDelay;
    IEEEtypes_CapInfo_t CapInfo;
#ifndef ERP

    IEEEtypes_DataRate_t BssBasicRateSet[IEEEtypes_MAX_DATA_RATES];
    IEEEtypes_DataRate_t OpRateSet[IEEEtypes_MAX_DATA_RATES];
#else

    IEEEtypes_DataRate_t BssBasicRateSet[IEEEtypes_MAX_DATA_RATES_G];
    IEEEtypes_DataRate_t OpRateSet[IEEEtypes_MAX_DATA_RATES_G];
#endif
}
PACK IEEEtypes_StartCmd_t;
//
// Start request message sent from the SME to start a new BSS; the BSS
// may be either an infrastructure BSS (with the MAC entity acting as the
// AP) or an independent BSS (with the MAC entity acting as the first
// station in the IBSS)
//

typedef struct IEEEtypes_StartCfrm_t
{
    IEEEtypes_StartResult_e Result;
}
PACK IEEEtypes_StartCfrm_t;
//
// Start confirm message sent from the MLME as a result of a start request
// message; it reports the results of the BSS creation procedure
//

typedef struct IEEEtypes_Frame_t
{
    IEEEtypes_GenHdr_t Hdr;
    unsigned char Body[8];
} PACK IEEEtypes_Frame_t;

typedef struct ether_hdr_t
{
    IEEEtypes_MacAddr_t  da;
    IEEEtypes_MacAddr_t  sa;
    unsigned short    type;
} PACK ether_hdr_t;


typedef struct IEEEtypes_8023_Frame_t
{
    ether_hdr_t   Hdr;
    unsigned char         Body[1600-14];
}PACK IEEEtypes_8023_Frame_t;

#endif /* _IEEE_TYPES_H_ */


