#ifndef _API_H_
#define _API_H_ 
/*
 *                Copyright 2003, Marvell Semiconductor, Inc.
 * This code contains confidential information of Marvell Semiconductor, Inc.
 * No rights are granted herein under any patent, mask work right or copyright
 * of Marvell or any third party.
 * Marvell reserves the right at its sole discretion to request that this code
 * be immediately returned to Marvell. This code is provided "as is".
 * Marvell makes no warranties, express, implied or otherwise, regarding its
 * accuracy, completeness or performance.
 */

/*!
 * \file    api.h
 * \brief   This file contains the function prototypes and definitions for the
 *          AP commands API
 *
 */

/* Use these macros for port name */
/* Wireless LAN port */
#define PORT_NAME_WLAN "wl0" 
/* WAN logical port attached to physical port eth0 */
#define PORT_NAME_WAN  "wan0" 
/* LAN logical port attached to physical port eth0 */
#define PORT_NAME_LAN  "lan0" 
/** DMZ logical port **/
#define PORT_NAME_DMZ "dmz0"


/* GPIO number cannot exceed this limit */
#define MAX_GPIO_NUM 15

extern uint32 CheckAvailablePool(void);
extern uint32 NoPoolBufAvailable;
void Wlan_Radio(uint8 option);


//=============================================================================
//                          API for Sending Commands
//=============================================================================

typedef	enum
{
    APCMD_BSS_START = 0x00000001,
    APCMD_BSS_START_REPLY,
    APCMD_GET_SSID ,
    APCMD_GET_SSID_REPLY,
    APCMD_SET_SSID ,
    APCMD_SET_SSID_REPLY,
    APCMD_GET_OPEN_SSID_EN_STATE ,
    APCMD_GET_OPEN_SSID_EN_STATE_REPLY,
    APCMD_SET_OPEN_SSID_EN_STATE,
    APCMD_SET_OPEN_SSID_EN_STATE_REPLY,
    APCMD_GET_RF_MAC_ADDRESS,
    APCMD_GET_RF_MAC_ADDRESS_REPLY,
    APCMD_SET_RF_MAC_ADDRESS,
    APCMD_SET_RF_MAC_ADDRESS_REPLY,
    APCMD_GET_RF_CHANNEL,
    APCMD_GET_RF_CHANNEL_REPLY,
    APCMD_SET_RF_CHANNEL,
    APCMD_SET_RF_CHANNEL_REPLY,
    APCMD_GET_BASIC_DATA_RATES,
    APCMD_GET_BASIC_DATA_RATES_REPLY,
    APCMD_SET_BASIC_DATA_RATES,
    APCMD_SET_BASIC_DATA_RATES_REPLY,
    APCMD_GET_SUP_DATA_RATES,
    APCMD_GET_SUP_DATA_RATES_REPLY,
    APCMD_SET_SUP_DATA_RATES,
    APCMD_SET_SUP_DATA_RATES_REPLY,
    APCMD_GET_TX_DATA_RATE,
    APCMD_GET_TX_DATA_RATE_REPLY,
    APCMD_SET_TX_DATA_RATE,
    APCMD_SET_TX_DATA_RATE_REPLY,
    APCMD_GET_TX_POWER_LEVEL,
    APCMD_GET_TX_POWER_LEVEL_REPLY,
    APCMD_SET_TX_POWER_LEVEL,
    APCMD_SET_TX_POWER_LEVEL_REPLY,
    APCMD_GET_RF_PREAMBLE_OPTION,
    APCMD_GET_RF_PREAMBLE_OPTION_REPLY,
    APCMD_SET_RF_PREAMBLE_OPTION,
    APCMD_SET_RF_PREAMBLE_OPTION_REPLY,
    APCMD_GET_ANTENNA_MODE,
    APCMD_GET_ANTENNA_MODE_REPLY,
    APCMD_SET_ANTENNA_MODE,
    APCMD_SET_ANTENNA_MODE_REPLY,
    APCMD_GET_FRAG_THRESHOLD,
    APCMD_GET_FRAG_THRESHOLD_REPLY,
    APCMD_SET_FRAG_THRESHOLD,
    APCMD_SET_FRAG_THRESHOLD_REPLY,
    APCMD_GET_RTS_THRESHOLD,
    APCMD_GET_RTS_THRESHOLD_REPLY,
    APCMD_SET_RTS_THRESHOLD,
    APCMD_SET_RTS_THRESHOLD_REPLY,
    APCMD_GET_RTS_RETRY_LIMIT,
    APCMD_GET_RTS_RETRY_LIMIT_REPLY,
    APCMD_SET_RTS_RETRY_LIMIT,
    APCMD_SET_RTS_RETRY_LIMIT_REPLY,
    APCMD_GET_DATA_RETRY_LIMIT,
    APCMD_GET_DATA_RETRY_LIMIT_REPLY,
    APCMD_SET_DATA_RETRY_LIMIT,
    APCMD_SET_DATA_RETRY_LIMIT_REPLY,
    APCMD_GET_BEACON_PERIOD,
    APCMD_GET_BEACON_PERIOD_REPLY,
    APCMD_SET_BEACON_PERIOD,
    APCMD_SET_BEACON_PERIOD_REPLY,
    APCMD_GET_DTIM_PERIOD,
    APCMD_GET_DTIM_PERIOD_REPLY,
    APCMD_SET_DTIM_PERIOD,
    APCMD_SET_DTIM_PERIOD_REPLY,
    APCMD_GET_802_11_STATS,
    APCMD_GET_802_11_STATS_REPLY,
    APCMD_ADD_WLAN_FILTER_ENTRY,
    APCMD_ADD_WLAN_FILTER_ENTRY_REPLY,
    APCMD_DEL_WLAN_FILTER_ENTRY,
    APCMD_DEL_WLAN_FILTER_ENTRY_REPLY,
    APCMD_GET_STA_LIST,
    APCMD_GET_STA_LIST_REPLY,
    APCMD_GET_STA_INFO,
    APCMD_GET_STA_INFO_REPLY,
    APCMD_DEAUTH_STA,
    APCMD_DEAUTH_STA_REPLY,
    APCMD_DISASSOC_STA,
    APCMD_DISASSOC_STA_REPLY,
    APCMD_ADD_ASSOC_FILTER_ENTRY,
    APCMD_ADD_ASSOC_FILTER_ENTRY_REPLY,
    APCMD_DEL_ASSOC_FILTER_ENTRY,
    APCMD_DEL_ASSOC_FILTER_ENTRY_REPLY,
    APCMD_GET_MAX_NUM_STA,
    APCMD_GET_MAX_NUM_STA_REPLY,
    APCMD_SET_MAX_NUM_STA,
    APCMD_SET_MAX_NUM_STA_REPLY,
    APCMD_GET_DEFAULT_WEP_KEYS,
    APCMD_GET_DEFAULT_WEP_KEYS_REPLY,
    APCMD_SET_DEFAULT_WEP_KEYS,
    APCMD_SET_DEFAULT_WEP_KEYS_REPLY,
    APCMD_SET_STA_WEP_KEY,
    APCMD_SET_STA_WEP_KEY_REPLY,
    APCMD_GET_PRIVACY_OPTION,
    APCMD_GET_PRIVACY_OPTION_REPLY,
    APCMD_SET_PRIVACY_OPTION,
    APCMD_SET_PRIVACY_OPTION_REPLY,
    APCMD_GET_AUTH_MODE,
    APCMD_GET_AUTH_MODE_REPLY,
    APCMD_SET_AUTH_MODE,
    APCMD_SET_AUTH_MODE_REPLY,
    APCMD_GET_ETH_MAC_ADDRESS,
    APCMD_GET_ETH_MAC_ADDRESS_REPLY,
    APCMD_GET_ETH_IP_CONFIG,
    APCMD_GET_ETH_IP_CONFIG_REPLY,
    APCMD_SET_ETH_IP_CONFIG,
    APCMD_SET_ETH_IP_CONFIG_REPLY,
#ifdef OS_VxW
    /* Added wireless lan and WAN ip info for VxWorks. */
    APCMD_GET_WL_IP_CONFIG,
    APCMD_GET_WL_IP_CONFIG_REPLY,
    APCMD_SET_WL_IP_CONFIG,
    APCMD_SET_WL_IP_CONFIG_REPLY,
    
    APCMD_GET_WAN_IP_CONFIG,
    APCMD_GET_WAN_IP_CONFIG_REPLY,
    APCMD_SET_WAN_IP_CONFIG,
    APCMD_SET_WAN_IP_CONFIG_REPLY,
#endif    
    APCMD_GET_ETH_PORT_STATUS,
    APCMD_GET_ETH_PORT_STATUS_REPLY,
    APCMD_SET_ETH_PORT_PARAMS,
    APCMD_SET_ETH_PORT_PARAMS_REPLY,
    APCMD_GET_ICMP_EN_STATE,
    APCMD_GET_ICMP_EN_STATE_REPLY,
    APCMD_SET_ICMP_EN_STATE,
    APCMD_SET_ICMP_EN_STATE_REPLY,
    APCMD_GET_802_3_STATES,
    APCMD_GET_802_3_STATES_REPLY,
    APCMD_READ_GPIO,
    APCMD_READ_GPIO_REPLY,
    APCMD_WRITE_GPIO,
    APCMD_WRITE_GPIO_REPLY,
    APCMD_SET_ETH_MAC_ADDRESS,
    APCMD_SET_ETH_MAC_ADDRESS_REPLY,
    APCMD_GET_WAN_MAC_ADDRESS,
    APCMD_GET_WAN_MAC_ADDRESS_REPLY,
    APCMD_SET_WAN_MAC_ADDRESS,
    APCMD_SET_WAN_MAC_ADDRESS_REPLY,
    APCMD_FW_SOFT_RESET,
    APCMD_FW_SOFT_RESET_REPLY,
    APCMD_GET_WLAN_FILTER_LIST,
    APCMD_GET_WLAN_FILTER_LIST_REPLY,
    APCMD_SET_WLAN_FILTER_LIST,
    APCMD_SET_WLAN_FILTER_LIST_REPLY,
    APCMD_GET_WLAN_FILTER_MODE,
    APCMD_GET_WLAN_FILTER_MODE_REPLY,
    APCMD_SET_WLAN_FILTER_MODE,
    APCMD_SET_WLAN_FILTER_MODE_REPLY,
    APCMD_UPDATE_FIRMWARE,
    APCMD_UPDATE_FIRMWARE_REPLY,
    APCMD_SAVE_USER_CONFIG,
    APCMD_SAVE_USER_CONFIG_REPLY,
    APCMD_SET_STA_AGING_TIME_IN_MINUTES,
    APCMD_SET_STA_AGING_TIME_IN_MINUTES_REPLY,
    APCMD_GET_FIRMWARE_VERSION,
    APCMD_GET_FIRMWARE_VERSION_REPLY,
    APCMD_SET_BEACON_TX_MODE,         /*on or off */
    APCMD_SET_BEACON_TX_MODE_REPLY,
    APCMD_SET_RATE_ADAPTATION_METHOD,
    APCMD_SET_RATE_ADAPTATION_METHOD_REPLY,
    APCMD_GET_RATE_ADAPTATION_PARAMS,
    APCMD_GET_RATE_ADAPTATION_PARAMS_REPLY,
    APCMD_SET_RATE_ADAPTATION_PARAMS,
    APCMD_SET_RATE_ADAPTATION_PARAMS_REPLY,
    APCMD_SET_SDRAM_BURST_MODE,
    APCMD_SET_SDRAM_BURST_MODE_REPLY,
    APCMD_GET_DEVICE_NAME,
    APCMD_GET_DEVICE_NAME_REPLY,
    APCMD_SET_DEVICE_NAME,
    APCMD_SET_DEVICE_NAME_REPLY,        
    
/* GET/SET Baseband Reg */
    APCMD_GET_BB_REG,
    APCMD_GET_BB_REG_REPLY,
    APCMD_SET_BB_REG,
    APCMD_SET_BB_REG_REPLY,
/* GET/SET Baseband Reg end */

    APCMD_GET_CUR_RADIO_TX_RATE,
    APCMD_GET_CUR_RADIO_TX_RATE_REPLY,
/* GET/SET 11g commands */
   APCMD_GET_BASIC_DATA_RATES_G,
   APCMD_GET_BASIC_DATA_RATES_G_REPLY,
   APCMD_SET_BASIC_DATA_RATES_G,
   APCMD_SET_BASIC_DATA_RATES_G_REPLY,
   APCMD_GET_TX_DATA_RATE_G,
   APCMD_GET_TX_DATA_RATE_G_REPLY,
   APCMD_SET_TX_DATA_RATE_G,
   APCMD_SET_TX_DATA_RATE_G_REPLY,
   APCMD_GET_AP_MODE,
   APCMD_GET_AP_MODE_REPLY,
   APCMD_SET_AP_MODE,
   APCMD_SET_AP_MODE_REPLY,
   APCMD_GET_STA_LIST_G,
   APCMD_GET_STA_LIST_G_REPLY,
   APCMD_GET_STA_INFO_G,
   APCMD_GET_STA_INFO_G_REPLY,
   /*GET/SET 11g commands end */
   APCMD_GET_ETH_CONFIG,
   APCMD_GET_ETH_CONFIG_REPLY,
   APCMD_SET_ETH_CONFIG,
   APCMD_SET_ETH_CONFIG_REPLY,
   APCMD_GET_PRIVACY_TBL_RSN_ENABLED,
   APCMD_GET_PRIVACY_TBL_RSN_ENABLED_REPLY,
   APCMD_SET_PRIVACY_TBL_RSN_ENABLED,
   APCMD_SET_PRIVACY_TBL_RSN_ENABLED_REPLY,
   APCMD_GET_RSN_CFG_AUTH_SUITE,
   APCMD_GET_RSN_CFG_AUTH_SUITE_REPLY,
   APCMD_SET_RSN_CFG_AUTH_SUITE,
   APCMD_SET_RSN_CFG_AUTH_SUITE_REPLY,
   APCMD_GET_RSN_CFG_UNICAST_CIPHERS,
   APCMD_GET_RSN_CFG_UNICAST_CIPHERS_REPLY,
   APCMD_SET_RSN_CFG_UNICAST_CIPHERS,
   APCMD_SET_RSN_CFG_UNICAST_CIPHERS_REPLY,
   APCMD_SET_RSN_CFG_MULTICAST_CIPHERS,
   APCMD_SET_RSN_CFG_MULTICAST_REPLY,
   APCMD_GET_RSN_CFG,
   APCMD_GET_RSN_CFG_REPLY,
   APCMD_SET_PSK_PASSPHRASE,
   APCMD_SET_PSK_PASSPHRASE_REPLY,
   APCMD_SET_GRP_REKEY_TIME,
   APCMD_SET_GRP_REKEY_TIME_REPLY,
   APCMD_GET_RSN_STATS,
   APCMD_GET_RSN_STATS_REPLY,

#if defined(WDS_FEATURE) || defined(AP31E) || defined(AP31CLIENT)
    APCMD_GET_WDS_MODE,
    APCMD_GET_WDS_MODE_REPLY,
    APCMD_SET_WDS_MODE,
    APCMD_SET_WDS_MODE_REPLY,
    APCMD_GET_WDS_LIST,
    APCMD_GET_WDS_LIST_REPLY,
    APCMD_SET_WDS_LIST,
    APCMD_SET_WDS_LIST_REPLY,
    APCMD_GET_DISABLE_ASSOC,
    APCMD_GET_DISABLE_ASSOC_REPLY,
    APCMD_SET_DISABLE_ASSOC,
    APCMD_SET_DISABLE_ASSOC_REPLY,
    /* WDS interop option */
    APCMD_GET_WDS_OPTION,
    APCMD_GET_WDS_OPTION_REPLY,
    APCMD_SET_WDS_OPTION,
    APCMD_SET_WDS_OPTION_REPLY,
#endif /* WDS_FEATURE*/

    APCMD_GET_FORCE_PROTECTION_MODE,
    APCMD_GET_FORCE_PROTECTION_MODE_REPLY,
    APCMD_SET_FORCE_PROTECTION_MODE,
    APCMD_SET_FORCE_PROTECTION_MODE_REPLY,

#if defined(WIPOD)
    /* WiPod 5-in-1 */
    APCMD_GET_WIPOD_MODE,
    APCMD_GET_WIPOD_MODE_REPLY,
    APCMD_SET_WIPOD_MODE,
    APCMD_SET_WIPOD_MODE_REPLY,
#endif

#if defined(BURST_MODE)
    APCMD_GET_BURST_MODE,
    APCMD_GET_BURST_MODE_REPLY,
    APCMD_SET_BURST_MODE,
    APCMD_SET_BURST_MODE_REPLY,
#endif

    /* Alway add items above this line */
} APCMD;

#if 0
typedef	enum
{
    APCMD_BSS_START = 0x00000001,
    APCMD_BSS_START_REPLY,
    APCMD_GET_SSID ,
    APCMD_GET_SSID_REPLY,
    APCMD_SET_SSID ,
    APCMD_SET_SSID_REPLY,
    APCMD_GET_OPEN_SSID_EN_STATE ,
    APCMD_GET_OPEN_SSID_EN_STATE_REPLY,
    APCMD_SET_OPEN_SSID_EN_STATE,
    APCMD_SET_OPEN_SSID_EN_STATE_REPLY,
    APCMD_GET_RF_MAC_ADDRESS,
    APCMD_GET_RF_MAC_ADDRESS_REPLY,
    APCMD_SET_RF_MAC_ADDRESS,
    APCMD_SET_RF_MAC_ADDRESS_REPLY,
    APCMD_GET_RF_CHANNEL,
    APCMD_GET_RF_CHANNEL_REPLY,
    APCMD_SET_RF_CHANNEL,
    APCMD_SET_RF_CHANNEL_REPLY,
    APCMD_GET_BASIC_DATA_RATES,
    APCMD_GET_BASIC_DATA_RATES_REPLY,
    APCMD_SET_BASIC_DATA_RATES,
    APCMD_SET_BASIC_DATA_RATES_REPLY,
    APCMD_GET_SUP_DATA_RATES,
    APCMD_GET_SUP_DATA_RATES_REPLY,
    APCMD_SET_SUP_DATA_RATES,
    APCMD_SET_SUP_DATA_RATES_REPLY,
    APCMD_GET_TX_DATA_RATE,
    APCMD_GET_TX_DATA_RATE_REPLY,
    APCMD_SET_TX_DATA_RATE,
    APCMD_SET_TX_DATA_RATE_REPLY,
    APCMD_GET_TX_POWER_LEVEL,
    APCMD_GET_TX_POWER_LEVEL_REPLY,
    APCMD_SET_TX_POWER_LEVEL,
    APCMD_SET_TX_POWER_LEVEL_REPLY,
    APCMD_GET_RF_PREAMBLE_OPTION,
    APCMD_GET_RF_PREAMBLE_OPTION_REPLY,
    APCMD_SET_RF_PREAMBLE_OPTION,
    APCMD_SET_RF_PREAMBLE_OPTION_REPLY,
    APCMD_GET_ANTENNA_MODE,
    APCMD_GET_ANTENNA_MODE_REPLY,
    APCMD_SET_ANTENNA_MODE,
    APCMD_SET_ANTENNA_MODE_REPLY,
    APCMD_GET_FRAG_THRESHOLD,
    APCMD_GET_FRAG_THRESHOLD_REPLY,
    APCMD_SET_FRAG_THRESHOLD,
    APCMD_SET_FRAG_THRESHOLD_REPLY,
    APCMD_GET_RTS_THRESHOLD,
    APCMD_GET_RTS_THRESHOLD_REPLY,
    APCMD_SET_RTS_THRESHOLD,
    APCMD_SET_RTS_THRESHOLD_REPLY,
    APCMD_GET_RTS_RETRY_LIMIT,
    APCMD_GET_RTS_RETRY_LIMIT_REPLY,
    APCMD_SET_RTS_RETRY_LIMIT,
    APCMD_SET_RTS_RETRY_LIMIT_REPLY,
    APCMD_GET_DATA_RETRY_LIMIT,
    APCMD_GET_DATA_RETRY_LIMIT_REPLY,
    APCMD_SET_DATA_RETRY_LIMIT,
    APCMD_SET_DATA_RETRY_LIMIT_REPLY,
    APCMD_GET_BEACON_PERIOD,
    APCMD_GET_BEACON_PERIOD_REPLY,
    APCMD_SET_BEACON_PERIOD,
    APCMD_SET_BEACON_PERIOD_REPLY,
    APCMD_GET_DTIM_PERIOD,
    APCMD_GET_DTIM_PERIOD_REPLY,
    APCMD_SET_DTIM_PERIOD,
    APCMD_SET_DTIM_PERIOD_REPLY,
    APCMD_GET_802_11_STATS,
    APCMD_GET_802_11_STATS_REPLY,
    APCMD_ADD_WLAN_FILTER_ENTRY,
    APCMD_ADD_WLAN_FILTER_ENTRY_REPLY,
    APCMD_DEL_WLAN_FILTER_ENTRY,
    APCMD_DEL_WLAN_FILTER_ENTRY_REPLY,
    APCMD_GET_STA_LIST,
    APCMD_GET_STA_LIST_REPLY,
    APCMD_GET_STA_INFO,
    APCMD_GET_STA_INFO_REPLY,
    APCMD_DEAUTH_STA,
    APCMD_DEAUTH_STA_REPLY,
    APCMD_DISASSOC_STA,
    APCMD_DISASSOC_STA_REPLY,
    APCMD_ADD_ASSOC_FILTER_ENTRY,
    APCMD_ADD_ASSOC_FILTER_ENTRY_REPLY,
    APCMD_DEL_ASSOC_FILTER_ENTRY,
    APCMD_DEL_ASSOC_FILTER_ENTRY_REPLY,
    APCMD_GET_MAX_NUM_STA,
    APCMD_GET_MAX_NUM_STA_REPLY,
    APCMD_SET_MAX_NUM_STA,
    APCMD_SET_MAX_NUM_STA_REPLY,
    APCMD_GET_DEFAULT_WEP_KEYS,
    APCMD_GET_DEFAULT_WEP_KEYS_REPLY,
    APCMD_SET_DEFAULT_WEP_KEYS,
    APCMD_SET_DEFAULT_WEP_KEYS_REPLY,
    APCMD_SET_STA_WEP_KEY,
    APCMD_SET_STA_WEP_KEY_REPLY,
    APCMD_GET_PRIVACY_OPTION,
    APCMD_GET_PRIVACY_OPTION_REPLY,
    APCMD_SET_PRIVACY_OPTION,
    APCMD_SET_PRIVACY_OPTION_REPLY,
    APCMD_GET_AUTH_MODE,
    APCMD_GET_AUTH_MODE_REPLY,
    APCMD_SET_AUTH_MODE,
    APCMD_SET_AUTH_MODE_REPLY,
    APCMD_GET_ETH_MAC_ADDRESS,
    APCMD_GET_ETH_MAC_ADDRESS_REPLY,
    APCMD_GET_ETH_IP_CONFIG,
    APCMD_GET_ETH_IP_CONFIG_REPLY,
    APCMD_SET_ETH_IP_CONFIG,
    APCMD_SET_ETH_IP_CONFIG_REPLY,
    APCMD_GET_ETH_PORT_STATUS,
    APCMD_GET_ETH_PORT_STATUS_REPLY,
    APCMD_SET_ETH_PORT_PARAMS,
    APCMD_SET_ETH_PORT_PARAMS_REPLY,
    APCMD_GET_ICMP_EN_STATE,
    APCMD_GET_ICMP_EN_STATE_REPLY,
    APCMD_SET_ICMP_EN_STATE,
    APCMD_SET_ICMP_EN_STATE_REPLY,
    APCMD_GET_802_3_STATES,
    APCMD_GET_802_3_STATES_REPLY,
    APCMD_READ_GPIO,
    APCMD_READ_GPIO_REPLY,
    APCMD_WRITE_GPIO,
    APCMD_WRITE_GPIO_REPLY,
    APCMD_SET_ETH_MAC_ADDRESS,
    APCMD_SET_ETH_MAC_ADDRESS_REPLY,
    APCMD_GET_WAN_MAC_ADDRESS,
    APCMD_GET_WAN_MAC_ADDRESS_REPLY,
    APCMD_SET_WAN_MAC_ADDRESS,
    APCMD_SET_WAN_MAC_ADDRESS_REPLY,
    APCMD_FW_SOFT_RESET,
    APCMD_FW_SOFT_RESET_REPLY,
    APCMD_GET_WLAN_FILTER_LIST,
    APCMD_GET_WLAN_FILTER_LIST_REPLY,
    APCMD_SET_WLAN_FILTER_LIST,
    APCMD_SET_WLAN_FILTER_LIST_REPLY,
    APCMD_SET_WLAN_RADIO,
    APCMD_SET_WLAN_RADIO_REPLY,
    APCMD_SET_STA_AGING_TIME_IN_MINUTES,
    APCMD_SET_STA_AGING_TIME_IN_MINUTES_REPLY,
#ifdef CONDOR
    APCMD_GET_BASIC_DATA_RATES_G,
    APCMD_GET_BASIC_DATA_RATES_G_REPLY,
    APCMD_SET_BASIC_DATA_RATES_G,
    APCMD_SET_BASIC_DATA_RATES_G_REPLY,
    APCMD_GET_TX_DATA_RATE_G,
    APCMD_GET_TX_DATA_RATE_G_REPLY,
    APCMD_SET_TX_DATA_RATE_G,
    APCMD_SET_TX_DATA_RATE_G_REPLY,
    APCMD_GET_AP_MODE,
    APCMD_GET_AP_MODE_REPLY,
    APCMD_SET_AP_MODE,
    APCMD_SET_AP_MODE_REPLY,
#endif
    APCMD_SET_SDRAM_BURST_MODE,
    APCMD_SET_SDRAM_BURST_MODE_REPLY,
#ifdef WPA
    /* 802.11i MIB supported */
    APCMD_GET_STA_CFG_RSN_OPTION_IMP,
    APCMD_GET_STA_CFG_RSN_OPTION_IMP_REPLY,
    APCMD_GET_STA_CFG_TKIP_NUM_RPLY_COUNTERS,
    APCMD_GET_STA_CFG_TKIP_NUM_RPLY_COUNTERS_REPLY,
    APCMD_GET_PRIVACY_TBL_RSN_ENABLED,
    APCMD_GET_PRIVACY_TBL_RSN_ENABLED_REPLY,
    APCMD_SET_PRIVACY_TBL_RSN_ENABLED,
    APCMD_SET_PRIVACY_TBL_RSN_ENABLED_REPLY,
    APCMD_GET_RSN_CFG_TBL,
    APCMD_GET_RSN_CFG_TBL_REPLY,
    APCMD_SET_RSN_CFG_TBL,
    APCMD_SET_RSN_CFG_TBL_REPLY,
    APCMD_GET_RSN_CFG_UNICAST_CIPHER,
    APCMD_GET_RSN_CFG_UNICAST_CIPHER_REPLY,
    APCMD_SET_RSN_CFG_UNICAST_CIPHER,
    APCMD_SET_RSN_CFG_UNICAST_CIPHER_REPLY,
    APCMD_GET_RSN_CFG_AUTH_SUITE,
    APCMD_GET_RSN_CFG_AUTH_SUITE_REPLY,
    APCMD_SET_RSN_CFG_AUTH_SUITE,
    APCMD_SET_RSN_CFG_AUTH_SUITE_REPLY,
    APCMD_GET_RSN_STATS,
    APCMD_GET_RSN_STATS_REPLY,
    /* End of 802.11i MIB supported */

    /* Marvell private MIB */
    APCMD_SET_RSN_DATA_TRAFFIC_ENABLED,
    APCMD_SET_RSN_DATA_TRAFFIC_ENABLED_REPLY,
    APCMD_SET_RSN_PWK,
    APCMD_SET_RSN_PWK_REPLY,
    APCMD_GET_RSN_PWK,
    APCMD_GET_RSN_PWK_REPLY,
    APCMD_SET_RSN_GRP_KEY,
    APCMD_SET_RSN_GRP_KEY_REPLY,
    APCMD_GET_RSN_GRP_KEY,
    APCMD_GET_RSN_GRP_KEY_REPLY,
    APCMD_GET_STA_ASSOC_STATE,
    APCMD_GET_STA_ASSOC_STATE_REPLY,
    APCMD_GET_THIS_STA_RSN_IE,
    APCMD_GET_THIS_STA_RSN_IE_REPLY,
    APCMD_GET_EXT_STA_RSN_IE,
    APCMD_GET_EXT_STA_RSN_IE_REPLY,
    APCMD_GET_PAIRWISE_TSC,
    APCMD_GET_PAIRWISE_TSC_REPLY,
    APCMD_SET_PAIRWISE_TSC,
    APCMD_SET_PAIRWISE_TSC_REPLY,
    APCMD_GET_GRP_TSC,
    APCMD_GET_GRP_TSC_REPLY,
    APCMD_SET_GRP_TSC,
    APCMD_SET_GRP_TSC_REPLY,
    APCMD_SET_COUNTER_MEASURE_ENABLED,
    APCMD_SET_COUNTER_MEASURE_ENABLED_REPLY,
    /* End of Marvell private MIB */
#endif
    APCMD_SET_BURST_MODE,
    APCMD_SET_BURST_MODE_REPLY,
    APCMD_GET_WLAN_FILTER_MODE,
    APCMD_GET_WLAN_FILTER_MODE_REPLY,
    APCMD_SET_WLAN_FILTER_MODE,
    APCMD_SET_WLAN_FILTER_MODE_REPLY,
    APCMD_UPDATE_FIRMWARE,
    APCMD_UPDATE_FIRMWARE_REPLY,
    APCMD_SAVE_USER_CONFIG,
    APCMD_SAVE_USER_CONFIG_REPLY,

    APCMD_GET_FIRMWARE_VERSION,
    APCMD_GET_FIRMWARE_VERSION_REPLY,
/* GET/SET Baseband Reg */
    APCMD_GET_BB_REG,
    APCMD_GET_BB_REG_REPLY,
    APCMD_SET_BB_REG,
    APCMD_SET_BB_REG_REPLY,
/* GET/SET Baseband Reg end */
#ifdef WPA
    /* For use by Marvell GUI only */
    APCMD_GUI_SET_PRIVACY_TBL_RSN_ENABLED,
    APCMD_GUI_SET_PRIVACY_TBL_RSN_ENABLED_REPLY,
    APCMD_SET_RSN_PMK,
    APCMD_SET_RSN_PMK_REPLY,
#endif
    APCMD_GET_BURST_MODE,
    APCMD_GET_BURST_MODE_REPLY,

    APCMD_GET_FORCE_PROTECTION_MODE,
    APCMD_GET_FORCE_PROTECTION_MODE_REPLY,
    APCMD_SET_FORCE_PROTECTION_MODE,
    APCMD_SET_FORCE_PROTECTION_MODE_REPLY,
    /* Alway add items above this line */
    MAX_APCMD
} APCMD;
#endif

typedef enum
{
    APCTL_OK = 1,
    APCTL_BUF_TOO_SMALL,
    APCTL_BAD_CMD,
    APCTL_CMD_FAILED,
    APCTL_TIMEOUT,
    APCTL_ERROR,
}APCMD_STATUS;

typedef struct _APICMDBUF
{
    uint16 cmdCode;
    uint16 seqNum;
    uint16 cmdStatus;
    uint16 cmdBodyLen;
    uint8 *cmdBody;
}
APICMDBUF;

#define apio_handle_t uint32
#define APIO_NOERROR 1
#define APIO_ERROR  0
#define apio_ErrCode_t uint8

typedef struct apio_bufdescr_t
{
    uint16 frameLen;  //Length of the frame
    uint16 reserved;  //reserved field
    uint8 *framePtr;  //Pointer to the frame
}
apio_bufdescr_t;

extern Status_e apctl(APICMDBUF * apicmdbuf);
extern apio_ErrCode_t apio_open(char *pName, apio_handle_t *pHandle, void (*) (apio_bufdescr_t *));
extern apio_ErrCode_t apio_close(apio_handle_t);
extern apio_ErrCode_t apio_read(apio_handle_t handle, apio_bufdescr_t *pBufDescr);
extern apio_ErrCode_t apio_write(apio_handle_t handle, apio_bufdescr_t *pBufDescr);
extern apio_ErrCode_t apio_alloc(apio_handle_t handle, apio_bufdescr_t *pBufDescr);
extern apio_ErrCode_t apio_free(apio_handle_t, apio_bufdescr_t *pBufDescr);
extern uint32 apio_select(void);

typedef enum
{
    OPEN_SYSTEM_ONLY = 1,
    SHARED_KEY_ONLY,
    OPEN_SYSTEM_AND_SHARED_KEY,
    NO_WEP,
}AUTH_MODE;

typedef enum
{
    APIO_POLL_RADIO_DATA_RDY = 1,
    APIO_POLL_ETH_DATA_RDY = 2,    /* Ethernet port has received data */
    APIO_POLL_WAN_DATA_RDY = (1 << 7),


}POLL_DATA;

#ifdef CONDOR
typedef enum
{
    B_CLIENT,
    G_CLIENT,
}CLIENT_MODE;

typedef enum
{
    AP_MODE_B_ONLY,
    AP_MODE_G_ONLY,
    AP_MODE_MIXED,
}AP_RATE_MODE;
#endif


typedef struct _APICMDBUF_GET_RF_CHANNEL
{

    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned short rfModulationType;
    char	availChanList[14];
    unsigned short curChanNum;

}
APICMDBUF_GET_RF_CHANNEL;

typedef struct _APICMDBUF_GET_OPEN_SSID_EN_STATE
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char openSSIDEnabled;
}
APICMDBUF_GET_OPEN_SSID_EN_STATE;

typedef struct _APICMDBUF_SET_OPEN_SSID_EN_STATE
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char openSSIDEnabled;
}
APICMDBUF_SET_OPEN_SSID_EN_STATE;


typedef struct _APICMDBUF_SET_RF_CHANNEL
{

    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char curChanNum;

}
APICMDBUF_SET_RF_CHANNEL;

typedef struct _APICMDBUF_GET_RF_MACADDR
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char physAddr[6];
}
APICMDBUF_GET_RF_MACADDR;

typedef struct _APICMDBUF_SET_RF_MACADDR
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char physAddr[6];
}
APICMDBUF_SET_RF_MACADDR;

typedef struct _APICMDBUF_GET_BASIC_DATA_RATES
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char basicDataRates[8];
}
APICMDBUF_GET_BASIC_DATA_RATES;

typedef struct _APICMDBUF_SET_BASIC_DATA_RATES
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char basicDataRates[8];
}
APICMDBUF_SET_BASIC_DATA_RATES;

typedef struct _APICMDBUF_GET_SUP_DATA_RATE
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char txDataRate[8];
}
APICMDBUF_GET_SUP_DATA_RATE;

typedef struct _APICMDBUF_SET_SUP_DATA_RATE
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char txDataRate[8];
}
APICMDBUF_SET_SUP_DATA_RATE;

typedef struct _APICMDBUF_GET_TX_DATA_RATE
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char txDataRate;
}
APICMDBUF_GET_TX_DATA_RATE;

typedef struct _APICMDBUF_SET_TX_DATA_RATE
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char txDataRate;
}
APICMDBUF_SET_TX_DATA_RATE;

typedef struct _APICMDBUF_GET_TX_POWER_LEVEL
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char txPowerLevel;
    unsigned char powerLevelList[8];
}
APICMDBUF_GET_TX_POWER_LEVEL;

typedef struct _APICMDBUF_SET_TX_POWER_LEVEL
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char txPowerLevel;
}
APICMDBUF_SET_TX_POWER_LEVEL;

typedef struct _APICMBUF_GET_RF_PREAMBLE_OPTION
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char rfPreambleOption;
}
APICMBUF_GET_RF_PREAMBLE_OPTION;

typedef struct _APICMBUF_SET_RF_PREAMBLE_OPTION
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char rfPreambleOption;
}
APICMBUF_SET_RF_PREAMBLE_OPTION;

typedef enum
{
    PREAMBLE_SHORT,
    PREAMBLE_LONG,
    PREAMBLE_AUTO_SELECT,
} PREAMBLE_TYPE;

typedef struct _APICMDBUF_GET_ANTENNA_MODE
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char rxAntennaMode;
    unsigned char txAntennaMode;
}
APICMDBUF_GET_ANTENNA_MODE;

typedef struct _APICMDBUF_SET_ANTENNA_MODE
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char whichAnt;
    unsigned char antennaMode;
}
APICMDBUF_SET_ANTENNA_MODE;

typedef enum
{
    AP_ANT_A,
    AP_ANT_B,
    AP_ANT_DIVERSITY,
} ANTENNA_TYPE;

typedef enum
{
    AP_RX_ANTENNA,
    AP_TX_ANTENNA,
} ANTENNA_MODE_TYPE;


typedef struct _APICMDBUF_GET_FRAG_THRESHOLD
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned short fragThreshold;
}
APICMDBUF_GET_FRAG_THRESHOLD;

typedef struct _APICMDBUF_SET_FRAG_THRESHOLD
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned short fragThreshold;
}
APICMDBUF_SET_FRAG_THRESHOLD;

typedef struct _APICMDBUF_GET_RTS_THRESHOLD
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned short rtsThreshold;
}
APICMDBUF_GET_RTS_THRESHOLD;

typedef struct _APICMDBUF_SET_RTS_THRESHOLD
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned short rtsThreshold;
}
APICMDBUF_SET_RTS_THRESHOLD;


typedef struct _APICMDBUF_GET_RTS_RETRY_LIMIT
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned short rtsRetryLimit;
}
APICMDBUF_GET_RTS_RETRY_LIMIT;

typedef struct _APICMDBUF_SET_RTS_RETRY_LIMIT
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned short rtsRetryLimit;
}
APICMDBUF_SET_RTS_RETRY_LIMIT;

typedef struct _APICMDBUF_GET_DATA_RETRY_LIMIT
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned short dataRetryLimit;
}
APICMDBUF_GET_DATA_RETRY_LIMIT;

typedef struct _APICMDBUF_SET_DATA_RETRY_LIMIT
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned short dataRetryLimit;
}
APICMDBUF_SET_DATA_RETRY_LIMIT;

typedef struct _APICMDBUF_GET_BEACON_PERIOD
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned long beaconPeriod;
}
APICMDBUF_GET_BEACON_PERIOD;

typedef struct _APICMDBUF_SET_BEACON_PERIOD
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned long beaconPeriod;
}
APICMDBUF_SET_BEACON_PERIOD;

typedef struct _APICMDBUF_GET_DTIM_PERIOD
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned long dtimPeriod;
}
APICMDBUF_GET_DTIM_PERIOD;


typedef struct _APICMDBUF_SET_DTIM_PERIOD
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned long dtimPeriod;
}
APICMDBUF_SET_DTIM_PERIOD;

typedef struct _APICMDBUF_GET_802_11_STATS
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned long totaltxBytes;
    unsigned long cntMcastPackets;
    unsigned long cntUcastPackets;
    unsigned long cntTxBad;
    unsigned long cntRetry;
    unsigned long cntMultRetry;
    unsigned long rtsOKCnt;
    unsigned long rtsBadCnt;
    unsigned long ackBadCnt;
    unsigned long totalRxBytes;
    unsigned long rxMcastOkCnt;
    unsigned long rxUcastOkCnt;
    unsigned long fcsErrorCnt;
    unsigned long duplicateCnt;
    unsigned long wepDecryptBadCnt;
}
APICMDBUF_GET_802_11_STATS;

typedef struct _APICMDBUF_GET_ETH_MAC_ADDRESS
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char phyAddress[6];
}
APICMDBUF_GET_ETH_MAC_ADDRESS;

typedef struct _APICMDBUF_SET_ETH_MAC_ADDRESS
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char phyAddress[6];
}
APICMDBUF_SET_ETH_MAC_ADDRESS;

typedef struct _APICMDBUF_GET_WAN_MAC_ADDRESS
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char phyAddress[6];
}
APICMDBUF_GET_WAN_MAC_ADDRESS;

typedef struct _APICMDBUF_SET_WAN_MAC_ADDRESS
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char phyAddress[6];
}
APICMDBUF_SET_WAN_MAC_ADDRESS;

typedef struct _APICMDBUF_GET_ETH_IP_CONFIG
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned long dhcpEnabled;
    unsigned long ipAddress;
    unsigned long subnetMask;
    unsigned long gatewayAddress;
}
APICMDBUF_GET_ETH_IP_CONFIG;

typedef struct _APICMDBUF_SET_ETH_IP_CONFIG
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned long dhcpEnabled;
    unsigned long ipAddress;
    unsigned long subnetMask;
    unsigned long gatewayAddress;
}
APICMDBUF_SET_ETH_IP_CONFIG;

typedef struct _APICMDBUF_GET_ETH_PORT_STATUS
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char portnumber;
    unsigned char ethOperStatus;
    unsigned char ethDuplex;
    unsigned char ethLinkSpeed;
}
APICMDBUF_GET_ETH_PORT_STATUS;

typedef enum
{
    ETH_OPER_STATUS_UP,
    ETH_OPER_STATUS_DOWN,
}ETHER_PORTSTATUS;


typedef struct _APICMDBUF_SET_ETH_PORT_PARAMS
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned short linkSpeed;
    unsigned short ethDuplex;
}
APICMDBUF_SET_ETH_PORT_PARAMS;

typedef struct _APICMDBUF_GET_ICMP_EN_STATE
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char icmpEnabled;
}
APICMDBUF_GET_ICMP_EN_STATE;

typedef struct _APICMDBUF_SET_ICMP_EN_STATE
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char icmpEnabled;
}
APICMDBUF_SET_ICMP_EN_STATE;


typedef struct _APICMDBUF_GET_802_3_STATS
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned long clearStats;
    unsigned long totalTxBytes;
    unsigned long cntMcastPackets;
    unsigned long cntUcastPackets;
    unsigned long cntTxBad;
    unsigned long totalRxBytes;
    unsigned long rxBadCnt;
    unsigned long rxMcastOKCnt;
    unsigned long rxUcastOKCnt;
    unsigned long crcErrorCnt;
}
APICMDBUF_GET_802_3_STATS;

typedef struct _APICMDBUF_GET_STA_INFO
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char staMACAddress[6];
    unsigned char staIPAddress;
    unsigned char staPowerSaveMode;
    unsigned char staWEPStatus;
    unsigned int staRSSI;
    unsigned int staSQ;
}
APICMDBUF_GET_STA_INFO;

typedef struct _APICMDBUF_DEAUTH_STA
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char staMACAddress[6];
}
APICMDBUF_DEAUTH_STA;

typedef struct _APICMDBUF_DISASSOC_STA
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char staMACAddress[6];
}
APICMDBUF_DISASSOC_STA;

// SECURITY COMMANDS
typedef struct _APICMDBUF_GET_DEFAULT_WEP_KEYS
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned short keyindex; //cntWEPKEYS;
    unsigned short keyType0;
    unsigned char keyValue0[16];
    unsigned short keyType1;
    unsigned char keyValue1[16];
    unsigned short keyType2;
    unsigned char keyValue2[16];
    unsigned short keyType3;
    unsigned char keyValue3[16];
}
APICMDBUF_GET_DEFAULT_WEP_KEYS;

typedef struct _APICMDBUF_SET_DEFAULT_WEP_KEYS
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned short keyindex;
    unsigned short keyType0;
    unsigned char keyValue0[16];
    unsigned short keyType1;
    unsigned char keyValue1[16];
    unsigned short keyType2;
    unsigned char keyValue2[16];
    unsigned short keyType3;
    unsigned char keyValue3[16];
}
APICMDBUF_SET_DEFAULT_WEP_KEYS;

typedef struct _APICMDBUF_SET_STA_WEP_KEY
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char staMACAddress[6];
    unsigned short keyType;
    unsigned short KeyValue[16];
}
APICMDBUF_SET_STA_WEP_KEY;

typedef enum
{
    WEP_NOT_SET,
    WEP_40_BIT,
    WEP_104_BIT,
} WEP_KEY_TYPE;

typedef struct _APICMDBUF_GET_PRIVACY_OPTION
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char dataEncrypt;
}
APICMDBUF_GET_PRIVACY_OPTION;

typedef struct _APICMDBUF_SET_PRIVACY_OPTION
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char dataEncrypt;
}
APICMDBUF_SET_PRIVACY_OPTION;

typedef enum
{
    DATA_ENCRYPT_NOT_REQUIRED,
    DATA_ENCRYPT_OPTIONA,
    FULL_DATA_ENCRYPT_REQUIRED,
} PRIVACY_TYPE;

typedef struct _APICMDBUF_GET_AUTH_MODE
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char authMode;
}
APICMDBUF_GET_AUTH_MODE;

typedef struct _APICMDBUF_SET_AUTH_MODE
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char authMode;
}
APICMDBUF_SET_AUTH_MODE;

//AP SYSTEM COMMANDS
typedef struct _APICMDBUF_GET_API_VER
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char apAPIVer[32];
}
APICMDBUF_GET_API_VER;

typedef struct _APICMDBUF_GET_AP_STATUS
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned short macStatus;
    unsigned short rfPortStatus;
    unsigned short ethPortStatus;
    unsigned short curChannel;
    unsigned char apname[32];
    unsigned char macAddress[6];
    unsigned short cntAssocSta;
    unsigned long maxLinkSpeed;
    unsigned char apFwVer[32];
    unsigned long apFwChecksum;
}
APICMDBUF_GET_AP_STATUS;

typedef enum
{
    OPER_STATUS_UP,
    OPER_STATUS_DOWN,
} AP_STATUS_TYPE;

typedef struct _APICMDBUF_GET_EVENT_LOG
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char eventlog;
}
APICMDBUF_GET_EVENT_LOG;

typedef struct _APICMDBUF_PURGE_EVENT_LOG
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
}
APICMDBUF_PURGE_EVENT_LOG;

typedef struct _APICMDBUF_UPDATE_FIRMWARE
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char *imgPtr;
    unsigned char apFwVer[32];
    unsigned long apChecksum;
}
APICMDBUF_UPDATE_FIRMWARE;

typedef enum
{
    FW_UPDATE_OK,
    FW_UPDATE_FAIL_FLASH,
    FW_UPDATE_FAIL_CHECKUSM,
    FW_UPDATE_SAME_VER,
} UPDATE_FIRMWARE_TYPE;

typedef struct _APICMDBUF_RUN_DIAG
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned short testNum;
    unsigned short testResult;
}
APICMDBUF_RUN_DIAG;

typedef enum
{
    TEST_PASS,
    TEST_FAIL,
    TEST_INCONCLUSIVE,
} DIAG_TYPE;

typedef struct _APICMDBUF_START_LINK_TEST
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char staMACAdress[6];
    unsigned long cntTxPacketsOK;
    unsigned long totalErrors;
    unsigned long totalWEPErrors;
}
APICMDBUF_START_LINK_TEST;

typedef struct _APICMDBUF_READ_GPIO
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char gpioNum;
    unsigned char gpioVal;
}
APICMDBUF_READ_GPIO;

typedef struct _APICMDBUF_WRITE_GPIO
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char gpioNum;
    unsigned char gpioVal;
}
APICMDBUF_WRITE_GPIO;


typedef struct _APICMDBUF_SET_STA_AGING_TIME
{

    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned short timeInMinutes;

}
APICMDBUF_SET_STA_AGING_TIME;


typedef struct _APICMDBUF_SET_WLAN_RADIO
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char radio;
}
APICMDBUF_SET_WLAN_RADIO;


typedef enum{
    APEVT_STA_AUTHENTICATED,
    APEVT_STA_ASSOCIATED,
    APEVT_STA_DEAUTHENTICATED,
    APEVT_STA_DISASSOCIATED,
    APEVT_STA_AUTHENTICATE_FAIL,
}STA_ASSOC_TYPE;

typedef enum{
    STA_8021X_NOT_AUTHENTICATED,
    STA_8021X_AUTHENTICATED,
} STA_8021X_STATUS;

/** use for mac access list **/
typedef enum{
    ACCESS_MODE,
    FILTER_MODE,
}FILTER_TYPE;

typedef struct _APICMDBUF_GET_BASIC_DATA_RATES_G
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char basicDataRates[16];
}
APICMDBUF_GET_BASIC_DATA_RATES_G;

typedef struct _APICMDBUF_SET_BASIC_DATA_RATES_G
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char basicDataRates[16];
}
APICMDBUF_SET_BASIC_DATA_RATES_G;

typedef struct _APICMDBUF_GET_TX_DATA_RATE_G
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char txDataRate;
}
APICMDBUF_GET_TX_DATA_RATE_G;

typedef struct _APICMDBUF_SET_TX_DATA_RATE_G
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char txDataRate;
}
APICMDBUF_SET_TX_DATA_RATE_G;

typedef struct _APICMDBUF_GET_AP_MODE
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned int apMode;
}
APICMDBUF_GET_AP_MODE;

typedef struct _APICMDBUF_SET_AP_MODE
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned int apMode;
}
APICMDBUF_SET_AP_MODE;


#define SDRAM_BURST_REG (W81_SDRAM_CFG_BASE+0x14)
#define SDRAM_BURST_1   0
#define SDRAM_BURST_2   1
#define SDRAM_BURST_4   2
#define SDRAM_BURST_8   3
typedef struct _APICMDBUF_SDRAM_BURST_MODE
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    uint8 sdramBurstMode;

}
APICMDBUF_SDRAM_BURST_MODE;

typedef struct _APICMDBUF_SET_BURST_MODE
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    uint8 BurstMode;
    uint8 BurstRate;
}
APICMDBUF_SET_BURST_MODE;

typedef struct _APICMDBUF_GET_BURST_MODE
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    uint8 BurstMode;
    uint8 BurstRate;
}
APICMDBUF_GET_BURST_MODE;
#ifdef WPA

/**************** 802.11i MIB supported *********************/
typedef struct _APICMDBUF_GET_STA_CFG_RSN_OPTION_IMP
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char RSN_Capable;
}
APICMDBUF_GET_STA_CFG_RSN_OPTION_IMP;

typedef struct _APICMDBUF_GET_STA_CFG_TKIP_NUM_RPLY_COUNTERS
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned long NumOfCounters;
}
APICMDBUF_GET_STA_CFG_TKIP_NUM_RPLY_COUNTERS;

typedef struct _APICMDBUF_GET_PRIVACY_TBL_RSN_ENABLED
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned long Enabled;
}
APICMDBUF_GET_PRIVACY_TBL_RSN_ENABLED;

typedef struct _APICMDBUF_SET_PRIVACY_TBL_RSN_ENABLED
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned long Enabled;
}
APICMDBUF_SET_PRIVACY_TBL_RSN_ENABLED;

typedef struct _APICMDBUF_GET_RSN_CFG_TBL
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned long Version;
    unsigned long PairwiseKeysSupported;
    unsigned char MulticastCipher[4];
    unsigned char GroupRekeyMethod;
    unsigned long GroupRekeyTime;
    unsigned long GroupRekeyPackets;
    unsigned char GroupRekeyStrict;
    //unsigned char  PSKValue[32];  // write only
    //unsigned char  PSKPassPhrase[64]; // write only
    unsigned char TSNEnabled;
    unsigned long GroupMasterRekeyTime;
    unsigned long GroupUpdateTimeOut;
    unsigned long GroupUpdateCount;
    unsigned long PairwiseUpdateTimeOut;
    unsigned long PairwiseUpdateCount;
}
APICMDBUF_GET_RSN_CFG_TBL;

typedef struct _APICMDBUF_SET_RSN_CFG_TBL
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    //unsigned long  Version;   // read only
    unsigned long PairwiseKeysSupported;
    unsigned char MulticastCipher[4];
    unsigned char GroupRekeyMethod;
    unsigned long GroupRekeyTime;
    unsigned long GroupRekeyPackets;
    unsigned char GroupRekeyStrict;
    unsigned char PSKValue[32]; //write only
    unsigned char PSKPassPhrase[64]; // write only
    unsigned char TSNEnabled;
    unsigned long GroupMasterRekeyTime;
    unsigned long GroupUpdateTimeOut;
    unsigned long GroupUpdateCount;
    unsigned long PairwiseUpdateTimeOut;
    unsigned long PairwiseUpdateCount;
}
APICMDBUF_SET_RSN_CFG_TBL;

typedef struct _APICMDBUF_GET_RSN_CFG_UNICAST_CIPHER
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char UnicastCipher[4];
    unsigned char Enabled;
}
APICMDBUF_GET_RSN_CFG_UNICAST_CIPHER;

typedef struct _APICMDBUF_SET_RSN_CFG_UNICAST_CIPHER
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char UnicastCipher[4];
    unsigned char Enabled;
}
APICMDBUF_SET_RSN_CFG_UNICAST_CIPHER;

typedef struct _APICMDBUF_GET_RSN_CFG_AUTH_SUITE
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char AuthSuites[4];
    unsigned char Enabled;
}
APICMDBUF_GET_RSN_CFG_AUTH_SUITE;

typedef struct _APICMDBUF_SET_RSN_CFG_AUTH_SUITE
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char AuthSuites[4];
    unsigned char Enabled;
}
APICMDBUF_SET_RSN_CFG_AUTH_SUITE;

typedef struct _APICMDBUF_GET_RSN_STATS
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char MacAddr[6];
    unsigned char Version;
    unsigned char SelectedUnicastCipher[4];
    unsigned long TKIPICVErrors;
    unsigned long TKIPLocalMICFailures;
    unsigned long TKIPRemoteMICFailures;
    unsigned long TKIPCounterMeasuresInvoked;
    unsigned long WRAPFormatErrors;
    unsigned long WRAPReplays;
    unsigned long WRAPDecryptErrors;
    unsigned long CCMPFormatErrors;
    unsigned long CCMPReplays;
    unsigned long CCMPDecryptErrors;
}
APICMDBUF_GET_RSN_STATS;
/* end of 802.11i MIB supported */

/************ Marvell private MIB **************************/

// Enable data traffic for a station. This will be enabled if authentication and key installation
// for the station is done.
typedef struct _APICMDBUF_SET_RSN_DATA_TRAFFIC_ENABLED
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char StaMacAddr[6];
    unsigned char Enabled;
}
APICMDBUF_SET_RSN_DATA_TRAFFIC_ENABLED;

// Set Pairwise Key (PWK) for a station
typedef struct _APICMDBUF_SET_RSN_PWK
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char StaMacAddr[6];
    unsigned char EncryptKey[16];
    unsigned int TxMICKey[2];
    unsigned int RxMICKey[2];
    //unsigned long  TempKey1[4];
    //unsigned long  TempKey2[4];
}
APICMDBUF_SET_RSN_PWK;

// Get Pairwise Key (PWK) for a station
typedef struct _APICMDBUF_GET_RSN_PWK
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char StaMacAddr[6];
    unsigned char EncryptKey[16];
    unsigned int TxMICKey[2];
    unsigned int RxMICKey[2];
    //unsigned long  TempKey1[4];
    //unsigned long  TempKey2[4];
}
APICMDBUF_GET_RSN_PWK;

// Set group key
typedef struct _APICMDBUF_SET_RSN_GRP_KEY
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char EncryptKey[16];
    unsigned int TxMICKey[2];
    unsigned int RxMICKey[2];
    //unsigned long  TempKey1[4];
    //unsigned long  TempKey2[4];
}
APICMDBUF_SET_RSN_GRP_KEY;

// Get group key
typedef struct _APICMDBUF_GET_RSN_GRP_KEY
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char EncryptKey[16];
    unsigned int TxMICKey[2];
    unsigned int RxMICKey[2];
    //unsigned long  TempKey1[4];
    //unsigned long  TempKey2[4];
}
APICMDBUF_GET_RSN_GRP_KEY;
/* End of Marvell private MIB */

// Get a station's association state. The states include APEVT_STA_ASSOCIATED and
// APEVT_STA_DISASSOCIATED defined in enum STA_ASSOC_TYPE. Whenever a station gets
// associated or deassociated, MacMgmt task will send the state to a message queue.
// This function can be called to check the state in the message queue.
typedef struct _APICMDBUF_GET_STA_ASSOC_STATE
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char staMACAddr[6];
    unsigned short assocType;
}
APICMDBUF_GET_STA_ASSOC_STATE;

// Get this station's RSN IE
typedef struct _APICMDBUF_GET_THIS_STA_RSN_IE
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    //unsigned char  staMACAddr[6];
    unsigned char RsnIEBuf[MAX_SIZE_RSN_IE_BUF];
}
APICMDBUF_GET_THIS_STA_RSN_IE;

// Get an external station's RSN IE
typedef struct _APICMDBUF_GET_EXT_STA_RSN_IE
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char staMACAddr[6];
    unsigned char RsnIEBuf[MAX_SIZE_RSN_IE_BUF];
}
APICMDBUF_GET_EXT_STA_RSN_IE;

// Get pairwise TSC
typedef struct _APICMDBUF_GET_PAIRWISE_TSC
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char staMACAddr[6];
    unsigned long TxIV32;
    unsigned short TxIV16;
}
APICMDBUF_GET_PAIRWISE_TSC;

// Set pairwise TSC
typedef struct _APICMDBUF_SET_PAIRWISE_TSC
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char staMACAddr[6];
    unsigned long TxIV32;
    unsigned short TxIV16;
}
APICMDBUF_SET_PAIRWISE_TSC;

// Get Group TSC
typedef struct _APICMDBUF_GET_GRP_TSC
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned long g_IV32;
    unsigned short g_IV16;
}
APICMDBUF_GET_GRP_TSC;

// Set Group TSC
typedef struct _APICMDBUF_SET_GRP_TSC
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned long g_IV32;
    unsigned short g_IV16;
}
APICMDBUF_SET_GRP_TSC;

typedef struct _APICMDBUF_SET_COUNTER_MEASURE_ENABLED
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char Enabled;
}
APICMDBUF_SET_COUNTER_MEASURE_ENABLED;

// Set Pairwise Master Key (PMK) for a station
typedef struct _APICMDBUF_SET_RSN_PMK
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char StaMacAddr[6];
    unsigned char PMK[32];
}
APICMDBUF_SET_RSN_PMK;

uint8 disableStaAssociate;

typedef enum {
    COUNTER_MEASURE_START,
    COUNTER_MEASURE_STOP,
} COUNTER_MEASURE_EVENT;
 /* GET/SET Baseband Reg */
typedef struct _APICMDBUF_GET_BB_REG{

unsigned short cmdCode;
unsigned short seqNum;
unsigned short cmdStatus;
unsigned short cmdBodyLen;
unsigned short regNum;
unsigned char regVal;

}
APICMDBUF_GET_BB_REG;

typedef struct _APICMDBUF_SET_BB_REG{

unsigned short cmdCode;
unsigned short seqNum;
unsigned short cmdStatus;
unsigned short cmdBodyLen;
unsigned short regNum;
unsigned char regVal;
 
}
APICMDBUF_SET_BB_REG;

typedef struct _APICMDBUF_GET_FORCE_PROTECTION_MODE
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned short forceProtectMode;
}
APICMDBUF_GET_FORCE_PROTECTION_MODE;

typedef struct _APICMDBUF_SET_FORCE_PROTECTION_MODE
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned short forceProtectMode;
}
APICMDBUF_SET_FORCE_PROTECTION_MODE;

 /* GET/SET Baseband Reg end*/

void (*MICCounterMeasureInvokedCbFunc)(COUNTER_MEASURE_EVENT event);
void DeauthenticateAllSta();

#endif

/* Timer API */
/**
 * Create a periodic timer. The callback function will run in the
 * Delayed Service Routine of the real-time clock interrupt.
 * 
 * @param period  The period in milliseconds
 * @param timerCb The callback function
 * @return 0: failed to create timer because exceed maximum
 *         number of timers supported
 *         >0: the timer index, to be used in
 *         subsequent call to apctlTimerDelete()
 */
unsigned long apctlTimerCreate(int period, void (*timerCb)(void));

/**
 * Delete a timer
 * 
 * @param timerIndex The timer index returned by apctlTimerCreate()
 * @return TRUE: success
 *         FALSE: failure because timer index is invalid
 */
int apctlTimerDelete(unsigned long timerIndex);

/* Interrupt control API */

/**
 * Get interrupt enable status
 * 
 * @return Interrupt status
 */
int apctlGetIntStatus(void);

/**
 * Enable all interrupts
 * 
 * @param status Status word returned by apctlIntDisable()
 * @return Interrupt status
 */
int apctlIntEnable(int status);
/**
 * Disable all interrupts
 * 
 * @return Interrupt status
 */
int apctlIntDisable(void);

/* GPIO control API */

/**
 * Initialize the GPIO pin for output or input
 * 
 * @param dirMap The GPIO direction bit map.
 *               Bit set to 1: the corresponding GPIO pin is configured
 *               for input.
 *               Bit set to 0: the corresponding GPIO pin is configured
 *               for output
 */
void apctlGPIOInit(unsigned long dirMap);

/**
 * Read the state of a GPIO pin
 * 
 * @param gpioNum the GPIO pin number
 * @return 0: GPIO state is "low"
 *         1: GPIO state is "high"
 *        -1: Error: invalid GPIO number
 */
int apctlGPIORead(int gpioNum);

/**
 * Set the state of a GPIO pin. The new GPIO state
 * after write is returned.
 * 
 * @param gpioNum The GPIO pin number
 * @param state   The state to set:
 *                0: set to "low"
 *                1: set to "high"
 * @return 0: the GPIO state is "low"
 *         1: the GPIO state is "high"
 *        -1: Error: invalid GPIO number
 *        -2: Error: GPIO pin is in input mode
 */
int apctlGPIOWrite(int gpioNum, int state);


Boolean apctlFlashEraseSector(int sectorNum);
Boolean apctlFlashWriteWord(int offset, uint16 value);

void apctlMsTimerCreate(int period, void (*timerCbFunc)(void));

#ifdef WAN_PHY_INIT

uint8 Wan_PhyConfig;

typedef enum
{
    WAN_AUTO_NEG,
    WAN_10_HALF_DUPLEX,
    WAN_10_FULL_DUPLEX,
    WAN_100_HALF_DUPLEX,
    WAN_100_FULL_DUPLEX,
}WAN_PHY_CONFIG;

#endif

#ifdef WPA
typedef struct _StaAssocStateMsg
{
    unsigned char staMACAddr[6];
    unsigned short assocType;
}
StaAssocStateMsg_t;

#if 0
typedef enum
{
    ERROR,
    KEYMGMTTIMEOUTEVENT, //key management timeout
    MICERRORTIMEOUTEVENT, //MIC error timeout
    MICERRORRSTTIMEOUTEVENT, //MIC error timeout within 60 secs
    GRPKEYTIMEOUTEVENT
}dist_timout_msg_type_e;

//Rahul
typedef struct
{
    //timer_Data_t timerData;
    //uint32   Id;
    IEEEtypes_MacAddr_t StnAddr;
    dist_timout_msg_type_e type;
}
dist_PendingData_t;


/*----------------*/
/* Timer Messages */
/*----------------*/

typedef struct
{
    dist_PendingData_t *PendingData_p;
    uint8 Id;
}
distQ_TimerMsg_t;

typedef enum
{
    ASSOMSGRECVD,
    TIMERMSGRECVD
}MsgType_e;

typedef struct
{
    MsgType_e MsgType;
    union
    {
        StaAssocStateMsg_t StaAssocStateMsg;
        distQ_TimerMsg_t distQ_TimerMsg;
    }msg;
}
DistTaskMsg_t;
#endif
#endif

#ifdef RESET_BUTTON_EVENT
typedef struct _ApEvtMsg
{
    uint16 evtCode;
    uint16 evtBodyLen;
    uint32 evtTimeStamp;
    uint8 *evtBody;
}
ApEvtMsg_t;

//EVENTS MESSAGES
typedef struct _APEVT_STA_ASSOC_TYPE
{
    unsigned short evtCode;
    unsigned short evtBodyLen;
    unsigned long evtTimeStamp;
    unsigned char staMACAddress[6];
    unsigned short assocType;
}
APEVT_STA_ASSOC_TYPE;

typedef enum
{
    APEVT_RST_BTN_PRESSED = 1,
    APEVT_ETH_PORT_STATUS_CHANGE = (1 << 1),
    APEVT_STA_ASSOC = (1 << 2),

}APEVT;

typedef enum
{
    GPIO_PIN_0 = (1 << 0),
    GPIO_PIN_1 = (1 << 1),
    GPIO_PIN_2 = (1 << 2),
    GPIO_PIN_3 = (1 << 3),
    GPIO_PIN_SW_RESET = (1 << 4),
    GPIO_PIN_5 = (1 << 5),
    GPIO_PIN_6 = (1 << 6),
    GPIO_PIN_7 = (1 << 7),
    GPIO_PIN_8 = (1 << 8),
    GPIO_PIN_9 = (1 << 9),
    GPIO_PIN_10 = (1 << 10),
    GPIO_PIN_11 = (1 << 11),
    GPIO_PIN_12 = (1 << 12),
    GPIO_PIN_13 = (1 << 13),
    GPIO_PIN_14 = (1 << 14),
    GPIO_PIN_15 = (1 << 15),
}GPIO;


// maximum events
#define API_EVENT_MAX_EVENT 3

// maximun subscribers
#define API_EVENT_MAX_SUB 2*API_EVENT_MAX_EVENT

/*
 ** apEvtSub
 *
 *  PARAMETERS: 
 *      eventSet - the set of events the caller want to subcribe. This parameter is a bit map
 *          each bit corresponds to an event type. Set the bit to 1 to subscribe for the event
 *      *evtCbFunc - Pointer to the callback function
 *      pSubID - Pointer to the variable that will receive the subscription ID. The caller must 
 *          save the subscription ID for later use to unscribe for the event
 *
 *  DESCRIPTION:
 *
 *  RETURNS: TRUE - the subscription is successful
 *           FALSE - the subscription fail
 *
 */
Boolean apEvtSub(unsigned long eventSet, void (*evtCbFunc)(ApEvtMsg_t *), unsigned int *pSubID);

/*
 ** apEvtUnsub
 *
 *  PARAMETERS:
 *      eventSet - the set of events the caller want to subcribe. This parameter is a bit map
 *          each bit corresponds to an event type. Set the bit to 1 to subscribe for the event
 *      subID - the subscription ID
 *
 *  DESCRIPTION:
 *
 *  RETURNS: TRUE - the unsubscription is successful
 *           FALSE - the unsubscription fail
 *
 */
Boolean apEvtUnsub(unsigned long eventSet, unsigned int subID);

#endif

void WriteRfReg(unsigned char offset, unsigned char value);
unsigned char ReadRfReg(unsigned char offset);
void Reset(void);
void interrogate();
void amdEraseChip(int sblk, int nblks);
void amdFlashWriteWord( int offset, unsigned short dat);
void Reset2();

#endif /* _SMEMAIN_H_ */




