#ifndef _PCMCIA_P_H
#define _PCMCIA_P_H

#define NORMAL_MODE 0
#define BRIDGE_MODE 1
#define REPEATER_MODE 2
#define CLIENT_MODE 3
#define POINT_MODE  4
#define MULTIPOINT_MODE 5

#define OPEN_SYSTEM 0
#define SHARE_KEY   1

typedef struct _PCMCIA_PARAMETERS {    
   char BSSID[32+1];       //default : "default"   
   char Channel;           //default : 1,  (range 1, 14)
   char Preamble;                    // 0:Long 1 : Short, 2 : Auto preamble, default : 2
   char DTIMPeriod;                  // 1~255, 0 is reseved, default : 1
   unsigned long BasicDataRate;     //default : 0,  (range 1, 15)
   unsigned long SupportedDataRate; //default : 0, (range 1, 15)
   unsigned long TxDataRate;        //default : 0, (range 1, 15)
   char Encryption;        //default : 0 (Off),   (range: 0, 1)
   char defaultKeyID;      //default : 0 (KEY 1), (range: 0, 3)
   char key_length;        //default : 5,  (5 [64 bit] or 13 [128 bit])
   char Keys[4][14];       //default : 0....0
   char Mac[6];            //default : 000000, use PC Card Mac Address
   char Authentication;    //default : 1(Open System), (range 1, 2(share key))
   unsigned short BeaconInterval;    //default : 100
   unsigned short RTS;               //default : 2432, range (0..3000)
   unsigned short Fragmentation;     //default : 2346, range (256..2346) Even number
   unsigned short Mode;              //default : 0 (AP + Auto)
                                     //          bit 0~1 : 0 (Auto),    1 (2.0 PC CARD), 2 (2.5 PC CARD)
                                     //          bit 2: 0(Infrastructure), 1(802.11 Ad-Hoc)
                                     //          bit 3: 1 (ad-hoc mode)
                                     //          bit 4~10: Reserve
                                     //          bit 11: FrameBurst 0:disable 1:enable
                                     //          bit 14 13 12
                                     //         *      0  0  0   -> Mixed mode Long (g & b mode)
                                     //         *      0  0  1   -> g only mode
                                     //         *      0  1  0   -> b only mode
                                     //               0  1  1   -> b only wifi mode
                                     //               1  0  0   -> Mixed mode  (g & b mode)
                                     //               1  0  1   -> Mixed mode Short  (g & b mode)
                                     //               others    -> reserved (g & b mode)
                                     //          bit 15  : 0 (AP mode), 1 (Station mode)
   char EnhanceSecurity;             //default : 0 (broadcast SSID in beacon frame)
                                     //   bit 0 : Hide SSID in beacon frame
                                     //   bit 1 : Block response to "Unspecified-SSID"
} PCMCIA_PARAMETERS;

#define TX_RATE_1M      0x00000001
#define TX_RATE_2M      0x00000002
#define TX_RATE_5_5M    0x00000004
#define TX_RATE_11M     0x00000008
//#define TX_RATE_22M     0x00000010
#define TX_RATE_6M      0x00000010
#define TX_RATE_9M      0x00000020
#define TX_RATE_12M     0x00000040
#define TX_RATE_18M     0x00000080
#define TX_RATE_24M     0x00000100
#define TX_RATE_36M     0x00000200
#define TX_RATE_48M     0x00000400
#define TX_RATE_54M     0x00000800
#define	TX_RATE_AUTO	0xFFFFFFFF

/*
	(Rate *2)+0x80

	Rate(Mbps)	Support_Data_Rate	Basic_Data_Rate
0	1		2	(0x02)		0x82
1	2		4	(0x04)		0x84
2	5.5		11	(0x0B)		0x8B
3	11		22	(0x16)		0x96
4	6		12	(0x0c)		0x8c
5	9		18	(0x12)		0x92
6	12		24	(0x18)		0x98
7	18		36	(0x24)		0xA4
8	24		48	(0x30)		0xB0
9	36		72	(0x48)		0xc8
10	48		96	(0x60)		0xE0
11	54		108	(0x6c)		0xEc

*/


#endif
