#ifndef ONE_POOL
#if !defined(BUFMGT_H)
#define BUFMGT_H
/*
*                Copyright 2003, Marvell International Ltd.
* This code contains confidential information of Marvell Semiconductor, Inc.
* No rights are granted herein under any patent, mask work right or copyright
* of Marvell or any third party.
* Marvell reserves the right at its sole discretion to request that this code
* be immediately returned to Marvell. This code is provided "as is".
* Marvell makes no warranties, express, implied or otherwise, regarding its
* accuracy, completeness or performance.
*/

/*!
 * \file    bufmgt.h
 * \brief   Basic commons in the BufMgt module must be defined in this file.
 *
*/

/*! Create common macros for BufMgt module */
#define PREWORD(x)   ( ((long)(x)&0x3)? (((long)(x)&0xfffffffc)-4) \
                     : (long)((char *)(x)-4) )
#define ROUND2(x)   ( ((long)(x)&0x1)? ((long)(x)&0xfffffffe)+2 : (long)(x) )
#define ROUND4(x)   ( ((long)(x)&0x3)? ((long)(x)&0xfffffffc)+4 : (long)(x) )
#define ROUND8(x)   ( ((long)(x)&0x7)? ((long)(x)&0xfffffff8)+8 : (long)(x) )
#define ROUND16(x)  ( ((long)(x)&0xf)? ((long)(x)&0xfffffff0)+16 : (long)(x) )
#define ROUND_N(x, n) ( ((long)(x)&(n-1))? ((long)(x)&~(n-1))+n : (long)(x) )
#define OFFSET2(x)  ( (long)(x)&0x1 )
#define OFFSET4(x)  ( (long)(x)&0x3 )
#define OFFSET8(x)  ( (long)(x)&0x7 )
#define OFFSET16(x) ( (long)(x)&0xf )
#define OFFSET_N(x, n) ( (n)? (long)(x)&(n-1) : 0 )
#define ISALIGN2(x)     !OFFSET2(x)
#define ISALIGN4(x)     !OFFSET4(x)
#define ISALIGN8(x)     !OFFSET8(x)
#define ISALIGN16(x)    !OFFSET16(x)
#define ISALIGN_N(x, n) !OFFSET_N(x, n)

/*! Create common values for BufMgt module */
#define QHANDLE_MAGIC   0xAABBCCDD
#define QUEUE_DATA_0    0
#define QUEUE_DATA_1    1
#define QUEUE_DATA_2    2
#define QUEUE_DATA_3    3
#define QUEUE_DATA_4    4
#define QUEUE_DATA_5    5
#define QUEUE_DATA_6    6
#define QUEUE_DATA_7    7
#define QUEUE_BCN       8       /* Beacons */
#define QUEUE_MLME      9       /* Mgmt queue */
#define QUEUE_CMD       10      /* Cmd queue */
#define QUEUE_MAX       (QUEUE_CMD+1)   /* Cmd queue is always the last */

#define QUEUE_ALL       QUEUE_MAX
#define QUEUE_ANY       QUEUE_MAX

#define QUEUE_PRI_HIGH  QUEUE_DATA_0
#define QUEUE_PRI_LOW   QUEUE_DATA_7

#define MUTEX_ENTER(cfg)    \
    (cfg.mutex_enter? (*cfg.mutex_enter)(cfg.mutex, MUTEX_TIMEOUT) : TRUE )
#define MUTEX_EXIT(cfg)     \
    (cfg.mutex_exit? (*cfg.mutex_exit)(cfg.mutex) : TRUE )


#define IO_CMD_OWNER    0x80000000

#define IO_ST_PENDING   0
#define IO_ST_DONE      1
#define IO_ST_ALL       2

typedef
struct io_buf_desc
{
    volatile UINT32 command;
    volatile UINT16 buf_size;       /* 0xfff8     ->  000@LSB */
    volatile UINT16 byte_count;
    char    *data;                  /* 0xfffffff8 ->  000@LSB */
    struct io_buf_desc *next;       /* 0xfffffff0 -> 0000@LSB */
    void    *info;                  /* io control info */
    UINT8   state;                  /* IO state */
}IO_BUF_DESC;

typedef
struct usr_buf_desc
{
    UINT32  command;                /* command and status */
    UINT16  buf_size;               /* buffer size for data */
    UINT16  byte_count;             /* data byte count */
    char    *data;                  /* pointer to data/packet */
    struct usr_buf_desc *next;      /* next descriptor */
    void    *usr_data;              /* pointer to user data */
    void    *info;                  /* per pkt io control info */
    struct qhandle *qhandle;        /* the last QHANDLE */
    struct qhandle *ohandle;        /* the old QHANDLE */
    IO_BUF_DESC *iobd;              /* the associated IO_BUF_DESC */
    UINT16  protocol;               /* protocol value of packet */
    UINT8   priority;               /* priority of the queue */
    UINT8   align_offset;           /* bytes off for data alignment */
    UINT8   flags;                  /* flags/mode of the buffer */
    UINT8   ref_cnt;                /* reference count */
}USR_BUF_DESC;

#ifndef size_t
#define size_t UINT32
#endif

typedef
struct pkt_q_cfg
{
    UINT8  type;                    /* ring buffer? */
    UINT8  bd_align;                /* byte alignment for buf desc */
    UINT8  buf_align;               /* byte alignment for buffer */
    UINT8  hdr_size;                /* packet header size in byte */
    UINT16 usr_data_size;           /* user specific data size in byte */
    UINT16 info_size;               /* pkt control info size */
    UINT16 buf_size;                /* buffer sizes */
    UINT16 length[QUEUE_MAX];       /* queue lengths */
    IO_BUF_DESC *head[QUEUE_MAX];   /* heads of buffer descriptors */
    void (*tx_done)(struct qhandle *, UINT8, UINT8);    /* tx_done */
    void (*data_ready)(struct qhandle *, UINT8, UINT8); /* rx_done */
    void (*evt_notify)(struct qhandle *, UINT32, void *);   /* event notify */
    int (*post_put)(struct qhandle *, USR_BUF_DESC *);  /* called after enqueue */
    int (*post_get)(struct qhandle *, USR_BUF_DESC *);  /* called after dequeue */
    char *(*nextRxPtr)(UINT8, void **); /* routine to get the next Rx pkt buffer */
    void *mem_sema;                 /* semaphore */
    void *mem_handle;               /* handle for buffer alloc & free */
    char *(*alloc)(void *, int);    /* routine to allocate buffers */
    void (*free)(void *, char *);   /* routine to free buffers */
    void *mutex;                    /* mutex token */
    BOOLEAN (*mutex_enter)(void *, int ms); /* mutex enter */
    BOOLEAN (*mutex_exit)(void *);          /* mutex exit */
    void *(*memcpy)(void *dst, void *src, size_t);  /* used to copy data if != 0 */
    UINT8  def_pri_put;         /* default priority for put QUEUE_ANY */
    UINT8  def_pri_get;         /* default priority for get QUEUE_ANY */
}PKT_Q_CFG;

/* Flags */
#define QUEUE_FULL      0x1
#define QUEUE_FREEMEM   0x2     /* free memory in msi_wl_PktsXmtd */

typedef
struct qhandle                  /* Queue handle */
{
    UINT32 magic;               /* Tag for QUEUE_CTRL: 0xAABBCCDD */
    struct pkt_q_cfg cfg;       /* Queue Config */
    UINT16  length[QUEUE_MAX];  /* current length of the queue */
    IO_BUF_DESC *rd[QUEUE_MAX], *wr[QUEUE_MAX];  /* read/write pointers */
    void *extra[QUEUE_MAX];     /* extra queue controls */
    struct dev_ctrl *dev_ctrl;  /* bound device control strcuture */
    UINT8 bd_offset[QUEUE_MAX]; /* bytes off for the pointer of buf desc */
    UINT8 flags[QUEUE_MAX];     /* flags */
    UINT16 buf_cnt;             /* Allocated buffer count */
    UINT8 usr_bd;               /* IO buffer descriptors were provided by user */
    BOOLEAN filled;             /* data buffers were filled in creation */
    UINT8 dma_op;               /* pending DMA op: put or get */
    UINT8 bd_size;              /* bd size + pad bytes for bd alignment */
    UINT8 buf_align_off;        /* additional offset after buf alignment */
}*QHANDLE;                      /* Queue Handle pointer */

typedef
struct dev_ctrl
{
    QHANDLE rxQ;                        /* Bound Rx PktQ */
    QHANDLE txQ;                        /* Bound Tx PktQ */
    int (*dev_init)(struct dev_ctrl *); /* called at the binding */
    int (*dev_intr)(BOOLEAN);           /* enable/disable interrupt */
    int (*dev_tx_start)(IO_BUF_DESC *, UINT8, UINT8);   /* start tx */
    int (*dev_ioctl)(UINT32, UINT32, void *);           /* dev ioctl */
    void (*dev_rx_evt_notify)(QHANDLE, UINT32, void *); /* Rx event notify */
    void (*dev_tx_evt_notify)(QHANDLE, UINT32, void *); /* Tx event notify */
    void (*dev_dma_done)(char *, char *, UINT32);       /* dma_done */
    BOOLEAN dev_busy[QUEUE_MAX];/* device busy flags */
}DEV_CTRL;


#define NUM_PKT_BUFS    256
#define MAX_PKT_SIZE    2346    /* +4 for FCS */
#define MAX_USER_DATA   128
#define MAX_CTRL_DATA   (sizeof(USR_BUF_DESC)+128)
#define MAX_BUF_SIZE    (MAX_PKT_SIZE+MAX_USER_DATA+MAX_CTRL_DATA)

#ifdef AP_USE_BUFFER_OFFSET
#define MAX_HDR_SIZE    128      /* max wireless header + RxInfo + 4 */
#define AP_BUFFER_OFFSET MAX_HDR_SIZE
#else
#define MAX_HDR_SIZE    64       /* max wireless header + RxInfo + 4 */
#endif

#define MAX_PKT_OVERHEAD    (MAX_CTRL_DATA+MAX_HDR_SIZE)
#define MAX_HDR_DATA_GAP    (sizeof(USR_BUF_DESC)+MAX_USER_DATA+MAX_HDR_SIZE)

#define DEFAULT_MALLOC  buf_alloc
#define DEFAULT_FREE    buf_free

#define QUEUE_LIST      0
#define QUEUE_RING      1

#define Q_DMA_DONE      0
#define Q_DMA_PUT       1
#define Q_DMA_GET       2

#define Q_OPT_NONE      0
#define Q_OPT_MEMCPY    1
#define Q_OPT_REFILL    2

#define MUTEX_TIMEOUT   10      /* max waiting time (ms) for Mutex */

#ifndef AP_SW
#define msi_wl_GetUsrBufPtr(data)    \
        ((data)? (USR_BUF_DESC *)(*(UINT32 *)PREWORD(data)) : NULL)
#endif
#if 0
#define msi_wl_FreePktBuf(data) msi_wl_FreeUsrBuf(msi_wl_GetUsrBufPtr(data))
#endif
#define msi_wl_GetHdrSize(data)    \
        ((USR_BUF_DESC *)msi_wl_GetUsrBufPtr(data))->qhandle->cfg.hdr_size
#define msi_wl_AdjtHdrPtr(data, delta)  \
        msil_wl_AdjtUsrHdrPtr(msi_wl_GetUsrBufPtr(data), delta)
#ifdef AP_SW
extern USR_BUF_DESC *msi_wl_GetUsrBufPtr(char *data);
#endif
void *msi_wl_GetUsrDataPtr(char *data);
void *msi_wl_GetInfoPtr(char *data);
extern USR_BUF_DESC *msi_wl_AllocUsrBuf(QHANDLE handle);
extern char *msi_wl_AllocPktBuf(QHANDLE handle);
extern int msi_wl_FreeUsrBuf(USR_BUF_DESC *);
extern int msi_wl_FreePktBuf(char *);
extern USR_BUF_DESC *msi_wl_ChainUsrBuf(USR_BUF_DESC *head, USR_BUF_DESC *tail);
extern BOOLEAN msi_wl_RefillPktQ(QHANDLE handle, UINT8 priority);
extern QHANDLE msi_wl_CreatePktQ(PKT_Q_CFG *cfg, BOOLEAN fillBuf);
extern BOOLEAN msi_wl_BindPktQ(DEV_CTRL *dcp, QHANDLE rx, QHANDLE tx);
extern int msi_wl_AvailBufs(QHANDLE handle, UINT8 priority);
extern int msi_wl_UsedBufs(QHANDLE handle, UINT8 priority);
extern int msi_wl_TotalBufs(QHANDLE handle, UINT8 priority);
extern int msi_wl_PutPkts(QHANDLE handle, USR_BUF_DESC *usp, UINT8 *priority, int option, USR_BUF_DESC **rem);
extern USR_BUF_DESC *msi_wl_GetPkts(QHANDLE handle, UINT8 *priority, int option, UINT32 *npkts, USR_BUF_DESC *reused);
extern USR_BUF_DESC *msi_wl_PeekPkt(QHANDLE handle, UINT8 priority);
extern int msi_wl_DelPkts(QHANDLE handle, UINT8 Qid, int freeBuf, int npkts);
extern int msi_wl_GetPktQlen(QHANDLE handle, int *priority, int state);
extern BOOLEAN msi_wl_CpyPktHdr(USR_BUF_DESC *usp, char *hdr_buf);
extern char *msi_wl_AdjtPktPtr(USR_BUF_DESC *usp, QHANDLE handle);
extern char *msi_wl_AdjtUsrHdrPtr(USR_BUF_DESC *usp, int delta);
extern BOOLEAN msi_wl_FlushPktQ(QHANDLE handle, UINT8 priority, BOOLEAN callPG, BOOLEAN freeBuf);
extern BOOLEAN msi_wl_DestroyPktQ(QHANDLE handle);
void msi_wl_StartTxQ(QHANDLE handle, int Qid);

IO_BUF_DESC *msi_wl_NextRxDesc(QHANDLE handle, UINT8 *priority, UINT8 *pkts);
int msi_wl_PktRcvd(QHANDLE handle, UINT8 priority, UINT8 pkts);
IO_BUF_DESC *msi_wl_NextTxDesc(QHANDLE handle, UINT8 *priority, UINT8 *pkts);
int msi_wl_PktXmtd(QHANDLE handle, UINT8 priority, UINT8 pkts);

#if !defined (ECL_MLME)
extern void *memset(void *, int, size_t);
extern void *memcpy(void *, const void *, size_t);
#endif

#define ResetQ(handle, i)   \
        {   \
        handle->rd[i] = handle->wr[i] = handle->cfg.head[i]; \
        handle->length[i] = 0; handle->flags[i] &= ~QUEUE_FULL; \
        }

extern char *(*genAlloc)(int);
extern void (*genFree)(char *);

#ifdef CYG_MEM_MGMT
typedef void *MHANDLE;
extern void os_var_pool_create( void *, UINT32, MHANDLE, void * );
extern void *os_var_alloc( MHANDLE handle, UINT32 size );
extern void os_var_free( MHANDLE handle, void *buf_p );
extern void os_fix_pool_create( void *, UINT32, UINT32, MHANDLE, void * );
extern void *os_fix_alloc( MHANDLE handle );
extern void os_fix_free( MHANDLE handle, void *buf_p );
#endif  /* CYG_MEM_MGMT */

#ifdef ECOS
#ifndef __SQRAM__
#define __SQRAM__   __attribute__ ((section (".sqbuf")))
#define __SDRAM__   __attribute__ ((section (".sdbuf")))
#define __DATA__    __attribute__ ((section (".data")))
#endif
#else
#define __SQRAM__
#define __SDRAM__
#define __DATA__
#endif 

#ifdef ONE_POOL
extern unsigned gBufAligmentSize;
#define GetUsrBufPtr(x) \
((x)? (USR_BUF_DESC *)((((long)x/gBufAligmentSize)*gBufAligmentSize)) : NULL)

#define msi_wl_GetUsrBufPtr GetUsrBufPtr
#if 0
#define msi_wl_FreeUsrBuf(x)    (*x->qhandle->cfg.free)(x->qhandle->cfg.mem_handle, (char *)x); \
                                x->qhandle->buf_cnt--;

#define msi_wl_FreePktBuf(x)    msi_wl_FreeUsrBuf(GetUsrBufPtr(x))
#endif
#endif
#endif /* !defined(BUFMGT_H) */

#else
#if !defined(BUFMGT_H)
#define BUFMGT_H

#ifdef AP_USE_BUFFER_OFFSET
#define MAX_HDR_SIZE    128      /* max wireless header + RxInfo + 4 */
#define AP_BUFFER_OFFSET MAX_HDR_SIZE
#else
#define MAX_HDR_SIZE    64       /* max wireless header + RxInfo + 4 */
#endif


#ifdef OS_VxW
#define ROUND_N(x, n) ( (((long)x)&(n-1))? ((((long)x)|(n-1))+1) : (x) )
#else
#define ROUND_N(x, n) ( ((long)(x)&(n-1))? ((long)(x)&~(n-1))+n : (long)(x) )
#endif
typedef struct qhandle                  /* Queue handle */
{
    UINT32 handle;               /* Tag for QUEUE_CTRL: 0xAABBCCDD */
}*QHANDLE;                      /* Queue Handle pointer */

typedef struct usr_buf_desc
{
    UINT32  data_offset;
    UINT32  inUse;                /* command and status */
    UINT16  buf_size;               /* buffer size for data */
    UINT16  byte_count;             /* data byte count */
    UINT8   info[128-32];             /* per pkt io control info */
    UINT8   usr_data[32];
    char    *data;                  /* pointer to data/packet */
}USR_BUF_DESC;

#if (POOL_BUFFER_SIZE == 3072) || defined(WB_EXCEPTION)
#define GetUsrBufPtr(x) \
((x)? (USR_BUF_DESC *)((((long)x & 0xfffffc00))) : NULL)
#else
extern unsigned gBufAligmentSize;
#define GetUsrBufPtr(x) \
((x)? (USR_BUF_DESC *)((((long)x/gBufAligmentSize)*gBufAligmentSize)) : NULL)
#endif

#ifdef VXWORKS_BUFFER_OPT
USR_BUF_DESC *msi_wl_GetUsrBufPtr(char *data);
#else
#define msi_wl_GetUsrBufPtr GetUsrBufPtr
#endif

extern char *msi_wl_AllocPktBuf(QHANDLE handle);
extern int msi_wl_FreePktBuf(char *);
extern USR_BUF_DESC *msi_wl_AllocUsrBuf(QHANDLE handle);
extern int msi_wl_FreeUsrBuf(USR_BUF_DESC *);
#ifdef VXWORKS_BUFFER_OPT
int msi_wl_FreeTxFrame(void *txFrame_p);
#endif

#endif /* !defined(BUFMGT_H) */
#endif
