#ifndef _TKIP_H_
#define _TKIP_H_

#include "keyMgmtCommon.h"
#include "bufmgt.h"

#ifdef AP_SW
#define HW_ENCR
#endif
#if !defined(ECL_WPA)
#define WPA_NO_WAIT
#endif

// fixed algorithm "parameters"
#define PHASE1_LOOP_CNT   8    // this needs to be "big enough"
#define TA_SIZE           6    //  48-bit transmitter address
#define TK_SIZE          16    // 128-bit temporal key
#define P1K_SIZE         10    // 80-bit Phase1 key
#define RC4_KEY_SIZE     16    // 128-bit RC4KEY (104 bits unknown)

#define IV_SIZE          4
#define EIV_SIZE         4
#define MIC_SIZE         8
#define ICV_SIZE         4
#define ExtIV                 0x20

#define MICFAILTIMEOUTTIME  5000//5000 x 12  = 60000 msec = 60 sec

// macros for extraction/creation of UINT8/UINT16 values
#define   Lo8(v16)   ((UINT8)( (v16)       & 0x00FF))
#define   Hi8(v16)   ((UINT8)(((v16) >> 8) & 0x00FF))
#define  Lo16(v32)   ((UINT16)( (v32)       & 0xFFFF))
#define  Hi16(v32)   ((UINT16)(((v32) >>16) & 0xFFFF))

#define RotR1(v16)   ((((v16) >> 1) & 0x7FFF) ^ (((v16) & 1) << 15))
#define  Mk16(hi,lo) ((lo) ^ (((UINT16)(hi)) << 8))

// select the Nth 16-bit word of the temporal key UINT8 array TK[]
#define  TK16(N)     Mk16(TK[2*(N)+1],TK[2*(N)])

// S-box lookup: 16 bits --> 16 bits
#define _S_(v16)     (Sbox[0][Lo8(v16)] ^ Sbox[1][Hi8(v16)])

/*
#define ROL32(A,n) (((A) << (n)) | ((A) >> (32-(n))))
#define ROR32(A,n) ROL32((A),32-(n))
*/
#if !defined(ECL_WPA)
#define ROR32(_out,_in,len) __asm("mov	%0, %1, ror %2\n" : "=r" (_out) : "r" (_in), "g" (len))
#define ROL32(out,in,len) ROR32(out,in,(32-len))
#else
#define ROL32(B,A,n) B = (((A) << (n)) | ((A) >> (32-(n))))
#define ROR32(A,B,n) ROL32((A),(B),(32-(n)))
#endif

#define XSWAP(A) (((A & 0xff00ff00) >> 8) | ((A & 0x00ff00ff) << 8))

#define CheckMIC(calc_MIC, rx_MIC) \
        ((*calc_MIC == *(UINT32*)rx_MIC) && \
        (*(calc_MIC + 1) == *(UINT32*)(rx_MIC + 4)))
        
#define InsertIVintoFrmBody(FramePtr, RC4Key, IV32, keyID) \
{ \
    FramePtr[0] = RC4Key[0]; \
    FramePtr[1] = RC4Key[1]; \
    FramePtr[2] = RC4Key[2]; \
    FramePtr[3] = keyID << 6; \
    FramePtr[3] |= ExtIV; \
    *((UINT32 *)(FramePtr+4)) = IV32; \
}

#define appendMIC(data_ptr,MIC) \
{ \
    data_ptr[0] = *MIC; \
    data_ptr[1] = *MIC >> 8; \
    data_ptr[2] = *MIC >> 16; \
    data_ptr[3] = *MIC >> 24; \
    data_ptr[4] = *(MIC+1); \
    data_ptr[5] = *(MIC+1) >> 8; \
    data_ptr[6] = *(MIC+1) >> 16; \
    data_ptr[7] = *(MIC+1) >> 24; \
}

#define block_function(L,R) \
{ \
    UINT32 temp; \
    ROL32(temp,*L,17); \
    *R ^= temp; \
    *L += *R; \
    *R ^= XSWAP(*L); \
    *L += *R; \
    ROL32(temp,*L,3); \
    *R ^= temp; \
    *L += *R; \
    ROR32(temp,*L,2); \
    *R ^= temp; \
    *L += *R; \
}

typedef enum
{
    Rx,
    Tx
}PacketType_e;

typedef enum
{
    NO_MIC_FAILURE,
    FIRST_MIC_FAIL_IN_60_SEC,
    SECOND_MIC_FAIL_IN_60_SEC
}MIC_Fail_State_e;

typedef struct
{
#if !defined(ECL_WPA)
    USR_BUF_DESC *usp;
#else
    IO_BUF_DESC *usp;
#endif
    UINT16 SeqNo;
    UINT8 *pCurrent;
    UINT32 time_out_time;
    UINT32 time_out_timeMSB;
    UINT8 FragNo;
} DeFragBufInfo_t;
#if defined(ECL_WPA)
    //Structure to hold all key information related to the peer Station from driver
typedef struct
{
    UINT16 Resrved;	
    UINT8 PairwiseTempKey[16];
    UINT32 RSNPwkTxMICKey[2];
    UINT32 RSNPwkRxMICKey[2];
    UINT8 PairwiseTempKey_group[16];
    UINT32 RSNPwkTxMICKey_group[2];
    UINT32 RSNPwkRxMICKey_group[2];
    UINT32 TxIV32;
    UINT16 TxIV16;
    UINT32 RxIV32;    
    UINT32 groupTxIV32;
    UINT16 groupTxIV16;
    UINT32 groupRxIV32;    
    UINT8 RSNDataTrafficEnabled;
	UINT16 Phase1KeyTx[5];
    UINT16 Phase1KeyRx[5];
    UINT16 groupPhase1KeyRx[5];

} keyMgmtInfo_t;
#endif

//Function Prototypes
void generateRand(UINT8 *Data, UINT32 length);
void EncryptGrpKey(UINT8 *Encr_Key, UINT8 *IV, UINT8 *Data, UINT16 data_length);
#if !defined(ECL_WPA)
void Insert8021xHdr(Hdr_8021x_t *pHdr, UINT16 data_length);
#endif
void apppendEAPOL_MIC(UINT8 *data, UINT8 *MIC_Data);
void ComputeEAPOL_MIC(UINT8 *data, UINT16 data_length,
                      UINT8 *MIC_Key, UINT8 MIC_Key_length,
                      UINT8 *computed_MIC);
Status_e CompareRSN_IE(UINT8 *pRSN_IE_data, UINT8 *PrbReqRSNIE);
Status_e checkEAPOL_MIC(UINT8 *MIC1, UINT8 *MIC2, UINT8 length);
void genetate_PTK(IEEEtypes_MacAddr_t *pAddr1, IEEEtypes_MacAddr_t *pAddr2,
                     UINT8 *pNonce1, UINT8 *pNonce2, UINT8 *);
void Phase1(UINT16 *P1K, const UINT8 *TK, const UINT8 *TA, UINT32 IV32);
extern void (*Phase2Fp)(UINT8 *RC4KEY, const UINT8 *TK, const UINT16 *P1K, UINT16 IV16);
#ifdef AP_SW
inline void MichaelTx(const IEEEtypes_GenHdr_t *pHdr,
                      const UINT32 *Key, UINT32 *data, UINT32 *res) SECTION_ITCM;

inline void MichaelRx(const IEEEtypes_GenHdr_t *pHdr,
                      const UINT32 *Key, UINT32 *data, UINT32 *res) SECTION_ITCM;
inline void MichaelRxSta(const IEEEtypes_GenHdr_t *pHdr,
                      const UINT32 *Key, UINT32 *data, UINT32 *res) SECTION_ITCM;
inline void MichaelTxSta(const IEEEtypes_GenHdr_t *pHdr,
                      const UINT32 *Key, UINT32 *data, UINT32 *res) SECTION_ITCM;
void (*MichaelRxFp)(const IEEEtypes_GenHdr_t *pHdr,
                    const UINT32 *Key, UINT32 *data, UINT32 *res);
void (*MichaelTxFp)(const IEEEtypes_GenHdr_t *pHdr,
                    const UINT32 *Key, UINT32 *data, UINT32 *res);
void (*MichaelRxStaFp)(const IEEEtypes_GenHdr_t *pHdr,
                      const UINT32 *Key, UINT32 *data, UINT32 *res);
void (*MichaelTxStaFp)(const IEEEtypes_GenHdr_t *pHdr,
                    const UINT32 *Key, UINT32 *data, UINT32 *res);
#endif
void MicErrTimerExpCb(void);
#if !defined(PORTABLE_ARCH)
inline void DoTKIPEncrypt(UINT8 *des, UINT8 *src, UINT32 len, UINT8 *key);
inline UINT32 DoTKIPDecrypt(UINT8 *des, UINT8 *src, UINT32 len, UINT8 *key);
#else
INLINE void DoTKIPEncrypt(UINT8 *des, UINT8 *src, UINT32 len, UINT8 *key);
INLINE UINT32 DoTKIPDecrypt(UINT8 *des, UINT8 *src, UINT32 len, UINT8 *key);
#endif

#endif
