#if !defined(_WLTYPES_H_)
#define _WLTYPES_H_ 
/*
*                Copyright 2003, Marvell Semiconductor, Inc.
* This code contains confidential information of Marvell Semiconductor, Inc.
* No rights are granted herein under any patent, mask work right or copyright
* of Marvell or any third party.
* Marvell reserves the right at its sole discretion to request that this code
* be immediately returned to Marvell. This code is provided "as is".
* Marvell makes no warranties, express, implied or otherwise, regarding its
* accuracy, completeness or performance.
*/

/*!
 * \file    wltypes.h
 * \brief   Basic types common to all the modules must be defined in this file.
 *
*/

#if defined(OS_VxW)
#include "vxWorks.h"
#include <stdbool.h>
#endif


/*! Create type names for native C types for portability reasons */
typedef unsigned long long  UINT64;
typedef signed long long    SINT64;
#ifndef OS_VxW
typedef unsigned long       UINT32;
#endif
typedef signed long         SINT32;
typedef unsigned short      UINT16;
typedef signed short        SINT16;
typedef unsigned char       UINT8;
typedef signed char         SINT8;

typedef unsigned long long  uint64;
typedef signed long long    sint64;
#ifndef OS_VxW
typedef unsigned long       uint32;
#endif
typedef signed long         sint32;
typedef unsigned short      uint16;
typedef signed short        sint16;
typedef unsigned char       uint8;
typedef signed char         sint8;
typedef signed char         int8;
typedef int                 BOOLEAN;
typedef signed int          BIT_FIELD;


/*! Type name for error code returned by most functions in the API.
 *
 * In general, API functions return either BOOLEAN or WL_STATUS
 * BOOLEAN API functions return TRUE when the function was successful and return
 * FALSE when the function failed.
 *
 * WL_STATUS API functions return WL_STATUS_OK when the function was successful
 * and return a error code that is less than 0 when the function failed.
 */
#if defined (PORTABLE_ARCH)
typedef	int Boolean;
#endif

#if !defined (PORTABLE_ARCH)
typedef int WL_STATUS;
#else
#define WL_STATUS Status_e
#endif

/*! Generic status code */
#define WL_STATUS_OK         0
#define WL_STATUS_ERR       -1
#define WL_STATUS_BAD_PARAM -2

/*! BOOLEAN values */
#define FALSE 0
#define TRUE  1

typedef enum
{
    SUCCESS,
    FAIL
}Status_e;

/*! Value for NULL pointer */
#if defined(NULL)
#undef NULL
#endif
#define NULL ((void *)0)
#ifdef __GNUC__
#define PACK_START
#define PACK_END  __attribute__ ((packed))
#else
#define PACK_START   __packed
#define PACK_END
#endif

#ifdef __GNUC__
#define ALLIGNED_START(x)
#define ALLIGNED_END(x) __attribute__ ((aligned(x)))
#else
#define ALLIGNED_START(x) __align(x)
#define ALLIGNED_END(x)
#endif

#ifdef __GNUC__
#define INLINE inline
#else
#define INLINE
#endif

#ifdef ECOS
#define SECTION_ITCM __attribute__ ((section (".itcmbuf")))
#else
#define SECTION_ITCM
#endif

#ifdef BE /* Big Endian */
#define SHORT_SWAP(X) (X)
#define WORD_SWAP(X) (X)
#define LONG_SWAP(X) ((l64)(X))

#else    /* Little Endian */

#define SHORT_SWAP(X) ((X <<8 ) | (X >> 8))

#define WORD_SWAP(X) (((X)&0xff)<<24)+      \
                    (((X)&0xff00)<<8)+      \
                    (((X)&0xff0000)>>8)+    \
                    (((X)&0xff000000)>>24)

#define LONG_SWAP(X) ( (l64) (((X)&0xffULL)<<56)+               \
                            (((X)&0xff00ULL)<<40)+              \
                            (((X)&0xff0000ULL)<<24)+            \
                            (((X)&0xff000000ULL)<<8)+           \
                            (((X)&0xff00000000ULL)>>8)+         \
                            (((X)&0xff0000000000ULL)>>24)+      \
                            (((X)&0xff000000000000ULL)>>40)+    \
                            (((X)&0xff00000000000000ULL)>>56))
#endif

#endif /* _WLTYPES_H_ */
