/******************* (c) Marvell Semiconductor, Inc., 2001 ********************
*  $Header: E:\\APCVSDEPOT/wlan/wlandrv/src/driver/eth/incl/ether.h,v 1.1 2003/09/23 23:49:42 cvu Exp $
*
*  Purpose:
*  This file contains the ethernet driver definitions
*
*  Public Procedures:
*     None.
*
*  Private Procedures:
*     None.
*
*  Notes:
*     None.
*****************************************************************************/
#ifndef W8100_ETH_
#define W8100_ETH_

#include "wltypes.h"
#include "wl_mib.h"
#include "wl_hal.h"


#define NT 32 /* Number of Transmit buffers */

#define NR 64 /* Number of Receive buffers */
#define W8100_INT_THRE 1
#define MAX_BUFF_SIZE 1536
#define ETH_ADDR_GAP (ETHER1_PORT_CONFIG_REG - ETHER0_PORT_CONFIG_REG)

#define NUM_TX_QUEUES 2
#define NUM_RX_QUEUES 1

#define RX_MATCHED_FRAMES   1
#define DISCARD_MATCHED_FRAMES 0

#define SKIP_ADDR_TABLE_ENTRY 1
#define DONOT_SKIP_ADDR_TABLE_ENTRY 0
#define GTREGREAD WL_REGS32

/*#define IFF_PROMISC   0x1 */
/*#define IFF_RUNNING   0x2 */

#define PACKET_HOST   1
#define CHECKSUM_NONE 1

#define SZ_PHY_ADDR 6   /*!< Number of bytes in ethernet MAC address */

/*struct ETHERNET_PRIV; */
struct net_device
{
    /*struct ETHERNET_PRIV *priv; */
/*    int trans_start; */
    UINT16 NetDevType; /* Net device type, see NET_DEV_TYPE */
    UINT8 PhyAddr[SZ_PHY_ADDR]; /*!< The physical address (MAC address) */
    UINT32 flags;    /*!< General control word */
    void *PrivData; /*!< Pointer to net device private data */
    /*!< User defined function to allocate buffer with allignment of "allign" bytes */
    UINT8* (*allocBuf)(UINT32 allign, UINT32 bytes);
    BOOLEAN (*freeBuf)(void *); /*!< User defined function to free buffer */
};


struct sk_buff
{
    unsigned int *data;
    unsigned int len;
    unsigned int pkt_type;
    unsigned int protocol;
    struct net_device *dev;
};


typedef struct ETH_DESC 
{
    volatile UINT32          command_status;
    volatile UINT32          bytecount_reserved;
    volatile UINT8           *buff_pointer;
    volatile struct ETH_DESC *next_desc;
}ETH_DESC;

typedef struct ETH_TX_DESC 
{
  volatile UINT32             command_status;
  volatile UINT32             bytecount_reserved;
  volatile UINT8              *buff_pointer;
  volatile struct ETH_TX_DESC *next_desc;
}ETH_TX_DESC;

typedef struct ETH_RX_DESC 
{
  volatile UINT32              command_status;
  volatile UINT32              buff_size_byte_count;
  volatile UINT8               *buff_pointer;
  volatile struct ETH_RX_DESC  *next_desc;
}ETH_RX_DESC;

typedef struct ETHERNET_PRIV {
    UINT32          ethernetDriverStatus;
    UINT32          ethernetRegisterBaseAddress;
    UINT32          addressTableHashBase;
    UINT32          addressTableHashSize;
    UINT32          addressTableHashMode;
    ETH_TX_DESC     *eth_tx_queue[NUM_TX_QUEUES];
    ETH_RX_DESC     *eth_rx_queue[NUM_RX_QUEUES];
    UINT32          TDN_ETH[NUM_TX_QUEUES];
    UINT32          TDN_ETH_TMP[NUM_TX_QUEUES];
    UINT32          RDN_ETH[NUM_RX_QUEUES];
    UINT32          ethernet_irq;
    NET_DEV_STATS   stats;
    UINT8           port;
}ETHERNET_PRIV;

/*struct pt_regs */
/*    {} */
/*; */

extern NET_DEV dev;
int w8100_eth_add_mac_address ( UINT8 *addr) ;
int w8100_eth_remove_mac_address(UINT8 * addr);
int w8100_eth_init (NET_DEV *);
int w8100_eth_open (NET_DEV *);
int w8100_eth_start_xmit (struct sk_buff * , NET_DEV *);
int w8100_eth_xmit (void *data , UINT32 len, NET_DEV *dev);
int w8100_eth_stop (NET_DEV *);
/*void w8100_eth_int_handler (int , void * , struct pt_regs *); */
int w8100_eth_set_mac_address (NET_DEV *, void *);
struct net_device_stats* w8100_eth_get_stats (NET_DEV *);
int w8100_eth_receive ( NET_DEV *dev, int queue , int * data, int * len) ;
unsigned int w8100_eth_free_tx_queue (NET_DEV *dev , int queue);

struct eth_tx_desc_struct
{
    volatile UINT32 command_status;
    volatile UINT32 bytecount_reserved;
    volatile unsigned char * buff_pointer; /* Note - This will not work for 64 bit addressing */
    ;
    volatile struct eth_tx_desc_struct * next_desc;
};

struct eth_rx_desc_struct
{
    volatile UINT32 command_status;
    volatile UINT32 buff_size_byte_count;
    volatile unsigned char * buff_pointer;
    volatile struct eth_rx_desc_struct * next_desc;
};

struct net_device_stats
{
    UINT32 rx_packets;
    UINT32 tx_packets;
    UINT32 rx_bytes;
    UINT32 tx_bytes;
    UINT32 rx_errors;
    UINT32 rx_dropped;
    UINT32 tx_dropped;
    UINT32 multicast;
    UINT32 collisions;
    UINT32 rx_length_errors;
    UINT32 rx_crc_errors;
    UINT32 tx_fifo_errors;

};

struct w8100_ethernet_priv
{
    /* other stuff can be added here such as GT96100 counters etc... */
    struct eth_tx_desc_struct *eth_tx_queue[NUM_TX_QUEUES];
    struct eth_rx_desc_struct *eth_rx_queue[NUM_RX_QUEUES];
    struct sk_buff *eth_tx_skb_pointer[NUM_TX_QUEUES][NT];
    struct sk_buff *eth_rx_skb_pointer[NUM_RX_QUEUES][NR];
    unsigned int TDN_ETH[NUM_TX_QUEUES] , TDN_ETH_TMP[NUM_TX_QUEUES] , RDN_ETH[NUM_RX_QUEUES];
    unsigned int ethernet_irq;
    struct net_device_stats stats;
    unsigned char port;
    /*  spinlock_t lock; */
};

extern NET_DEV eth_dev;
extern UINT8 ether_port_down;  /*ether port down flag */
#endif /* EVB96132_ETH_ */
