#if !defined(_MCU_H_)
#define _MCU_H_ 
/*
 *                Copyright 2003, Marvell Semiconductor, Inc.
 * This code contains confidential information of Marvell Semiconductor, Inc.
 * No rights are granted herein under any patent, mask work right or copyright
 * of Marvell or any third party.
 * Marvell reserves the right at its sole discretion to request that this code
 * be immediately returned to Marvell. This code is provided "as is".
 * Marvell makes no warranties, express, implied or otherwise, regarding its
 * accuracy, completeness or performance.
 */


/*!
 * \file    mcu.h
 * \brief   This file contains generic macro for accessing hardware registers
 *          and others common to the HAL module.
 *
 */



/*============================================================================= */
/*                                PUBLIC MACROS */
/*============================================================================= */
/*!
 *  Marvell proprietary Rx header that accompany each 802.11 frame in the
 *  hardware MAC receive buffer
 */
typedef PACK_START struct _W8310_MRVL_RX_HDR
{
    UINT8 RxParam;          /* Receive parameters 0: received using OFDM */
    /*   1: received using CCK */
    UINT8 Sq2;              /* Signal Quality 2 */
    UINT16 Signature;        /* Signature to distinguish fragmented or whole frame */
    UINT8 AntPream;         /* Antenna/Preamble */
    UINT8 Sq1;              /* Signal Quality 1 */
    UINT8 Rate;             /* rate at which frame was received */
    UINT8 RSSI;             /* RF Signal Strength Indicator */
    UINT16 QosControl;       /* QoS Control field */
    UINT16 Service;
}
PACK_END W8310_MRVL_RX_HDR;

typedef PACK_START struct _W8310_MRVL_RX_HDR_DEFRAG
{
   UINT16   DeFragPtr;
   UINT16   Signature;
} 
PACK_END W8310_MRVL_RX_HDR_DEFRAG;

#define NON_FRAG_BUF_SIGNATURE   0xBEEF   /* Signature in Non fragmented Rx buffers */


/* */
/* For the promiscuous mode, setting the following are allowed settings: */
/*   0x0 = Filter on BSS ID */
/*   0x2 = Do not filter on BSS ID */
/*   0xA = Enable receipt of messages */
/* */
/* The first two settings are mutually exclusive, but either of them may */
/* be combined with the third setting. The following macros are provided */
/* for control of this register. */
/* */

/* */
/* 1) Retain whatever the message receipt bit is */
/* 2) Set the bit pattern to enable filtering on BSS ID */
/* */
#define WL_ENABLE_BSSID_FILTER                   \
           MACRO_START                            \
              int _x;                             \
              WL_READ_WORD(RX_PROMISCUOUS, _x);  \
              _x &= WL_ALLOW_RCV_MSGS;           \
              _x |= WL_FILTER_ON_BSS_ID;         \
              WL_WRITE_WORD(RX_PROMISCUOUS, _x); \
           MACRO_END


/* */
/* 1) Retain whatever the message receipt bit is */
/* 2) Set the bit pattern to disable filtering on BSS ID (allow all */
/*    messages irrespective of BSS ID) */
/* */
#define WL_DISABLE_BSSID_FILTER                  \
           MACRO_START                            \
              int _x;                             \
              WL_READ_WORD(RX_PROMISCUOUS, _x);  \
              _x &= WL_ALLOW_RCV_MSGS;           \
              _x |= WL_ALLOW_ALL_BSS_ID;             \
              WL_WRITE_WORD(RX_PROMISCUOUS, _x); \
           MACRO_END


/*
 *  1) Retain whatever the BSS ID filtering bit is set
 *  2) Set the bit to enable message receipt
 */
#define WL_ENABLE_MSG_RECEIPT                                  \
           MACRO_START                                          \
              int _x;                                           \
              WL_READ_WORD(RX_PROMISCUOUS, _x);                \
              _x &= (WL_FILTER_ON_BSS_ID | WL_ALLOW_ALL_BSS_ID);  \
              _x |= WL_ALLOW_RCV_MSGS;                         \
              WL_WRITE_WORD(RX_PROMISCUOUS, _x);               \
           MACRO_END


/*
 *  1) Retain whatever the BSS ID filtering bit is set
 *  2) Clear the bit that enables message receipt
 */
#define WL_DISABLE_MSG_RECEIPT                               \
           MACRO_START                                          \
              int _x;                                           \
              WL_READ_WORD(RX_PROMISCUOUS, _x);                \
              _x &= (WL_FILTER_ON_BSS_ID | WL_ALLOW_ALL_BSS_ID);  \
              _x &= WL_STOP_RCV_MSGS;                          \
              WL_WRITE_WORD(RX_PROMISCUOUS, _x);               \
           MACRO_END


/*!
 * Set the bit pattern to enable filtering on BSS ID and
 *    to receive messages
 */
#define WL_ENABLE_BSSID_FILTER_AND_MSG_RECEIPT                  \
           MACRO_START                                           \
              int _x = 0;                                        \
              _x = (WL_FILTER_ON_BSS_ID | WL_ALLOW_RCV_MSGS);  \
              WL_WRITE_WORD(RX_PROMISCUOUS, _x);                \
           MACRO_END



/*!
 *  Set the bit pattern to enable filtering on BSS ID and
 *  to receive messages
 */
#define WL_ENABLE_BSSID_FILTER_AND_MSG_RECEIPT_AND_BRDCST_SSID  \
           MACRO_START                                           \
              int _x = 0;                                        \
              _x = (WL_FILTER_ON_BSS_ID |                       \
                    WL_ALLOW_RCV_MSGS   |                       \
                    WL_ALLOW_BRDCST_SSID);                      \
              WL_WRITE_WORD(RX_PROMISCUOUS, _x);                \
           MACRO_END

/*!
 *  Set the bit pattern to disable filtering on BSS ID and
 *  to receive messages
 */
#define WL_DISABLE_BSSID_FILTER_AND_ENABLE_MSG_RECEIPT          \
           MACRO_START                                           \
              int _x = 0;                                        \
              _x = (WL_ALLOW_ALL_BSS_ID | WL_ALLOW_RCV_MSGS);  \
              WL_WRITE_WORD(RX_PROMISCUOUS, _x);                \
           MACRO_END

/*!
 * Set the bit pattern to enable filtering on PROMISCUOUS mode and
 *    to receive messages
 */
#define WL_ENABLE_PROMISCUOUS_MODE_AND_MSG_RECEIPT          \
           MACRO_START                                           \
              int _x = 0;                                        \
              _x = (WL_PROMISCUOUS_ON | WL_ALLOW_RCV_MSGS);  \
              WL_WRITE_WORD(RX_PROMISCUOUS, _x);                \
           MACRO_END


/*!
 *  set IV to the HW register for HW to encrypt Tx packets
 */
#define WL_TxSetWepIV(IV) \
        WL_WRITE_WORD(RX_SW_ENC_KEY0, (WL_REGS32(RX_SW_ENC_KEY0)&0xFF000000)|IV)

#endif /* _MCU_H_ */


