#if !defined(_WLMAC_H_)
#define _WLMAC_H_ 
/*
*                Copyright 2003, Marvell Semiconductor, Inc.
* This code contains confidential information of Marvell Semiconductor, Inc.
* No rights are granted herein under any patent, mask work right or copyright
* of Marvell or any third party.
* Marvell reserves the right at its sole discretion to request that this code
* be immediately returned to Marvell. This code is provided "as is".
* Marvell makes no warranties, express, implied or otherwise, regarding its
* accuracy, completeness or performance.
*/

/*!
 * \file    wlmac.h
 * \brief   Definitions for 802.11 MAC unit.
 *
*/

/* Macro to compute MAC register addresses */
#define MAC_REG_ADDR(offset) ((WL_WLMAC_CFG_BASE) | offset)
#define AEU_REG_ADDR(offset) ((WL_LBU_GPU_CFG_BASE) | offset)

/*--------------------------------------------------------------*/
/*  TX Core registers                                           */
/*--------------------------------------------------------------*/
#define TX_PTR_BASE        MAC_REG_ADDR(0x0000)
#define TX_BCN_BASE        MAC_REG_ADDR(0x0004)
#define TX_PR_BASE         MAC_REG_ADDR(0x0008) /* Probe Response Base */
#define TX_DONE_WRPTR      MAC_REG_ADDR(0x0010)
#define TX_DONE_RDPTR      MAC_REG_ADDR(0x0014)

#define TX_PS_WRPTR        MAC_REG_ADDR(0x0018)
#define TX_PS_RDPTR        MAC_REG_ADDR(0x001C)
#define TX_CF_WRPTR        MAC_REG_ADDR(0x0020)
#define TX_CF_RDPTR        MAC_REG_ADDR(0x0024)
#define TX_MC_WRPTR        MAC_REG_ADDR(0x0028)
#define TX_MC_RDPTR        MAC_REG_ADDR(0x002C)
#define TX_CM_WRPTR        MAC_REG_ADDR(0x0030)
#define TX_CM_RDPTR        MAC_REG_ADDR(0x0034)
#define TX_BW_WRPTR        MAC_REG_ADDR(0x0038)
#define TX_BW_RDPTR        MAC_REG_ADDR(0x003C)
#define TX_TCQ0_WRPTR      MAC_REG_ADDR(0x0040)
#define TX_TCQ0_RDPTR      MAC_REG_ADDR(0x0044)
#define TX_TCQ1_WRPTR      MAC_REG_ADDR(0x0048)
#define TX_TCQ1_RDPTR      MAC_REG_ADDR(0x004C)
#define TX_TCQ2_WRPTR      MAC_REG_ADDR(0x0050)
#define TX_TCQ2_RDPTR      MAC_REG_ADDR(0x0054)
#define TX_TCQ3_WRPTR      MAC_REG_ADDR(0x0058)
#define TX_TCQ3_RDPTR      MAC_REG_ADDR(0x005C)
#define TX_TCQ4_WRPTR      MAC_REG_ADDR(0x0060)
#define TX_TCQ4_RDPTR      MAC_REG_ADDR(0x0064)
#define TX_TCQ5_WRPTR      MAC_REG_ADDR(0x0068)
#define TX_TCQ5_RDPTR      MAC_REG_ADDR(0x006C)
#define TX_TCQ6_WRPTR      MAC_REG_ADDR(0x0070)
#define TX_TCQ6_RDPTR      MAC_REG_ADDR(0x0074)
#define TX_TCQ7_WRPTR      MAC_REG_ADDR(0x0078)
#define TX_TCQ7_RDPTR      MAC_REG_ADDR(0x007C)
#define TX_TCQ_PERIODIC    MAC_REG_ADDR(0x0090)
#define TX_CW0_MIN         MAC_REG_ADDR(0x00A0)    /*minimum size of contention windows 0 */
#define TX_CW0_MAX         MAC_REG_ADDR(0x00A4)    /*maximum size of contention windows 0 */
#define TX_CW1_MIN         MAC_REG_ADDR(0x00A8)    /*minimum size of contention windows 1 */
#define TX_CW1_MAX         MAC_REG_ADDR(0x00Ac)    /*maximum size of contention windows 1 */
#define TX_CW2_MIN         MAC_REG_ADDR(0x00B0)    /*minimum size of contention windows 2 */
#define TX_CW2_MAX         MAC_REG_ADDR(0x00B4)    /*maximum size of contention windows 2 */
#define TX_CW3_MIN         MAC_REG_ADDR(0x00B8)    /*minimum size of contention windows 3 */
#define TX_CW3_MAX         MAC_REG_ADDR(0x00BC)    /*maximum size of contention windows 3 */
#define TX_CW4_MIN         MAC_REG_ADDR(0x00C0)    /*minimum size of contention windows 4 */
#define TX_CW4_MAX         MAC_REG_ADDR(0x00C4)    /*maximum size of contention windows 4 */
#define TX_CW5_MIN         MAC_REG_ADDR(0x00C8)    /*minimum size of contention windows 5 */
#define TX_CW5_MAX         MAC_REG_ADDR(0x00CC)    /*maximum size of contention windows 5 */
#define TX_CW6_MIN         MAC_REG_ADDR(0x00D0)    /*minimum size of contention windows 6 */
#define TX_CW6_MAX         MAC_REG_ADDR(0x00D4)    /*maximum size of contention windows 6 */
#define TX_CW7_MIN         MAC_REG_ADDR(0x00D8)    /*minimum size of contention windows 7 */
#define TX_CW7_MAX         MAC_REG_ADDR(0x00DC)    /*maximum size of contention windows 7 */
#define TX_BCN_CW          MAC_REG_ADDR(0x00E0)    /*beacon back-off contention windows */
#define TX_DATA_RATE0      MAC_REG_ADDR(0x0100)        
#define TX_DATA_RATE1      MAC_REG_ADDR(0x0104)        
#define TX_DATA_RATE2      MAC_REG_ADDR(0x0108)        
#define TX_DATA_RATE3      MAC_REG_ADDR(0x010C)        
#define TX_DATA_RATE4      MAC_REG_ADDR(0x0110)        
#define TX_DATA_RATE5      MAC_REG_ADDR(0x0114)        
#define TX_DATA_RATE6      MAC_REG_ADDR(0x0118)        
#define TX_DATA_RATE7      MAC_REG_ADDR(0x011C)        
#define TX_DATA_RATE8      MAC_REG_ADDR(0x0120)        
#define TX_DATA_RATE9      MAC_REG_ADDR(0x0124)        
#define TX_DATA_RATE10     MAC_REG_ADDR(0x0128)        
#define TX_DATA_RATE11     MAC_REG_ADDR(0x012C)        
#define TX_DATA_RATE12     MAC_REG_ADDR(0x0130)        
#define TX_DATA_RATE13     MAC_REG_ADDR(0x0134)        
#define TX_PDU_FACTOR0     MAC_REG_ADDR(0x0140)    /*The factor in uSec/8bytes used to */
#define TX_PDU_FACTOR1     MAC_REG_ADDR(0x0144)    /*  determine the transmission time of */
#define TX_PDU_FACTOR2     MAC_REG_ADDR(0x0148)    /*  a frame at DataRate0,1, .. or 7 */
#define TX_PDU_FACTOR3     MAC_REG_ADDR(0x014C)
#define TX_PDU_FACTOR4     MAC_REG_ADDR(0x0150)
#define TX_PDU_FACTOR5     MAC_REG_ADDR(0x0154)
#define TX_PDU_FACTOR6     MAC_REG_ADDR(0x0158)
#define TX_PDU_FACTOR7     MAC_REG_ADDR(0x015C)
#define TX_PDU_FACTOR8     MAC_REG_ADDR(0x0160)
#define TX_PDU_FACTOR9     MAC_REG_ADDR(0x0164)
#define TX_PDU_FACTOR10    MAC_REG_ADDR(0x0168)
#define TX_PDU_FACTOR11    MAC_REG_ADDR(0x016C)
#define TX_PDU_FACTOR12    MAC_REG_ADDR(0x0170)
#define TX_PDU_FACTOR13    MAC_REG_ADDR(0x0174)
#define TX_MIN_PDU_TIME0   MAC_REG_ADDR(0x0180)
#define TX_MIN_PDU_TIME1   MAC_REG_ADDR(0x0184)
#define TX_MIN_PDU_TIME2   MAC_REG_ADDR(0x0188)
#define TX_MIN_PDU_TIME3   MAC_REG_ADDR(0x018C)
#define TX_MIN_PDU_TIME4   MAC_REG_ADDR(0x0190)
#define TX_MIN_PDU_TIME5   MAC_REG_ADDR(0x0194)
#define TX_MIN_PDU_TIME6   MAC_REG_ADDR(0x0198)
#define TX_MIN_PDU_TIME7   MAC_REG_ADDR(0x019C)
#define TX_MIN_PDU_TIME8   MAC_REG_ADDR(0x01A0)
#define TX_MIN_PDU_TIME9   MAC_REG_ADDR(0x01A4)
#define TX_MIN_PDU_TIME10  MAC_REG_ADDR(0x01A8)
#define TX_MIN_PDU_TIME11  MAC_REG_ADDR(0x01AC)
#define TX_MIN_PDU_TIME12  MAC_REG_ADDR(0x01B0)
#define TX_MIN_PDU_TIME13  MAC_REG_ADDR(0x01B4)
#define TX_ACK_CTS_DURA0   MAC_REG_ADDR(0x01C0)
#define TX_ACK_CTS_DURA1   MAC_REG_ADDR(0x01C4)
#define TX_ACK_CTS_DURA2   MAC_REG_ADDR(0x01C8)
#define TX_ACK_CTS_DURA3   MAC_REG_ADDR(0x01CC)
#define TX_ACK_CTS_DURA4   MAC_REG_ADDR(0x01D0)
#define TX_ACK_CTS_DURA5   MAC_REG_ADDR(0x01D4)
#define TX_ACK_CTS_DURA6   MAC_REG_ADDR(0x01D8)
#define TX_ACK_CTS_DURA7   MAC_REG_ADDR(0x01DC)
#define TX_ACK_CTS_DURA8   MAC_REG_ADDR(0x01E0)
#define TX_ACK_CTS_DURA9   MAC_REG_ADDR(0x01E4)
#define TX_ACK_CTS_DURA10  MAC_REG_ADDR(0x01E8)
#define TX_ACK_CTS_DURA11  MAC_REG_ADDR(0x01EC)
#define TX_ACK_CTS_DURA12  MAC_REG_ADDR(0x01F0)
#define TX_ACK_CTS_DURA13  MAC_REG_ADDR(0x01F4)
#define TX_SRL             MAC_REG_ADDR(0x0204)      /*Short retry limit */
#define TX_LRL             MAC_REG_ADDR(0x0208)      /*Long retry limit */
#define TX_ACK_TIMEOUTA    MAC_REG_ADDR(0x0210)
#define TX_ACK_TIMEOUTB    MAC_REG_ADDR(0x0214)
#define TX_ACK_TIMEOUTG    MAC_REG_ADDR(0x0218)
#define TX_SHORT_PLCP_TIMEA MAC_REG_ADDR(0x021C)
#define TX_LONG_PLCP_TIMEA  MAC_REG_ADDR(0x0220)
#define TX_SHORT_PLCP_TIMEB MAC_REG_ADDR(0x0224)
#define TX_LONG_PLCP_TIMEB  MAC_REG_ADDR(0x0228)
#define TX_SHORT_PLCP_TIMEG MAC_REG_ADDR(0x022C)
#define TX_LONG_PLCP_TIMEG  MAC_REG_ADDR(0x0230)
#define TX_PS_POLL         MAC_REG_ADDR(0x0234)      /*[0] = 0 PS-Poll frame not valid */
#define TX_PS_POLL_RATE_ID MAC_REG_ADDR(0x0238)
#define TX_PS_POLL_RETRY   MAC_REG_ADDR(0x023C)
#define TX_BKOFF_TMR_FACTOR MAC_REG_ADDR(0x0248)
#define TX_QLOCKED         MAC_REG_ADDR(0x0250)
#define   BCN_BUSY           (1<<5)                 /* 1 - SW can't write to BCN buffer */
#define TX_EDCF_TXOP0      MAC_REG_ADDR(0x0260)
#define TX_EDCF_TXOP1      MAC_REG_ADDR(0x0264)
#define TX_EDCF_TXOP2      MAC_REG_ADDR(0x0268)
#define TX_EDCF_TXOP3      MAC_REG_ADDR(0x026C)
#define TX_EDCF_TXOP4      MAC_REG_ADDR(0x0270)
#define TX_EDCF_TXOP5      MAC_REG_ADDR(0x0274)
#define TX_EDCF_TXOP6      MAC_REG_ADDR(0x0278)
#define TX_EDCF_TXOP7      MAC_REG_ADDR(0x027C)
#define TX_TCDI0           MAC_REG_ADDR(0x0280)
#define TX_TCDI1           MAC_REG_ADDR(0x0284)
#define TX_TCDI2           MAC_REG_ADDR(0x0288)
#define TX_TCDI3           MAC_REG_ADDR(0x028C)
#define TX_TCDI4           MAC_REG_ADDR(0x0290)
#define TX_TCDI5           MAC_REG_ADDR(0x0294)
#define TX_TCDI6           MAC_REG_ADDR(0x0298)
#define TX_TCDI7           MAC_REG_ADDR(0x029C)
/* Each Traffic Category has its own TX lifetime */
#define TX_MAX_TX_LIFETIME0 MAC_REG_ADDR(0x02B0)
#define TX_MAX_TX_LIFETIME1 MAC_REG_ADDR(0x02B4)
#define TX_MAX_TX_LIFETIME2 MAC_REG_ADDR(0x02B8)
#define TX_MAX_TX_LIFETIME3 MAC_REG_ADDR(0x02BC)
#define TX_MAX_TX_LIFETIME4 MAC_REG_ADDR(0x02C0)
#define TX_MAX_TX_LIFETIME5 MAC_REG_ADDR(0x02C4)
#define TX_MAX_TX_LIFETIME6 MAC_REG_ADDR(0x02C8)
#define TX_MAX_TX_LIFETIME7 MAC_REG_ADDR(0x02CC)

#define TX_FORCE_TX         MAC_REG_ADDR(0x02D0)

#define TX_MODE            MAC_REG_ADDR(0x0500)
#define   STA_MODE           (0x0)                    /* 1 - AP */
#define   AP_MODE            (0x1)                    /* 1 - AP */
#define   IBSS_MODE          (0x2)                    /* 1 - IBSS */

#define TX_GRP_MASK_LO     MAC_REG_ADDR(0x0520)
#define TX_GRP_MASK_HI     MAC_REG_ADDR(0x0524)

#define TX_MAC_ADDR_LO     MAC_REG_ADDR(0x0528)
#define TX_MAC_ADDR_HI     MAC_REG_ADDR(0x052C)
#define TX_BSSID_LO        MAC_REG_ADDR(0x0530)
#define TX_BSSID_HI        MAC_REG_ADDR(0x0534)
#define TX_AID             MAC_REG_ADDR(0x0538)
#define TX_SSID_LEN        MAC_REG_ADDR(0x0550)
#define TX_SSID0           MAC_REG_ADDR(0x0554)      /*SSID characters 0-3 */
#define TX_SSID1           MAC_REG_ADDR(0x0558)
#define TX_SSID2           MAC_REG_ADDR(0x055C)
#define TX_SSID3           MAC_REG_ADDR(0x0560)
#define TX_SSID4           MAC_REG_ADDR(0x0564)
#define TX_SSID5           MAC_REG_ADDR(0x0568)
#define TX_SSID6           MAC_REG_ADDR(0x056C)
#define TX_SSID7           MAC_REG_ADDR(0x0570)
#define TX_PHY_DELAY       MAC_REG_ADDR(0x0580)

#define TX_TSF_LO          MAC_REG_ADDR(0x0600)
#define TX_TSF_HI          MAC_REG_ADDR(0x0604)
#define TX_ACC_INTVL_LO    MAC_REG_ADDR(0x0608)
#define TX_ACC_INTVL_HI    MAC_REG_ADDR(0x060C)
#define TX_SW_DTIM_CNT     MAC_REG_ADDR(0x0610)
#define TX_SW_CFP_CNT      MAC_REG_ADDR(0x0614)
#define TX_BCN_PERIOD      MAC_REG_ADDR(0x0618)
#define TX_ATIM_WINDOW     MAC_REG_ADDR(0x061C)
#define TX_DTIM_PERIOD     MAC_REG_ADDR(0x0620)
#define TX_LISTEN_INTVL    MAC_REG_ADDR(0x0624)
#define TX_CFP_PERIOD      MAC_REG_ADDR(0x0628)
#define TX_CFP_MAX_DUR     MAC_REG_ADDR(0x062C)
#define TX_LOCAL_NAV       MAC_REG_ADDR(0x0630)
#define TX_TIMER_COMMIT    MAC_REG_ADDR(0x0640)

#define TX_EXT_BBU         MAC_REG_ADDR(0x0900)

#define COMMIT_TSF             (1<<0)
#define COMMIT_ACC_INTVL       (1<<1)
#define COMMIT_DTIM_CNT        (1<<2)
#define COMMIT_CFP_CNT         (1<<3)
#define COMMIT_BCN_PERD        (1<<4)
#define COMMIT_ATIM_WINDW      (1<<5)
#define COMMIT_DTIM_PERD       (1<<6)
#define COMMIT_CFP_PERD        (1<<7)
#define COMMIT_CFP_MAX_DUR     (1<<8)
#define COMMIT_ALL             (0x1FF)

#define TX_TIMER_RUN          MAC_REG_ADDR(0x0644)
#define TX_RTS_NAV_TIMEOUT0   MAC_REG_ADDR(0x06B0)
#define TX_RTS_NAV_TIMEOUT1   MAC_REG_ADDR(0x06B4)
#define TX_RTS_NAV_TIMEOUT2   MAC_REG_ADDR(0x06B8)
#define TX_RTS_NAV_TIMEOUT3   MAC_REG_ADDR(0x06BC)
#define TX_RTS_NAV_TIMEOUT4   MAC_REG_ADDR(0x06C0)
#define TX_RTS_NAV_TIMEOUT5   MAC_REG_ADDR(0x06C4)
#define TX_RTS_NAV_TIMEOUT6   MAC_REG_ADDR(0x06C8)
#define TX_RTS_NAV_TIMEOUT7   MAC_REG_ADDR(0x06CC)
#define TX_RTS_NAV_TIMEOUT8   MAC_REG_ADDR(0x06D0)
#define TX_RTS_NAV_TIMEOUT9   MAC_REG_ADDR(0x06D4)
#define TX_RTS_NAV_TIMEOUT10  MAC_REG_ADDR(0x06D8)
#define TX_RTS_NAV_TIMEOUT11  MAC_REG_ADDR(0x06DC)
#define TX_RTS_NAV_TIMEOUT12  MAC_REG_ADDR(0x06E0)
#define TX_RTS_NAV_TIMEOUT13  MAC_REG_ADDR(0x06E4)

/*--------------------------------------------------------------*/
/*  RX Core registers                                           */
/*--------------------------------------------------------------*/
#define RX_PROMISCUOUS     MAC_REG_ADDR(0x0300)
#define RX_FILTER_CMND     MAC_REG_ADDR(0x0310)
#define FILTER_CMND_RD       (1<<8)
#define FILTER_CMND_WR       (1<<9)
#define FILTER_CMND_BUSY     (1<<16)
#define FILTER_CMND_STAT     (1<<17)
#define FILTER_MCADDR_OFFSET 0xC0
#define FILTER_SF_SIZE       0x100
#define RX_SW_SFDATA_LO    MAC_REG_ADDR(0x0314)
#define RX_SW_SFDATA_HI    MAC_REG_ADDR(0x0318)
#define RX_WEP_ENABLE      MAC_REG_ADDR(0x0320)
#define RX_WEP_CMD         MAC_REG_ADDR(0x0324)
#define WEP_CMD_SW_READ      (1<<8)   /* cmd to read dedicate wep SRAM */
#define WEP_CMD_SW_WRITE     (1<<9)   /* cmd to write to dedicate wep SRAM */
#define WEP_CMD_WEP_STOP     (1<<11)
#define WEP_CMD_WEP_START    (1<<12)  /* start TX wep */
#define WEP_CMD_WEP128       (1<<13)  /* set 128 wep2 mode */
#define WEP_CMD_DECRYPT      (1<<14)  /* set to decrypt packets */
#define WEP_CMD_HWBUSY       (1<<16)  /* SRAM is busy, can't write */
#define WEP_CMD_RXWEP_BUSY   (1<<17)  /* HW is busy to do RX wep */
#define WEP_CMD_TXWEP_TRIG   (1<<18)  /* HW rcvd TX wep requet */
#define WEP_CMD_TXWEP_BUSY   (1<<19)  /* HW is busy to do TX wep */
#define WEP_CMD_ICV_PASS     (1<<20)  /* the ICV of the decrypted packet is good */
#define RX_SW_SWDATA_LO    MAC_REG_ADDR(0x0328)
#define RX_SW_SWDATA_HI    MAC_REG_ADDR(0x032C)
#define RX_ADDR2_MODE_LO   MAC_REG_ADDR(0x0340)
#define RX_ADDR2_MODE_HI   MAC_REG_ADDR(0x0344)
#define RX_AES_MODE_LO     MAC_REG_ADDR(0x0350)
#define RX_AES_MODE_HI     MAC_REG_ADDR(0x0354)
#define RX_KEY0_W0         MAC_REG_ADDR(0x0360)
#define RX_KEY0_W1         MAC_REG_ADDR(0x0364)
#define RX_KEY0_W2         MAC_REG_ADDR(0x0368)
#define RX_KEY0_W3         MAC_REG_ADDR(0x036C)
#define RX_KEY1_W0         MAC_REG_ADDR(0x0370)
#define RX_KEY1_W1         MAC_REG_ADDR(0x0374)
#define RX_KEY1_W2         MAC_REG_ADDR(0x0378)
#define RX_KEY1_W3         MAC_REG_ADDR(0x037C)
#define RX_KEY2_W0         MAC_REG_ADDR(0x0380)
#define RX_KEY2_W1         MAC_REG_ADDR(0x0384)
#define RX_KEY2_W2         MAC_REG_ADDR(0x0388)
#define RX_KEY2_W3         MAC_REG_ADDR(0x038C)
#define RX_KEY3_W0         MAC_REG_ADDR(0x0390)
#define RX_KEY3_W1         MAC_REG_ADDR(0x0394)
#define RX_KEY3_W2         MAC_REG_ADDR(0x0398)
#define RX_KEY3_W3         MAC_REG_ADDR(0x039C)
#define RX_SW_ENC_WRADDR   MAC_REG_ADDR(0x03B0)
#define RX_SW_ENC_RDADDR   MAC_REG_ADDR(0x03B4)
#define RX_SW_WR_MASK_OFF  MAC_REG_ADDR(0x03B8)
#define RX_SW_WR_MASK_ON   MAC_REG_ADDR(0x03BC)
#define RX_SW_ENC_LEN      MAC_REG_ADDR(0x03C0)
#define RX_SW_ENC_KEY0     MAC_REG_ADDR(0x03D0)
#define RX_SW_ENC_KEY1     MAC_REG_ADDR(0x03D4)
#define RX_SW_ENC_KEY2     MAC_REG_ADDR(0x03D8)
#define RX_SW_ENC_KEY3     MAC_REG_ADDR(0x03DC)
#define RX_MAX_LIFE_TM     MAC_REG_ADDR(0x03E0)
#define RX_DNF_BUF_SIZE    MAC_REG_ADDR(0x03F0)
#define RX_DNF_BUF_BASE    MAC_REG_ADDR(0x03F4)

#define RX_F0_BUF_SIZE     MAC_REG_ADDR(0x03F8)
#define RX_F0_BUF_BASE     MAC_REG_ADDR(0x03FC)
#define RX_F1_BUF_SIZE     MAC_REG_ADDR(0x0400)
#define RX_F1_BUF_BASE     MAC_REG_ADDR(0x0404)
#define RX_F2_BUF_SIZE     MAC_REG_ADDR(0x0408)
#define RX_F2_BUF_BASE     MAC_REG_ADDR(0x040C)

#define RX_MNF_BUF_SIZE    MAC_REG_ADDR(0x0410)
#define RX_MNF_BUF_BASE    MAC_REG_ADDR(0x0414) /*base address of non-fragment management frame buffer */
#define RX_BCN_BUF_SIZE    MAC_REG_ADDR(0x0418) /*the total size of the beacon buffer array */
#define RX_BCN_BUF_BASE    MAC_REG_ADDR(0x041C) /*base address of beacon buffer array */
#define RX_DNF_RD_PTR      MAC_REG_ADDR(0x0430)
#define RX_F0_RD_PTR       MAC_REG_ADDR(0x0434)
#define RX_F1_RD_PTR       MAC_REG_ADDR(0x0438)
#define RX_F2_RD_PTR       MAC_REG_ADDR(0x043C)
#define RX_MNF_RD_PTR      MAC_REG_ADDR(0x0440)
#define RX_DNF_WR_PTR      MAC_REG_ADDR(0x0444)
#define RX_F0_WR_PTR       MAC_REG_ADDR(0x0448)
#define RX_F1_WR_PTR       MAC_REG_ADDR(0x044C)
#define RX_F2_WR_PTR       MAC_REG_ADDR(0x0450)
#define RX_MNF_WR_PTR      MAC_REG_ADDR(0x0454)
#define RX_BCN_WR_PTR      MAC_REG_ADDR(0x0458) /* Ptr to next wr addr of control buffer */
#define RX_BCN_BUF_LEN     MAC_REG_ADDR(0x045C) /* Configured beacon frame length */
#define RX_PWRSAVE_MODE_LSB   MAC_REG_ADDR(0x0460) /* Pwr save mode least significant byte */
#define RX_PWRSAVE_MODE_MSB   MAC_REG_ADDR(0x0464) /* Pwr save mode most significant byte */
#define RX_MISSING_BCN     MAC_REG_ADDR(0x0468) /* Beacon missed before processor interrupt */
#define RX_BLOCK_FRAME0    MAC_REG_ADDR(0x046C)
#define RX_BLOCK_FRAME1    MAC_REG_ADDR(0x0470)

/*AEU Registers */
#define STS_REG            AEU_REG_ADDR(0x0800)
#define CFG_REG            AEU_REG_ADDR(0x0804)
#define IKEY_REG           AEU_REG_ADDR(0x0808)
#define NONCE_REG          AEU_REG_ADDR(0x0828)
#define SRC_REG            AEU_REG_ADDR(0x0838)
#define DST_REG            AEU_REG_ADDR(0x083C)
#define CBC_IV_REG         AEU_REG_ADDR(0x0840)
#define CBC_MIC_HDR1       AEU_REG_ADDR(0x0850)
#define CBC_MIC_HDR2       AEU_REG_ADDR(0x0860)
#define AEU_ENC_KEY0       IKEY_REG + 0x00
#define AEU_ENC_KEY1       IKEY_REG + 0x04
#define AEU_ENC_KEY2       IKEY_REG + 0x08
#define AEU_ENC_KEY3       IKEY_REG + 0x0C

/*Bits of STS_REG*/                               
#define AEU_DATA_ISR        (1<<9)
#define AEU_KEY_ISR         (1<<8)
#define AEU_ENC_UNIT_BUSY   (1<<5)
#define AEU_KEY_UNIT_BUSY   (1<<4)
#define AEU_MIC_STATUS      (1<<3)
#define AEU_KEY_DONE        (1<<2)
#define AEU_ENC_START       (1<<1)
#define AEU_KEY_START       (1<<0)

/*Bits of CFG_REG*/
#define AEU_CCM_MODE    (1<<30)
#define AEU_DATA_RSR        (1<<11)
#define AEU_DATA_IMR        (1<<10)
#define AEU_KEY_RSR         (1<<9)
#define AEU_KEY_IMR         (1<<8)
#define AEU_RND_KEY_ACCESS  (1<<5)
#define AEU_SBOX_ACCESS     (1<<4)
#define AEU_DECRYPT         (1<<3)
//#define AEU_KEY_LEN    (1<<9)

/*--------------------------------------------------------------*/
/*  MAC Interrupts registers (MIR)                               */
/*--------------------------------------------------------------*/
#define MIR_INTR_STATUS     MAC_REG_ADDR(0x0510)      /* Interrupt status register */
#define TX_UNDERFLOW          (1<<0)
#define TX_WDOG_TIMEOUT       (1<<1)
#define RX_FRAG_ERR           (1<<2)
#define RX_MEM_ERR            (1<<3)
#define PTR_ERR               (1<<4)
#define RX_OVERFLOW           (1<<5)
#define DATA_BUF_RDY          (1<<7)
#define MGMT_BUF_RDY          (1<<11)
#define TX_DONE               (1<<15)
#define TXWEP_DONE            (1<<17)
#define POWER_CHNG            (1<<19)
#define CFP_END               (1<<23)
#define CFP_START             (1<<24)
#define BCN_LOST              (1<<25)
#define BCN_DATA_RDY          (1<<26)
#define BCN_FREE              (1<<27)
#define TBTT_EARLY            (1<<28)
#define CTRL_MSG_RDY          (1<<31)

#define MIR_INTR_MASK       MAC_REG_ADDR(0x0514)      /* Interrupt mask register */
#define MSK_TX_UNDERFLOW      (1<<0)        
#define MSK_TX_WDOG_TIMEOUT   (1<<1)        
#define MSK_RX_FRAG_ERR       (1<<2)        
#define MSK_RX_MEM_ERR        (1<<3)        
#define MSK_PTR_ERR           (1<<4)        
#define MSK_RX_OVERFLOW       (1<<5)        
#define MSK_DATA_BUF_RDY      (1<<7)        
#define MSK_MGMT_BUF_RDY      (1<<11)        
#define MSK_TX_DONE           (1<<15)        
#define MSK_TXWEP_DONE        (1<<17)        
#define MSK_POWER_CHNG        (1<<19)
#define MSK_POLLED_TX_OPEND   (1<<22)        
#define MSK_CFP_END           (1<<23)        
#define MSK_CFP_START         (1<<24)        
#define MSK_BCN_LOST          (1<<25)        
#define MSK_BCN_DATA_RDY      (1<<26)        
#define MSK_BCN_FREE          (1<<27)        
#define MSK_TBTT_EARLY        (1<<28)
#define MSK_CTRL_MSG_RDY      (1<<31)

#define MSK_ENABLE_ALL_MASK    (MSK_TX_UNDERFLOW   | \
                                MSK_TX_WDOG_TIMEOUT| \
                                MSK_RX_FRAG_ERR    | \
                                MSK_RX_MEM_ERR     | \
                                MSK_PTR_ERR        | \
                                MSK_RX_OVERFLOW    | \
                                MSK_DATA_BUF_RDY   | \
                                MSK_MGMT_BUF_RDY   | \
                                MSK_TX_DONE        | \
                                MSK_TXWEP_DONE     | \
                                MSK_POWER_CHNG     | \
                                MSK_POLLED_TX_OPEND| \
                                MSK_CFP_END        | \
                                MSK_CFP_START      | \
                                MSK_BCN_LOST       | \
                                MSK_BCN_DATA_RDY   | \
                                MSK_BCN_FREE       | \
                                MSK_TBTT_EARLY     | \
                                MSK_CTRL_MSG_RDY)
                                   
/* Default mask enable all, disable MSK_CFP_START, MSK_BCN_LOST, MSK_BCN_DATA_RDY,
   MSK_BCN_FREE, and MSK_TBTT_EARLY */  
#define MSK_DEFAULT_MASK       (MSK_ENABLE_ALL_MASK & \
                                ~(MSK_CFP_START       | \
                                  MSK_BCN_LOST        | \
                                  MSK_BCN_DATA_RDY    | \
                                  MSK_BCN_FREE        | \
                                  MSK_TBTT_EARLY))


/* If polling wireless mac device enable default interuppts and 
   disable MSK_DATA_BUF_RDY and MSK_TX_DONE */
#define MSK_POLLING_INTR_MASK       MSK_DEFAULT_MASK & ~(MSK_TX_DONE | MSK_DATA_BUF_RDY)

/* If not polling mode and freeing buffers in tx task then enable default
   interrupts including MSK_DATA_BUF_RDY and MSK_TX_DONE             */                                  
#define MSK_FREE_IN_TX_TASK_MASK    MSK_DEFAULT_MASK 

/* If not polling and not freeing buffer in tx task then disable MSK_TX_DONE */
#define MSK_DEFAULT_DISABLE_TXDONE  MSK_DEFAULT_MASK & ~MSK_TX_DONE

/* Define interrupt enable mask for wireless mac device. */
#ifdef POLLING
#define MSK_INTERRUPT_ENABLE    MSK_POLLING_INTR_MASK
#else
#ifdef FREE_IN_TXTASK   
#define MSK_INTERRUPT_ENABLE    MSK_DEFAULT_MASK
#else
#define MSK_INTERRUPT_ENABLE    MSK_DEFAULT_DISABLE_TXDONE
#endif /* FREE_IN_TXTASK */
#endif /* Polling        */
        
#ifdef BRIDGE_POLLING_1
#undef  MSK_INTERRUPT_ENABLE
#define MSK_INTERRUPT_ENABLE   MSK_DEFAULT_MASK & ~MSK_DATA_BUF_RDY
#endif
       
       
/*1 - read clear, 0 - write clear */
#define MIR_INTR_RSR        MAC_REG_ADDR(0x0518)      /* Interrupt mask register */
#define RSR_TX_UNDERFLOW      (1<<0)
#define RSR_TX_WDOG_TIMEOUT   (1<<1)
#define RSR_RX_FRAG_ERR       (1<<2)
#define RSR_RX_MEM_ERR        (1<<3)
#define RSR_PTR_ERR           (1<<4)
#define RSR_RX_OVERFLOW       (1<<5)
#define RSR_DATA_BUF_RDY      (1<<7)
#define RSR_MGMT_BUF_RDY      (1<<11)
#define RSR_TX_DONE           (1<<15)
#define RSR_TXWEP_DONE        (1<<17)
#define RSR_POWER_CHNG        (1<<19)
#define RSR_POLLED_TX_OPEND   (1<<22)
#define RSR_CFP_END           (1<<23)
#define RSR_CFP_START         (1<<24)
#define RSR_BCN_LOST          (1<<25)
#define RSR_BCN_DATA_RDY      (1<<26)
#define RSR_BCN_FREE          (1<<27)
#define RSR_TBTT_EARLY        (1<<28)
#define RSR_CTRL_MSG_RDY      (1<<31)

#define RSR_CLEAR_ON_READ_ALL  (RSR_TX_UNDERFLOW   | \
                             RSR_TX_WDOG_TIMEOUT| \
                             RSR_RX_FRAG_ERR    | \
                             RSR_RX_MEM_ERR     | \
                             RSR_PTR_ERR        | \
                             RSR_RX_OVERFLOW    | \
                             RSR_DATA_BUF_RDY   | \
                             RSR_MGMT_BUF_RDY   | \
                             RSR_TX_DONE        | \
                             RSR_TXWEP_DONE     | \
                             RSR_POWER_CHNG     | \
                             RSR_POLLED_TX_OPEND| \
                             RSR_CFP_END        | \
                             RSR_CFP_START      | \
                             RSR_BCN_LOST       | \
                             RSR_BCN_DATA_RDY   | \
                             RSR_BCN_FREE       | \
                             RSR_TBTT_EARLY     | \
                             RSR_CTRL_MSG_RDY)   


#define MIR_ERR_STAT        MAC_REG_ADDR(0x0610)      /* Indicate cause of error interrupt */
#define MIR_ERR_MASK        MAC_REG_ADDR(0x0614)      /* Mask error interrupts */
#define MIR_PS_ADDR_LO      MAC_REG_ADDR(0x0620)      /* Interrupt mask register */
#define MIR_PS_ADDR_HI      MAC_REG_ADDR(0x0624)      /* Interrupt mask register */

/*--------------------------------------------------------------*/
/*  MAC Test/Diagnostic registers (MTD)                               */
/*--------------------------------------------------------------*/
#define MTD_MCU_RESET      MAC_REG_ADDR(0x0800)
#define   RESET_MCU          (1<<0)
#define   RESET_TX           (1<<1)
#define   RESET_RX           (1<<2)
#define MTD_MUX_SELECT     MAC_REG_ADDR(0x0810)
#define MTD_TX_WATCHDOG    MAC_REG_ADDR(0x0820)
#define   TX_WATCHDOG_ENABLE (1<<16)
#define MTD_BY_PASS        MAC_REG_ADDR(0x0824)
#define   BP_TXBUF           (1<<0)
#define   BP_BACKOFF         (1<<1)
#define   BP_MC_BACKOFF      (1<<2)
#define   BP_TX_FCS          (1<<3)
#define MTD_RTS_FAIL_CNT   MAC_REG_ADDR(0x0830)
#define MTD_RTS_GOOD_CNT   MAC_REG_ADDR(0x0834)
#define MTD_BCNTX_CNT      MAC_REG_ADDR(0x0838) /* Beacon Transmit Counter */
#define MTD_PROBERSPTX_CNT MAC_REG_ADDR(0x083C) /* Probe Response Transmit Counter */
#define RX_FCS_ERR_CNT     MAC_REG_ADDR(0x0840) /* FCS Error Count */
#define RX_UC_FRAME_CNT    MAC_REG_ADDR(0x0844) /* Unicast Frames Received */
#define RX_MC_FRAME_CNT    MAC_REG_ADDR(0x0848) /* Multcast Frames Received */
#define RX_DUP_FRAME_CNT   MAC_REG_ADDR(0x084C) /* Duplicate Frames Received Cunt */
#define RX_TRAFFIC_CNT     MAC_REG_ADDR(0x0850) /* Accumulated Radio Traffic in bytes */
#define RX_TRAFFIC_ERR_CNT MAC_REG_ADDR(0x0854) /* Accumulated Radio FCS error in bytes */
#define RX_ICV_ERR_CNT     MAC_REG_ADDR(0x0858) /* ICV Error Count */
#define RX_UNDECRYP_CNT    MAC_REG_ADDR(0x085C) /* Undecryptable Frames Received */
#define RX_EXCL_FRAME_CNT  MAC_REG_ADDR(0x0860) /* Excluded Frames Received */
#define MTD_PLL_BYPASS     MAC_REG_ADDR(0x0870)
#define MPP_FORCEPLL       MAC_REG_ADDR(0x0874)
#define MTD_TEST_TXPTR     MAC_REG_ADDR(0x0880)
#define MTD_TEST_LEN       MAC_REG_ADDR(0x0884)
#define MTD_TEST_TXRATE    MAC_REG_ADDR(0x0888)
#define MTD_TEST_TX        MAC_REG_ADDR(0x088C) /* Transmit Test Pattern */
#define MTD_TXFSM_DISPL    MAC_REG_ADDR(0x0890) /* Transmit FSM Display */
#define MTD_RXFSM_DISPL    MAC_REG_ADDR(0x08B0) /* Receive FSM Display */
#define MTD_EXTRA0         MAC_REG_ADDR(0x08D0) /* Extra 0 */
#define MTD_EXTRA1         MAC_REG_ADDR(0x08D4) /* Extra 1 */
#define MTD_RTS_DATA_RATE  MAC_REG_ADDR(0x08D4) /* Extra 1 */


/*--------------------------------------------------------------*/
/*  WLMAC Buffer Configuration                                  */
/*--------------------------------------------------------------*/
#define TX_RAM_BASE        WL_SQRAM_BASE
#define tx_PTRQ_NUM        14
#define tx_PTRQ_ENTRIES    64       
#define tx_PTR_ARRAY_SIZE  (tx_PTRQ_NUM * tx_PTRQ_ENTRIES * 4)
#define tx_DONE_SIZE       (tx_PTRQ_ENTRIES * 4)
#define tx_BCN_SIZE        0x100
#define rx_STA_BCN_BUF_LEN     0x200  /*-> from 0x180 - hold 1 802.11e beacon frame */
#ifdef AP_SW
#ifdef WB_EXCEPTION
#define rx_BCN_BUF_SIZE    rx_STA_BCN_BUF_LEN*2  /*-> from 0x300 - holds 2 beacon frames */
#else
#define rx_BCN_BUF_SIZE    rx_STA_BCN_BUF_LEN*8  /*-> from 0x300 - holds 2 beacon frames */
#endif /* WB_EXCEPTION */
#else
#define rx_BCN_BUF_SIZE    0x400  /*-> from 0x300 - holds 2 beacon frames */
#endif
#define rx_STA_MNF_BUF_SIZE    0xc00    /* hold at least 10 mamt frames */
#define rx_STA_DNF_BUF_SIZE    0x3800   /* hold at least 6 data frames */
#define rx_STA_DF_BUF_SIZE     0x980    /* fragment data msg buffer size */
#define rx_HEADER_SIZE         10       /* hardware mac overhead */
#define RX_STA_DF_BUF_NUM      3

#ifdef ECOS
#define RX_AP_DATA_NFRAG_BUFSIZE   (63 * 1024)  /* 40Kbytes) */
#define RX_AP_MGMT_NFRAG_BUFSIZE  (8 * 1024)   /* 1 Kytes */
#define RX_AP_FRAG0_BUFSIZE   (20 * 1024)   /* 3 Kbytes */
#define RX_AP_FRAG1_BUFSIZE   (20 * 1024)   /* 3 Kbytes */
#define RX_AP_FRAG2_BUFSIZE   (20 * 1024)   /* 3 Kbytes */
#else /* VxWorks */
/* Settings for VxWorks, need to put in sqram for now so sizes
   cannot be the same as that for Ecos AP32. */
#define RX_AP_DATA_NFRAG_BUFSIZE   (16 * 1024)  /* 32Kbytes) */
#define RX_AP_MGMT_NFRAG_BUFSIZE  (4 * 1024)   /* 1 Kytes */
#define RX_AP_FRAG0_BUFSIZE   (4 * 1024)   /* 2 Kbytes */
#define RX_AP_FRAG1_BUFSIZE   (4 * 1024)   /* 2 Kbytes */
#define RX_AP_FRAG2_BUFSIZE   (4 * 1024)   /* 2 Kbytes */

#endif /* ECOS_OS */

/*--------------------------------------------------------------*/
/*  MAC PowerSave / PHY Controls registers (MPP)                */
/*--------------------------------------------------------------*/
#define MPP_PSM             MAC_REG_ADDR(0x0650)        /*Power-Save Mode. 0x3 - sleep; 0x0 - active */
#define   TX_RX_SLEEPED       0x3
#define   AUTO_PSM            (1<<2)
#define   PWR_DOWN_RF         (1<<3)
#define MPP_MASTER_PS       MAC_REG_ADDR(0x0654)

/*???#define MPP_FCF_PMBIT       MAC_REG_ADDR(0x0408) */

#define MPP_TXPE_RISE       MAC_REG_ADDR(0x0240) /*condor */
#define MPP_TXPE_FALL       MAC_REG_ADDR(0x0244) /* condor */

#define MPP_BCN_EARLY       MAC_REG_ADDR(0x0638) /*# of usec ahead to assert TX_BCN_BUSY */
/*???#define MPP_SLOT_TIME       MAC_REG_ADDR(0x0420) */

#define MPP_UFACTOR         MAC_REG_ADDR(0x0660)
#define MPP_DCF_SLOT_TIME   MAC_REG_ADDR(0x0670)

#define MPP_SIFS_TIME       MAC_REG_ADDR(0x0674)
#define MPP_PIFS_TIME       MAC_REG_ADDR(0x0678)
#define MPP_EIFS_TIME       MAC_REG_ADDR(0x067C)
#define MPP_TBTT_EARLY      MAC_REG_ADDR(0x0634)
#define MPP_AIFS_TIME0      MAC_REG_ADDR(0x0680)
#define MPP_AIFS_TIME1      MAC_REG_ADDR(0x0684)
#define MPP_AIFS_TIME2      MAC_REG_ADDR(0x0688)
#define MPP_AIFS_TIME3      MAC_REG_ADDR(0x068C)
#define MPP_AIFS_TIME4      MAC_REG_ADDR(0x0690)
#define MPP_AIFS_TIME5      MAC_REG_ADDR(0x0694)
#define MPP_AIFS_TIME6      MAC_REG_ADDR(0x0698)
#define MPP_AIFS_TIME7      MAC_REG_ADDR(0x069C)

#define TX_BUF_NUM            1
#define TX_BUF_SIZE           0x2000
/*#define TX_INFO_CTRL_SIZE     0x2C */
#define TX_INFO_CTRL_SIZE     (sizeof(TxInfoCtrl_t)+4) /*0x2C */
#define NUM_TX_INFO_CTRL      8

#define BUF_HEADER_SIZE       8
#define BUF_FREE              0
#define BUF_BUSY              0x00000001
#define MAX_BUF_UNIT          16

/* Definitions for TxParam */
#define HEADER_TYPE1       0x0001
#define SHORT_PREAMBLE     0x0004
#define ANTENNA1           0x0008
#define LONG_RETRY_LIMIT   0x0010
#define TX_WDS             0x0001
#define SEND_RTS           0x0020
#define MULTI_CAST         0x0040
#define NOT_DIRECTED_QAP   0x0080
#define TX_USE_CCK         0x0100
#define LISTEN_USE_CCK     0x0200
#define LISTEN_USE_OFDM    0x0400

typedef PACK_START struct MacTxInfo_t
{
    UINT8                RetryCnt;
    UINT8                RetryReq;
    UINT16               Status;
    IEEEtypes_GenHdr_t*  HdrAddr;
    UINT16               Service;
	UINT8                Rate;
	UINT8                Power;
	UINT16               Rts_Dur0;	 /*rts for regular fragments */
	UINT16               Rts_Dur1;	 /*rts for last fragment */
	UINT32               TSF0;
	UINT32               TSF1;
	UINT16               FragBasicDur_Id;
	UINT16               RemBasicDur_Id;
	UINT16               TxParam;
    UINT16               QosControl; /* Not used for non-QoS frames */
} PACK_END MacTxInfo_t;

typedef PACK_START struct TxInfoCtrl_t
{
   MacTxInfo_t    TxInfo;
   UINT32         WcbHRef;
   UINT32         TxBufPtr;
   UINT32         stat;
} PACK_END TxInfoCtrl_t;

typedef PACK_START struct SwInfo_t
{
   UINT32 Reserved;
   UINT16 StnId;
   UINT16 MacQId;
}
PACK_END SwInfo_t;

typedef PACK_START struct _txBcnInfo_t
{
  UINT8             RetryCnt;
  UINT8             Reserved0;
  UINT16            Status;
  IEEEtypes_Frame_t *HdrAddr;
  UINT16            Service;
  UINT8             Rate;
  UINT8             Power;
  UINT8             CfOffset;
  UINT8             TimOffset;
  UINT16            ProbeRespLen;
  UINT32            Tsf0;
  UINT32            Tsf1;
  UINT16            FragBasicDurId0;
  UINT16            FragBasicDurId1;
  UINT16            TxParam;
  UINT16            Reserved1;
}
PACK_END txBcnInfo_t;

typedef PACK_START struct _txInfo_t
{
  UINT8             RetryCnt;
  UINT8             Reserved;
  UINT16            Status;
  IEEEtypes_Frame_t *HdrAddr;
  UINT16            Service;
  UINT8             Rate;
  UINT8             Power;
  UINT16            RtsDurId0;
  UINT16            RtsDurId1;
  UINT32            Tsf0;
  UINT32            Tsf1;
  UINT16            FragBasicDurId0;
  UINT16            FragBasicDurId1;
  UINT16            TxParam;
  UINT16            QosControl;      /* Not used for non-QoS frames */
  UINT32            BufId;
  UINT32            Rsrv1;          /* this is added to make the end of this buffer aligned
									 * at 8 byte boundary. This enables the buffer allocated for
									 * ether rx to have this much as the reserved size and also alighed 
									 * at 8 byte boundary which is a requirement for ether rx buffer */
  SwInfo_t          SwInfo;         /* SwInfo is placed as the last element, so that it can beused in
									 * tx_pktbuf_t as orig txinfo fileds + RxSign_t + 11b header + body
                                     * In this field, the station id and qid are stored */
}
PACK_END txInfo_t;

#define TC0_QID     0
#define TC1_QID     1
#define TC2_QID     2
#define TC3_QID     3
#define TC4_QID     4
#define TC5_QID     5
#define TC6_QID     6
#define TC7_QID     7
#define CM_QID      8    /* Control/management Queue */
#define BW_QID      9    /* Bandwidth Allocation/Request Queue */
#define PS_QID      10
#define CFP_QID     11
#define MCBC_QID    12   /* Multicast/Broadcast queue */
#define DONE_QID    13
#define MAX_QID     14


/* Bit settings for RX_BLOCK_FRAME register. */
#define BLOCK_ASSOC_REQ             0x00000001  /* Association Request */
#define BLOCK_ASSOC_RSP             0x00000002  /* Association Reponse */
#define BLOCK_REASSOC_REQ           0x00000004  /* Reassociation Request */
#define BLOCK_REASSOC_RSP           0x00000008  /* Reassociation Response */
#define BLOCK_PROBE_REQ             0x00000010  /* Probe Request */
#define BLOCK_PROBE_RSP             0x00000020  /* Probe Reponse */
#define BLOCK_BEACON                0x00000040  /* Beacon */
#define BLOCK_ATIM                  0x00000080  /* ATIM */
#define BLOCK_DISASSOC              0x00000100  /* Disassociation */
#define BLOCK_AUTH                  0x00000200  /* Authentication */
#define BLOCK_DEAUTH                0x00000400  /* Deauthentication */
#define BLOCK_PS_POLL               0x00000800  /* PS-POLL */
#define BLOCK_RTS                   0x00001000  /* RTS */
#define BLOCK_CTS                   0x00002000  /* CTS */
#define BLOCK_ACK                   0x00004000  /* ACK */
#define BLOCK_CF_END                0x00008000  /* CF-End */
#define BLOCK_CF_END_CF_ACK         0x00010000  /* CF-End+CF-Ack */
#define BLOCK_DATA                  0x00020000  /* Data */
#define BLOCK_DATA_CF_ACK           0x00040000  /* Data+CF-Ack */
#define BLOCK_DATA_CF_POLL          0x00080000  /* Data+CF-Poll */
#define BLOCK_DATA_CF_ACK_CF_POLL   0x00100000  /* Data+CF-Ack+CF-Poll */
#define BLOCK_NULL                  0x00200000  /* Null function */
#define BLOCK_CF_ACK                0x00400000  /* CF-Ack */
#define BLOCK_CF_POLL               0x00800000  /* CF-Poll */
#define BLOCK_CF_ACK_CF_POLL        0x01000000  /* CF-Ack+CF-Poll */


#endif /* _WLMAC_H_ */
